#!/bin/ksh
# @(#) CS/delivery/bin/metrica_logserviceadmin.ksh CS_r4.79.0.3:cset.000896:2:2 01/19/05 @(#)
###############################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                     ADC Metrica
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Metrica, 2001.
#
###############################################################################
#
# Administration shell script for logging and unloading log messages from DB using the Log Service
#
# Exit Codes:
#     0 = Successful
#     1 = Incorrect syntax
#     2 = Environment variable not set
#     3 = Directory referred to by environment variable does not exist
#     4 = JAR file was not found
#
# Created    30 May 2002
# Author     MadhuSudhan
# Version    CS_r4.79.0.3
#
###############################################################################

#
# Exit codes
#
exit_rc_success=0
exit_rc_syntax=1
exit_rc_bad_env=2
exit_rc_bad_dir=3
exit_rc_bad_jar=4
###############################################################################
#
# Function    : print_component_version
#
# Description : Prints the version of the component
#
# Parameters  : none
#
###############################################################################
print_component_version() {

  . $METRICA_DIR/CS/bin/metrica_version.ksh
  print_version "Log Service Administration" "CS_r4.79.0.3:cset.000896:2:2"

}

###############################################################################
#
# Function    : start_service
#
# Description : Start the Property Service
#
# Parameters  : none
#
###############################################################################
start_service() {
    java \
    -DSVCnameroot=namingservice \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    com.adc.metrica.cs.logservice.cmdlinetool.LogServiceCommandLineToolApp $*
}

###############################################################################
#
# Function    : usage
#
# Description : Shows program syntax and arguments
#
# Parameters  : none
#
###############################################################################
usage() {
    PROGNAME=`basename $0`

    echo
    echo "Usage: "$PROGNAME {
    echo "            -h | -help | --help |"
    echo "            -v | -version | --version"
    echo "       } "
    echo " 	 | "
    echo " 	 {"
    echo "            -log <log message> context <context>|" 
    echo "            -unload <" 
    echo "              startdate <startdate> starttime <starttime> enddate <enddate> endtime <endtime> ] |" 
    echo "              starttime <starttime> endtime <endtime> ]"
    echo "            >" 
    echo "        }"
    echo
    echo
    echo "Options:"
    echo
    echo "      -v | -version | --version        	Display version information"
    echo "      -h | -help | --help              	Display command help"
    echo "      -v | -verbose | --verbose        	Display verbose information "
    echo "      -log <log message> context <context>   	Log the message into the appropriate context"
    echo "       NOTE: If the context is not provided, default context is assumed "

    echo "      -unload startime <starttime> endtime <endtime> " 

    echo "       Unload the Log Messages within the time period"

    echo "      -unload startdate <stardate> starttime <starttime> enddate <enddate> endtime <endtime>"


    echo "	 Unload the Log Messages within the time period "

    echo

}

###############################################################################
#
# Function    : check_cmdline
#
# Description : Validate command line arguments
#
# Parameters  : none
#
###############################################################################
check_cmdline() {
    # Make sure only one argument is passed in
    if [ $# -ne 1 ]
    then
        usage
        exit $exit_rc_syntax
    fi
}

###############################################################################
#
# Function    : addJar
#
# Description : Add a Jar to the runtime CLASSPATH. A check is made to ensure
#               the Jar file exists.
#
# Parameters  : none
#
###############################################################################
addJar() {

    # First ensure the JAR file exists
    if [ ! -f "$1" ]
    then
        bad_jar $1
    fi

    # Add it to the classpath
    CLASSPATH=$CLASSPATH:$1
}

###############################################################################
#
# Function    : setup_runtime
#
# Description : Set the CLASSPATH needed at runtime
#
# Parameters  : none
#
###############################################################################
setup_runtime() {

    # Initialise the classpath	
    CLASSPATH=''

    # Command Line Tool Jar
    addJar $METRICA_DIR/CS/lib/logservice_cmdlinetool.jar

    # SAX Parser
    addJar $METRICA_DIR/ext/lib/xerces.jar

    # Property Service jars
    addJar $METRICA_DIR/CS/lib/propertyservice_cache.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_client.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_common.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_persistence.jar

    # Log Service jars
    addJar $METRICA_DIR/CS/lib/logservice.jar

    # Exceptions
    addJar $METRICA_DIR/CS/lib/exceptionutils.jar

    # Appserver jars
    addJar $METRICA_DIR/ext/lib/vbjorb.jar
    addJar $METRICA_DIR/ext/lib/asrt.jar
    addJar $METRICA_DIR/ext/lib/vbsec.jar
    addJar $METRICA_DIR/ext/lib/lm.jar
    addJar $METRICA_DIR/ext/lib/xmlrt.jar

    # Informix Jars
    addJar $METRICA_DIR/ext/lib/ifxjdbc.jar
    addJar $METRICA_DIR/ext/lib/ifxjdbcx.jar

    #
    # Database services
    #
    addJar $METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_logging.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_util.jar
    addJar $METRICA_DIR/DBElements/lib/metricaudt.jar


    export CLASSPATH
}

###############################################################################
#
# Function    : bad_env
#
# Description : Environment variable is not set
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_env() {
    echo
    echo "Environment variable \$$1 is not set" >&2
    echo
    exit $exit_rc_bad_env
}

###############################################################################
#
# Function    : bad_dir
#
# Description : Directory pointed to by environment variable does not exist
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_dir() {
    echo
    echo "Directory referred to by environment variable \$$1 does not exist" >&2
    echo
    exit $exit_rc_bad_dir
}

###############################################################################
#
# Function    : bad_jar
#
# Description : Jar file does not exist
#
# Parameters  : Full path/name of JAR file
#
###############################################################################
bad_jar() {
    echo
    echo "JAR file not found:" >&2
    echo $1 >&2
    exit $exit_rc_bad_jar
}

###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variable is set and optionally that it
#               refers to a directory that exists.
#
# Parameters  : "var" = check variable, "var_and_dir" = also check dir exists
#               Environment variable name (without "$" sign)
#               Value of the environment variable
#
###############################################################################
check_env() {
    if [ -z "$3" ] ; then
        bad_env "$2"
    fi

    if [ "$1" = "var_and_dir" -a ! -d "$3" ] ; then
        bad_dir "$2"
    fi
}

###############################################################################
#
# Function    : init
#
# Description : Perform the necessary checks to make sure the environment is
#               set up ready to run administration functions.
#
# Parameters  : none
#
###############################################################################
init() {
    check_env "var"         "OSAGENT_PORT" $OSAGENT_PORT
    check_env "var_and_dir" "METRICA_DIR" $METRICA_DIR

    setup_runtime
}

###############################################################################
###############################################################################
#
# Main program
#
###############################################################################
###############################################################################
#check_cmdline $*
#
case "$1" in
    "-h"|"-help"|"--help" )
        print_component_version 
        usage
        exit $exit_rc_success
        ;;
    "-log" )
        init
        start_service $*
        ;;
    "-unload" )
        init
        start_service $*
        ;;
    "-v"|"-version" )
        print_component_version 
        exit $exit_rc_success
        ;;
    * )
        print_component_version 
        usage
        exit $exit_rc_syntax
        ;;
esac

exit $exit_rc_success
