#!/bin/ksh
# @(#) CS/delivery/bin/metrica_notificationserviceadmin.ksh CS_r4.79.0.3:aset.000575:1:1 02/17/04 @(#)
###############################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                     ADC Metrica
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Metrica, 2001.
#
###############################################################################
#
# Script to start, stop and assess the status of the Notification Service
#
# Created    7 June 2002
# Author     Patrick Wood / John Peckham
# Version    CS_r4.79.0.3
#
###############################################################################
#
# Exit codes
#
exit_rc_success=0
exit_rc_syntax=1
exit_rc_bad_env=2
exit_rc_bad_dir=3

#
# Constants
#
PROGNAME=`basename $0`
VERNO=1.0
NS_NAME=NotificationService
NS_ON_OFF_FLAG=0
NS_STATUS_FLAG=2
ECHO_NS_STATUS=

###############################################################################
#
# Function    : print_component_version
#
# Description : Prints the version of the component
#
# Parameters  : none
#
###############################################################################
print_component_version() {

  . $METRICA_DIR/CS/bin/metrica_version.ksh
  print_version "Notification Service Administration" "CS_r4.79.0.3:aset.000575:1:1"

}

###############################################################################
#
# Function    : cleanup
#
# Description : Remove Notification Service IORs
#
# Parameters  : none
#
###############################################################################
cleanup() {
    rm -rf $NOTIFICATION_DIR/domains/OpenFusion/localhost/NotificationService/NotificationSingleton/*.ior
    rm -rf $NOTIFICATION_DIR/domains/OpenFusion/localhost/NotificationService/ProcessSingleton/*.ior
    rm -rf $NOTIFICATION_DIR/domains/OpenFusion/localhost/NotificationService/TypedNotificationSingleton/*.ior
}

###############################################################################
#
# Function    : start_server
#
# Description : Start Notification Service
#
# Parameters  : none
#
###############################################################################
start_server() {
    status
    if [[ $NS_STATUS_FLAG = "0" ]] ; then
        echo $NS_NAME" Already Running"
        return
    elif [[ $NS_STATUS_FLAG = "2" ]] ; then
        echo "Starting $NS_NAME ..."

        # Clean up IORs before restarting
        cleanup

        START_OUTPUT=`server -start $NS_NAME`
    fi

    while status; [[ $NS_STATUS_FLAG != "0" ]]
    do
        sleep 10
    done

    echo "$NS_NAME started"
}

###############################################################################
#
# Function    : stop_server
#
# Description : Stop Notification Service
#
# Parameters  : none
#
###############################################################################
stop_server() {
    status
    if [[ $NS_STATUS_FLAG != "2" ]]; then
        echo "Stopping $NS_NAME ..."
        STOP_OUTPUT=`server -stop $NS_NAME`
    fi

    while status; [[ $NS_STATUS_FLAG != "2" ]]
    do
        sleep 10
    done

    echo "$NS_NAME stopped"
}

###############################################################################
#
# Function    : status
#
# Description : Get status of Notification Service
#
# Parameters  : none
#
###############################################################################
status()
{
    NS_STATUS_STRING=`server -status $NS_NAME`

    if [[ ${NS_STATUS_STRING% Stopped} != $NS_STATUS_STRING ]]; then
        # NotificationService is stopped
        NS_STATUS_STRING=$NS_NAME" Stopped"
        NS_ON_OFF_FLAG=0
        NS_STATUS_FLAG=2
    elif [[ ${NS_STATUS_STRING% Starting} != $NS_STATUS_STRING ]]; then
        # NotificationService is starting
        NS_STATUS_STRING=$NS_NAME" Starting"
        NS_ON_OFF_FLAG=0
        NS_STATUS_FLAG=1
    elif [[ ${NS_STATUS_STRING% Running} != $NS_STATUS_STRING ]]; then
        # NotificationService is running
        NS_STATUS_STRING=$NS_NAME" Running"
        NS_ON_OFF_FLAG=1
        NS_STATUS_FLAG=0
    fi

    if [ ! -z "$ECHO_NS_STATUS" ]; then
        echo $NS_STATUS_STRING
    fi

    return $NS_STATUS_FLAG
}

###############################################################################
#
# Function    : usage
#
# Description : Shows program syntax and arguments
#
# Parameters  : none
#
###############################################################################
usage() {
    PROGNAME=`basename $0`

    echo
    echo "Usage: "$PROGNAME {
    echo "            -h | -help | --help |"
    echo "            -v | -version | --version"
    echo "        } | {"
    echo "            -start | -restart | -stop | -status"
    echo "        }"
    echo
    echo
    echo "Options:"
    echo
    echo "  -v | -version | --version     Display version information"
    echo "  -h | -help | --help           Display command help"
    echo "  -start                        Start Notification Service"
    echo "  -restart                      Restart the Notification Service"
    echo "  -stop                         Stop the Notification Service"
    echo "  -status                       Display status of the Notification Service"
    echo

}

###############################################################################
#
# Function    : check_cmdline
#
# Description : Validate command line arguments
#
# Parameters  : none
#
###############################################################################
check_cmdline() {
    # Make sure an argument is passed in
    if [[ $# -eq 0 ]] ; then
        usage
        exit $exit_rc_syntax
    fi
}

###############################################################################
#
# Function    : bad_env
#
# Description : Environment variable is not set
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_env() {
    echo
    echo "Environment variable \$$1 is not set" >&2
    echo
    exit $exit_rc_bad_env
}

###############################################################################
#
# Function    : bad_dir
#
# Description : Directory pointed to by environment variable does not exist
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_dir() {
    echo
    echo "Directory referred to by environment variable \$$1 does not exist" >&2
    echo
    exit $exit_rc_bad_dir
}

###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variable is set and optionally that it
#               refers to a directory that exists.
#
# Parameters  : "var" = check variable, "var_and_dir" = also check dir exists
#               Environment variable name (without "$" sign)
#               Value of the environment variable
#
###############################################################################
check_env() {
    if [ -z "$3" ] ; then
        bad_env "$2"
    fi

    if [ "$1" = "var_and_dir" -a ! -d "$3" ] ; then
        bad_dir "$2"
    fi
}

###############################################################################
#
# Function    : init
#
# Description : Perform the necessary checks to make sure the environment is
#               set up ready to run administration functions.
#
# Parameters  : none
#
###############################################################################
init() {
    check_env "var_and_dir" "NOTIFICATION_DIR" $NOTIFICATION_DIR

    # Make sure appserver binaries are on path
    export PATH=$PATH:$NOTIFICATION_DIR/bin
}

###############################################################################
###############################################################################
#
# Main program
#
###############################################################################
###############################################################################
check_cmdline $*

case "$1" in
    "-h"|"-help"|"--help" )
        print_component_version
        usage
        exit $exit_rc_success
        ;;

    "-v"|"-version"|"--version" )
        print_component_version
        exit $exit_rc_success
        ;;

    "-start" )
        print_component_version
        init
        start_server
        exit $exit_rc_success
        ;;

    "-stop" )
        init
        stop_server
        exit $exit_rc_success
        ;;

    "-restart" )
        init
        stop_server
        start_server
        exit $exit_rc_success
        ;;

    "-status" )
        ECHO_NS_STATUS=true
        init
        status
        exit $exit_rc_success
        ;;

    * )
        print_component_version
        usage
        exit $exit_rc_syntax
        ;;
esac

exit $exit_rc_success
