#!/usr/bin/perl
#@(#) DBservices/delivery/unix/bin/metrica_offliner.pm DBservices_r4.79.0.3:cset.041555:29:29 05/02/06 @(#)

######################################################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                 Vallent Software Systems UK 
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright @ 2005 Vallent Software Systems UK. All rights reserved. 
#
# Title:            metrica_offliner.pm
# Author:           Shane O'Rourke
#
# Description:      Archiving perl script to put data from the performance
#                   tables offline with the option of compresing the data.
#                   The data can be be delete from the table and restored
#                   when required.
#
#
# Usage:            metrica_offliner.pm
#		              -v | -version | --version |
#		              -h | -help | --help 
#		              [-noheader]        
#		    } | {   
#		              -extract 
#		              [-startdate <YYYYMMDD>] 
#		              [-rundate <YYYYMMDD>] 
#		              [-nodelete] 
#		              [-noheader]         
#		    } | {       
#		              -restore 
#		              [-agedate <YYYYMMDD>] 
#		              [-noheader]          
#		    } | {
#		              -tidy {raw | daily | weekly | monthly} | performancedata 
#		                    [-startdate <YYYYMMDD>] [-enddate <YYYYMMDD>] |
#		              -tidy { all | nc }
#		              -object <object-type>
#		              { -elements <element-list> | -file <filename> }
#		              [-nodelete]         
#		              [-verbose]         
#			      [-noheader]            			      		         
#		    } 
#
#
######################################################################################################

 
=head1 Class metrica_offliner 

 DESCRIPTION:
   Main offliner class.

=cut

#---------------------------------------------------------------------------
#       PACKAGE NAME
#---------------------------------------------------------------------------
package metrica_offliner;
 
#---------------------------------------------------------------------------
#       CORE PERL LIBRARIES
#---------------------------------------------------------------------------
use strict;                 # require declarations
use Getopt::Long;           # for option processing
use Time::Local;            # for time processing
use Time::localtime;        # override the core localtime()
use File::Basename;         # parse the file minus extensions
use Text::ParseWords;       # parse lines into arrays
use Cwd;                    # get pathname of current directory


#---------------------------------------------------------------------------
#       PROGRAM LIBRARIES
#---------------------------------------------------------------------------


#---------------------------------------------------------------------------
#       GLOBAL VARIABLES
#---------------------------------------------------------------------------

my %param  = ();                      # Commandline options

# Archiving command line variables
my ($opt_extract, $opt_startdate, $opt_rundate, $opt_nodelete, $opt_restore, $opt_agedate);
my ($opt_stop, $opt_status);

# Tidy command line variables
my (@opt_tidy, $opt_enddate, $opt_object, @opt_elements, $opt_file, $opt_verbose);

# Information command line variables
my ($opt_help, $opt_h, $opt_version, $opt_v, $opt_noheader);

# Environment variables
my ($database, $admin_database, $sys_database, $extract_dir, $logfile, $metrica_dir, $restore_dir);
my ($compression, $uncompression);
# Command files
my $sqlfile;
my $tables;

# Results Files
my $results_file;

#File used to stop the offline
my $stop_file; 

#historical periods
my $summary_ids;

# Tidy Variables
my $tabletype; 
my $element; 
my $element;
my @elementIDs;
my $objectid;
my $tidyNCAction = 0;
my $tidyRawAction = 0;
my $tidyDailyAction = 0;
my $tidyWeeklyAction = 0;
my $tidyMonthlyAction = 0;
my $tidyPerformancedataAction = 0;  
my $tidyAllAction = 0;  
my $elementsSelection = 0;
my $fileSelection = 0;

my $delete_msg;
my $update_msg;

# Disable nc updates
my $disable_update_functionality = 0;


#---------------------------------------------------------------------------
#       CONSTANTS
#---------------------------------------------------------------------------

my $COMPONENT_NAME = "\"Metrica Offliner\"";
# Static seconds per day
my $DAY_IN_SECONDS = 86400;
# Informix access command
my $DATABASE_CMD =  "dbaccess";
# Return Codes
my $EXIT_RC_SUCCESS="0";
my $EXIT_RC_SYNTAX= "1";
my $EXIT_RC_BAD_ENV="2";
my $EXIT_RC_BAD_DIR="3";
my $EXIT_RC_FAILURE="4";
# histrical summary period constants
# identifier of tables of summary tables
my $TYPE_SUMMARY = "summary";
my $SUMM_CONFIG_TABLE = "summariser_config";
my $SUMM_PERIODS_TABLE = "summariser_periods";
my $TEMP_SUFFIX = "_offliner_extract_temp";
my $CAT_CMD = "cat";
my $APPEND_CMD = ">>";
my $TRUE  = "1";
my $FALSE = "0";


# Getopt settings
$Getopt::Long::autoabbrev=0; # turn off auto-abbreviation
$Getopt::Long::runthrough=1; # exit when errors occur
$Getopt::Long::ignorecase=0; # turn on case sensitivity

# Check the environment vars
check_env();

# Process switches from the command line
my $result=0;
$result =
GetOptions(
 "extract!"     => \$opt_extract,
 "rundate=i"    => \$opt_rundate,
 "startdate=i"  => \$opt_startdate,
 "nodelete"     => \$opt_nodelete,
 "restore!"     => \$opt_restore,
 "agedate=i"    => \$opt_agedate,
 "tidy=s"       => \@opt_tidy,
 "enddate=i"    => \$opt_enddate,
 "object=s"     => \$opt_object, 
 "elements=s"   => \@opt_elements, 
 "file=s"       => \$opt_file, 
 "stop!"        => \$opt_stop,
 "status!"      => \$opt_status,
 "verbose!"     => \$opt_verbose,
 "version!"     => \$opt_version,
 "v!"           => \$opt_v,
 "help!"        => \$opt_help,
 "h!"           => \$opt_h,
 "noheader!"    => \$opt_noheader,
 "<>"           => \&process_unknown_arguments
  );

# Check for correct usage
if ($result != 1) {
    usage();
    exit "$EXIT_RC_SYNTAX";
}

# Uncomment call for debug
# logvars();


# Process
if (!defined @opt_tidy) {
  process();
} else {  
  process_tidy();
}


###############################################################################
#
# Subroutine  : process_unknown_arguments
#
# Description : Exits gracefully when unknown options are given.
#
# Parameters  : None
#
###############################################################################
sub process_unknown_arguments {
    # print "Unknown arguments\n";
    usage();
    exit "$EXIT_RC_SYNTAX";
}


###############################################################################
#
# Subroutine  : check_minimum_env
#
# Description : Check the environment vars and sets dependent vars.
#
# Parameters  : None
#
###############################################################################
sub check_minimum_env {

  # Verify Environment variables
  check_dir('METRICA_DIR');

  # Store Environment variables
  $metrica_dir = $ENV{'METRICA_DIR'};

}

###############################################################################
#
# Subroutine  : check_minimum_action_env
#
# Description : Check the environment vars and sets dependent vars.
#
# Parameters  : None
#
###############################################################################
sub check_minimum_action_env {

  check_minimum_env();

  # Verify Environment variables
  check_file('METRICA_DBSERV_OFFLINER_LOG');
  check_dbase('METRICA_DBSERV_OFFLINER_DATABASE'); 
  check_dbase('METRICA_ADMINISTRATION_DB'); 
  check_dir('METRICA_DBSERV_OFFLINER_EXTRACT_DIR');

  # Store Environment variables
  $logfile =  $ENV{'METRICA_DBSERV_OFFLINER_LOG'};
  $database = $ENV{'METRICA_DBSERV_OFFLINER_DATABASE'};
  $admin_database = $ENV{'METRICA_ADMINISTRATION_DB'};      
  $extract_dir = $ENV{'METRICA_DBSERV_OFFLINER_EXTRACT_DIR'}; 
  
  $stop_file="$metrica_dir/DBServices/bin/stop_offliner";
  unlink $stop_file; # remove file if it exists
  
  # Set dependent variables
  $sqlfile = "$extract_dir/command.sql";
  $results_file = "$extract_dir/results.tmp";

}


###############################################################################
#
# Subroutine  : check_env
#
# Description : Check the environment vars and sets dependent vars.
#
# Parameters  : None
#
###############################################################################
sub check_env {

  check_minimum_action_env();

  # Verify Environment variables
  check_dbase('METRICA_SYSTEM_DB');
  check_dir('METRICA_DBSERV_OFFLINER_RESTORE_DIR');  
  check_compression('METRICA_DBSERV_OFFLINER_COMPRESSION',
                    'METRICA_DBSERV_OFFLINER_UNCOMPRESSION');

  # Store Environment variables
  $sys_database = $ENV{'METRICA_SYSTEM_DB'};  
  $restore_dir = $ENV{'METRICA_DBSERV_OFFLINER_RESTORE_DIR'};
  $compression = $ENV{'METRICA_DBSERV_OFFLINER_COMPRESSION'}
    if defined $ENV{'METRICA_DBSERV_OFFLINER_COMPRESSION'};
  $uncompression = $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'}
    if defined $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'};

  # Set dependent variables
  $sqlfile = "$extract_dir/command.sql";
  $tables="$extract_dir/offline_tables1.ixd";
  #historical period variables
  $summary_ids="$extract_dir/summary_ids.ixd";

}


###############################################################################
#
# Subroutine  : process
#
# Description : Switches according to command line args.
#
# Parameters  : None
#
###############################################################################
sub process {

  # Set a value for nodelete option
  if (defined $opt_nodelete){
     $opt_nodelete = 1; #on (no deletion)
  } 
  else {
     $opt_nodelete = 0; #off (deletion)
  }
  
  # Allow only certain switch combinations otherwise exit
  if ((($opt_extract  eq "")   && ($opt_rundate   ne ""))  ||  #  rundate without extract
      (($opt_nodelete eq "1")  && ($opt_extract   eq ""))  ||  #  nodelete without extract
      (($opt_extract  eq "")   && ($opt_startdate ne ""))  ||  #  startdate without extract
      (($opt_restore  eq "")   && ($opt_agedate   ne ""))  ||  #  agedate without restore
      (($opt_extract  ne "")   && ($opt_restore   ne ""))) {   #  extract and restore
         usage();
         exit "$EXIT_RC_SYNTAX";
  }

  # Switch according to command line args
  if (defined $opt_extract) {
    check_env();
    make_tmp_files();
    print_header();
    get_offline();
    extract();
    clean_tmp_files();
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_restore) {
    check_env();
    make_tmp_files();
    print_header();
    get_offline();
    restore();
    clean_tmp_files();
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_stop) {
	  stop();
  } 
  elsif (defined $opt_status) {
	  status();
  }
  elsif (defined $opt_version) {
    show_version();
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_v) {
    show_version();
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_help) {
    usage();
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_h) {
    usage();
    exit $EXIT_RC_SUCCESS;
  }
  else {
    usage();
    exit $EXIT_RC_SYNTAX;
  }
}

###############################################################################
#
# Subroutine  : usage
#
# Description : Prints usage
#
# Parameters  : None
#
###############################################################################
sub usage {

  # Show script name without directory
  my $file_exe = basename($0);

  print_header();

# Quit with usage
print <<"EOUSAGE";
Usage: $file_exe {
          -v | -version | --version |
          -h | -help | --help 
          [-noheader]
} | {   
          -extract 
          [-startdate <YYYYMMDD>] 
          [-rundate <YYYYMMDD>] 
          [-nodelete] 
          [-noheader]
} | {                              
          -restore 
          [-agedate <YYYYMMDD>] 
          [-noheader]
} | {
          -tidy {raw | daily | weekly | monthly} | performancedata 
                [-startdate <YYYYMMDD>] [-enddate <YYYYMMDD>] |
          -tidy { all | nc }
          -object <object-type>
          { -elements <element-list> | -file <filename> }
          [-nodelete]         
          [-verbose]         
          [-noheader]                                          
}

EOUSAGE

# Check for multiple option selections
multiple_option_selection_message();

}


###############################################################################
#
# Subroutine  : multiple_option_selection_message
#
# Description : Prints warning in case of misuse of multiple options
#
# Parameters  : None
#
###############################################################################
sub multiple_option_selection_message {

if (defined @opt_tidy) {

# Quit with tidy warning note
print <<"EOTIDY";

Note : When giving multiple options with the -tidy argument, please
       ensure that the options are comma delimited. For example
       
       -tidy daily,weekly,monthly       
       
EOTIDY
}


if (defined @opt_elements) {
  
# Quit with elements warning note
print <<"EOELEMENTS";

Note : When giving multiple options with the -elements argument, please
       ensure that the options are comma delimited. For example
       
       -elements cell1,cell2,"1-1-1-1"

EOELEMENTS
}
  
}



###############################################################################
#
# Subroutine  : check_file
#
# Description : Check to see if directory variable is set and a writeable file
#
# Parameters  : IN :File evironment variable
#
###############################################################################
sub check_file {
  my $variable=$_[0];

  if (!defined $ENV{$variable}) {
       warn "Metrica Offliner failed due to environment variable \$$variable not set\n";
       exit $EXIT_RC_BAD_ENV;
  }
  elsif ($ENV{$variable} eq "") {
       warn "Metrica Offliner failed due to environment variable \$$variable set incorrectly\n";
       exit $EXIT_RC_BAD_ENV;
  }
}

###############################################################################
#
# Subroutine  : check_config_file
#
# Description : Check to see if optional config file variable is set and a writeable file
#
# Parameters  : IN :File evironment variable
#
###############################################################################
sub check_config_file {
  my $variable=$_[0];
  my $returnVal = 1;
  

  if (!defined $ENV{$variable}) {
       writelog("\| WARNING \| metrica_offliner \| The offliner nc configuration file environment variable \$$variable is unset.");       
       $returnVal = 0;
  }
  elsif ($ENV{$variable} eq "") {
       writelog("\| WARNING \| metrica_offliner \| The offliner nc configuration file environment variable \$$variable is unset.");       
       $returnVal = 0;
  } 
  elsif (!(-r $ENV{$variable} )) { 
       writelog("\| WARNING \| metrica_offliner \| The offliner nc configuration file $ENV{$variable} is not readable.");       
       $returnVal = 0;
  }   
  elsif (-z $ENV{$variable} ) { 
       writelog("\| WARNING \| metrica_offliner \| The offliner nc configuration file $ENV{$variable} is empty.");       
       $returnVal = 0;
  }   
  
  return $returnVal;
}

###############################################################################
#
# Subroutine  : check_dir
#
# Description : Check to see if directory variable is set and a directory
#
# Parameters  : IN :Directory evironment variable
#
###############################################################################
sub check_dir {
  my $variable=$_[0];

  if (!defined $ENV{$variable}) {
       print "Metrica Offliner failed due to environment variable \$$variable not set\n";
       exit $EXIT_RC_BAD_DIR;
  }
  elsif ($ENV{$variable} eq "") {
       print "Metrica Offliner failed due to environment variable \$$variable set incorrectly\n";
       exit $EXIT_RC_BAD_DIR;
  }
  elsif (!(-d $ENV{$variable})) {
       print "Metrica Offliner failed due to environment variable \$$variable directory access error\n";
       exit $EXIT_RC_BAD_DIR;
  }
}

###############################################################################
#
# Subroutine  : check_dbase
#
# Description : Check to see if directory variable is set 
#
# Parameters  : IN : Database evironment variable
#
###############################################################################
sub check_dbase {
  my $database=$_[0];

  if (!defined $ENV{$database}) {
       print "Metrica Offliner failed due to environment variable \$$database not set\n";
       exit $EXIT_RC_BAD_ENV;
  }
  elsif ($ENV{$database} eq "") {
       print "Metrica Offliner failed due to environment variable \$$database set incorrectly\n";
       exit $EXIT_RC_BAD_ENV;
  }
}

###############################################################################
#
# Subroutine  : check_compression
#
# Description : Check to see if compression variables are either both set
#               or both unset.
#
# Parameters  : IN : Compression evironment variable
#               IN : Uncompression evironment variable
#
###############################################################################
sub check_compression {
  my $comp=$_[0];
  my $uncomp=$_[1];

  if ((!defined $ENV{$comp})&&(defined $ENV{$uncomp}) ||
      (defined $ENV{$comp})&&(!defined $ENV{$uncomp}))
  {
       print "Metrica Offliner failed: environment variables \$$comp and 
       \$$uncomp must both be declared or both undeclared\n";
       exit $EXIT_RC_BAD_ENV;
  }
  elsif (($ENV{$comp} ne "") && ($ENV{$uncomp} eq "") ||
         ($ENV{$comp} eq "") && ($ENV{$uncomp} ne ""))
  {
       print "Metrica Offliner failed: environment variables \$$comp and 
       \$$uncomp must both be set or both unset\n";
       exit $EXIT_RC_BAD_ENV;
  }
}


###############################################################################
#
# Subroutine  : print_header
#
# Description : prints copyright if required
#
# Parameters  : None
#
###############################################################################
sub print_header {
  if (!defined $opt_noheader) {
  
      my $arg1 = "$metrica_dir/DBServices/bin/metrica_dbserv_copyright.ksh";
      my @arg_list = ( $arg1, $COMPONENT_NAME );

      my $rc = 0xffff & system( "@arg_list");
      if ($rc){
         offliner_exit("Call to $arg1 failed");
      }  
  }
}



###############################################################################
#
# Subroutine  : extract
#
# Description : Extracts from the database. Optional rundate parameter. 
#               If rundate is omitted, runtime defaults to today.
#
# Parameters  : None
#
###############################################################################
sub extract {

  my $start_date;        # start date to be extracted
  my $extract_date;      # date to be extracted
  my $table_name;        # table to be extracted
  my $age_in_seconds;    # age in seconds
  my $format_date_stamp; # date in YYYMMDD format
  my $datestamp;         # date formatted
  my $file;              # extracted file
  my $ext_command;       # extraction command
  my $del_command;       # delete command
  my $marker;            # program control marker

  # If rundate is not defined default to today in utime else convert to utime
  if(($opt_rundate eq "") && ($opt_startdate ne "")) {
    $opt_rundate=time();
    check_valid_date($opt_startdate) or offliner_exit
       ("Metrica Offliner failed due to startdate $opt_startdate in incorrect format", "", $EXIT_RC_SYNTAX);
    $start_date = get_utime($opt_startdate);
    validate_date_order($start_date, $opt_rundate) or offliner_exit
       ("Metrica Offliner failed due to startdate is later or equal to rundate", "", $EXIT_RC_SYNTAX);
    writelog("\| DEBUG \| metrica_offliner \| Command : -extract \| default rundate $opt_rundate \| startdate $start_date\n");    
  }
  elsif(($opt_rundate eq "") && ($opt_startdate eq "")) {
    $opt_rundate=time();
    $start_date = "";
    writelog("\| DEBUG \| metrica_offliner \| Command : -extract \| default rundate $opt_rundate \| no startdate\n");
  }
  elsif(($opt_rundate ne "") && ($opt_startdate ne "")) {
    check_valid_date($opt_startdate) or offliner_exit
       ("Metrica Offliner failed due to startdate $opt_startdate in incorrect format", "", $EXIT_RC_SYNTAX);
    check_valid_date($opt_rundate) or offliner_exit
       ("Metrica Offliner failed due to rundate $opt_rundate in incorrect format", "", $EXIT_RC_SYNTAX);    
    $opt_rundate = get_utime($opt_rundate);
    $start_date = get_utime($opt_startdate);
    validate_date_order($start_date, $opt_rundate) or offliner_exit
       ("Metrica Offliner failed due to startdate is later or equal to rundate", "", $EXIT_RC_SYNTAX);
    writelog("\| DEBUG \| metrica_offliner \| Command : -extract \| -rundate $opt_rundate \| startdate $start_date\n");    
  }
  elsif(($opt_rundate ne "") && ($opt_startdate eq "")) {
    check_valid_date($opt_rundate) or offliner_exit
       ("Metrica Offliner failed due to rundate $opt_rundate in incorrect format", "", $EXIT_RC_SYNTAX);   
    $opt_rundate = get_utime($opt_rundate);
    $start_date = "";
    writelog("\| DEBUG \| metrica_offliner \| Command : -extract \| -rundate $opt_rundate \| no startdate\n");
  }
     
  # Read from the offline table file
  open TABLES, $tables or offliner_exit("Metrica Offliner failed, cannot open database table $tables for read", $!, $EXIT_RC_FAILURE);
  while (<TABLES>) {
    my ($table_name, $age, $date, $extract, $delete, $table_type) = split (/\|/,$_,6);
    
    chomp $table_type;
    chop  $table_type;   # Remove final pipe
    $age_in_seconds = ($DAY_IN_SECONDS * $age); # age in DB * 1 day in seconds
    $extract_date = ($opt_rundate - $age_in_seconds); # no of seconds from today - age

    # Only delete data if extract is successful
    $marker = 1;

    # If data is to be extracted
    if ($extract eq "t") {
      $marker = extract_data($start_date, $extract_date, $table_name, $date);
    }

    # If extracted data is to be deleted from the table
    if (($delete eq "t") && ($opt_nodelete eq "0") && ($marker eq "1")) {
      delete_data($start_date, $extract_date, $table_name, $date);
    }
    elsif (($delete eq "t") && ($opt_nodelete eq "0")  && ($marker eq "0")) {
      offliner_warn("WARNING", "No data deletion from table $table_name", "Previous unsuccessfully extraction");
    }
    
    if (lc $table_type eq lc $TYPE_SUMMARY) { # if summary type table, offline period history data
    
	    # If data is to be extracted
	    if ($extract eq "t") {
	      $marker = extract_historical_data($start_date, $extract_date, $table_name);
	    }

	    # If extracted data is to be deleted from the table
	    if (($delete eq "t") && ($opt_nodelete eq "0") && ($marker eq "1")) {
	      delete_historical_data($start_date, $extract_date, $table_name);
	    }
	    elsif (($delete eq "t") && ($opt_nodelete eq "0")  && ($marker eq "0")) {
	      offliner_warn("WARNING", "No Summary period history data deletion for table $SUMM_PERIODS_TABLE", "Previous unsuccessfully extraction");
	    }    
    }
    
    # Check for stop file and exit if it exists
    if( does_stop_file_exist() ) {
       writelog("\| INFO \| metrica_offliner \| Received stop signal, exiting...\n");       
       last;
    }

  } #while (<TABLES>)
}

###############################################################################
#
# Subroutine  : extract_data
#
# Description : Extracts data from the database.
#
# Parameters  : None
#
###############################################################################
sub extract_data {

  my @arg_list;
  my $start_date   = $_[0];  
  my $extract_date = $_[1];
  my $table_name   = $_[2];
  my $date         = $_[3];
 # my $table_type   = $_[4];  
  my $formatted_extract;
  my $formatted_start;
  my $ext_command;

  my $file_sql;
  my $file_unquoted;
  
  my $marker;            # program control marker
 

  if ($start_date eq ""){
    $formatted_extract=get_format_date($extract_date);
    $file_sql = "\'$extract_dir\/$table_name,$formatted_extract\.ixd\'";
    $file_unquoted = "$extract_dir\/$table_name,$formatted_extract\.ixd";
    $ext_command = "unload to $file_sql select * from $table_name where $date <= $extract_date;";
  } 
  else {
    $formatted_extract=get_format_date($extract_date);
    $formatted_start=get_format_date($start_date); 
    $file_sql = "\'$extract_dir\/$table_name,$formatted_start\_$formatted_extract\.ixd\'";
    $file_unquoted = "$extract_dir\/$table_name,$formatted_start\_$formatted_extract\.ixd";
    $ext_command = "unload to $file_sql select * from $table_name where $date BETWEEN $start_date AND $extract_date;";
  }

  $marker = perform_extract($ext_command,$database,$file_unquoted,$table_name,$FALSE);  
  return $marker;  
}

###############################################################################
#
# Subroutine  : extract_historical_data
#
# Description : Extracts historical data from the database.
#
# Parameters  : None
#
###############################################################################
sub extract_historical_data {

  my @arg_list;
  my $start_date   = $_[0];  
  my $extract_date = $_[1];
  my $table_name   = $_[2];
  #my $table_type   = $_[3];  
  my $formatted_extract;
  my $formatted_start;
  my $ext_command;

  my $file_sql;
  my $file_unquoted;
  
  my $marker;            # program control marker
 
   
    get_summaries($table_name);
    # Read from the summary id's file
    open SUMMARY_IDS, $summary_ids or offliner_exit("Metrica Offliner failed, cannot open file $summary_ids for read", $!, $EXIT_RC_FAILURE);
    while (<SUMMARY_IDS>) {

	my ($summary_id) = split (/\|/,$_,1);
	chomp $summary_id;
	chop  $summary_id;   # Remove final pipe
	if ($start_date eq "") {
    		$formatted_extract=get_format_date($extract_date);	
		$file_unquoted = "$extract_dir\/$SUMM_PERIODS_TABLE,$formatted_extract\.ixd";
		#don't overwrite any existing file
		$file_sql = "\'$extract_dir\/$SUMM_PERIODS_TABLE,$formatted_extract\.ixd$TEMP_SUFFIX\'";
		$ext_command = "unload to $file_sql select * from $SUMM_PERIODS_TABLE where summaryid = \"$summary_id\" and startTime <= $extract_date;";
	} else {
    		$formatted_extract=get_format_date($extract_date);
    		$formatted_start=get_format_date($start_date); 	
		$file_unquoted = "$extract_dir\/$SUMM_PERIODS_TABLE,$formatted_start\_$formatted_extract\.ixd";
		#don't overwrite any existing file
		$file_sql = "\'$extract_dir\/$SUMM_PERIODS_TABLE,$formatted_start\_$formatted_extract\.ixd$TEMP_SUFFIX\'";
		$ext_command = "unload to $file_sql select * from $SUMM_PERIODS_TABLE where summaryid = \"$summary_id\" and startTime BETWEEN $start_date AND $extract_date;";
	}

	$marker = perform_extract($ext_command,$sys_database,$file_unquoted,$SUMM_PERIODS_TABLE,$TRUE); 	
	if ($marker eq "0") { # if extract fails don't delete
	    return $marker;
	}

    } #while (<SUMMARY_IDS>)   
  close SUMMARY_IDS; 
  return $marker;  
}


###############################################################################
#
# Subroutine  : perform_extract
#
# Description : performs the extract.
#
# Parameters  : None
#
###############################################################################
sub perform_extract {

  my @arg_list;
  my $ext_command   		= $_[0];  
  my $database 	    		= $_[1];
  my $file_unquoted 		= $_[2];
  my $table_name    		= $_[3];
  my $extract_already_existed 	= $_[4];

  
  my $rc;


  writesql("$ext_command");
  writelog("\| INFO \| metrica_offliner \| $ext_command\n");

  @arg_list = ( $DATABASE_CMD, $database, $sqlfile);
  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
  if ($rc){
    offliner_warn("ERROR", "Metrica Offliner error on $database extracting from $table_name\n", $!);
    return 0;
  }
  
  if ($extract_already_existed eq $TRUE) {
	append_to_existing_extract($file_unquoted);  
  }
  
  
  # If file is non-empty and not removed
  if ( remove_empty_file($file_unquoted) ) {
     # If extracted data file is to be compressed
     if (defined $ENV{'METRICA_DBSERV_OFFLINER_COMPRESSION'} &&
                 $ENV{'METRICA_DBSERV_OFFLINER_COMPRESSION'} ne "") {
       writelog("\| INFO \| metrica_offliner \| $compression on file $file_unquoted\n");
       @arg_list = ( $compression, $file_unquoted);
       $rc = 0xffff & system("@arg_list >> $logfile 2>&1") ;
       if ($rc){
          offliner_warn("ERROR", "Compression on file $file_unquoted failed", $!);
          return 0;
       }
     }
   }
  return 1;
}


###############################################################################
#
# Subroutine  : check_existing_extract
#
# Description : checks if an existing extract and uncompresses if necessary
# Parameters  : None
#
###############################################################################
sub check_existing_extract {

  my @arg_list;
  my $file_unquoted = $_[0];  
  my @zipped;                      # compressed files for restoration
  my $file;
  my $rc;

  if (defined $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} &&
              $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} ne "") {
     @zipped = glob("$file_unquoted.*") ;
     foreach $file (@zipped) {
     
	#-e "$file_unquoted.*" or return 0;
	writelog("\| INFO \| metrica_offliner \| $uncompression on file $file\n");
	@arg_list = ( $uncompression, $file);
	$rc = 0xffff & system("@arg_list >> $logfile 2>&1") ;
	if ($rc){
		offliner_exit("ERROR", "Uncompression on file $file failed", $!,$EXIT_RC_FAILURE);
	}
     	return $TRUE;	
    }
  } else {  
    @zipped = glob("$file_unquoted");
    foreach $file (@zipped) {
    # -e $file_unquoted or return 0;
     return $TRUE;
    }
  }
  return $FALSE;     
}


###############################################################################
#
# Subroutine  : append_to_existing_extract
#
# Description : appends temporary extract to existing historical periods extract.
#
# Parameters  : None
#
###############################################################################
sub append_to_existing_extract {

	my @arg_list;
	my $file_unquoted = $_[0];  
	my $file_unquoted_temp = "$file_unquoted$TEMP_SUFFIX";
	my $rc;
  	my @zipped;                      # compressed files for restoration
  	my $file;


        # if compression then need to uncompress
	if (defined $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} &&
	      $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} ne "") {
		@zipped = glob("$file_unquoted.*") ;
		foreach $file (@zipped) {

			#-e "$file_unquoted.*" or return 0;
			writelog("\| INFO \| metrica_offliner \| $uncompression on existing file $file in order to append to it\n");
			@arg_list = ( $uncompression, $file);
			$rc = 0xffff & system("@arg_list >> $logfile 2>&1") ;
			if ($rc){
				offliner_exit("ERROR", "Uncompression on file $file failed", $!,$EXIT_RC_FAILURE);
			}
			last; # will only be one	
		}
	}


	writelog("\| INFO \| metrica_offliner \| appending $file_unquoted_temp to $file_unquoted\n");
	@arg_list = ( $CAT_CMD,$file_unquoted_temp,">> $file_unquoted");
	
	$rc = 0xffff & system("@arg_list 2>> $logfile") ;	
	if ($rc){
		offliner_exit("ERROR", "append of file $file_unquoted_temp to $file_unquoted failed", $!,$EXIT_RC_FAILURE);
	}
	unlink $file_unquoted_temp;
}


###############################################################################
#
# Subroutine  : remove_empty_file
#
# Description : Removes empty files from the extract directory
#
# Parameters  : None
#
###############################################################################
sub remove_empty_file {
   my $file = $_[0]; # current file
   
   if (-z $file){
      unlink $file;
      writelog("\| DEBUG \| metrica_offliner \| Removing empty file $file\n");
      return 0; #empty file
   }
   else {
      return 1; #non-empty file
   }
}

###############################################################################
#
# Subroutine  : delete_data
#
# Description : Deletes data from the database. 
#
# Parameters  : None
#
###############################################################################
sub delete_data {

  my @arg_list;
  my $start_date   = $_[0];  
  my $extract_date = $_[1];
  my $table_name   = $_[2];
  my $date         = $_[3];
  my $rc;
  my $del_command;
   
  if ($start_date eq ""){
     $del_command = "delete from $table_name where $date <= $extract_date;";
  } 
  else {
     $del_command = "delete from $table_name where $date BETWEEN $start_date AND $extract_date;";
  }
  
  writesql("$del_command");
  writelog("\| INFO \| metrica_offliner \| $del_command\n");
  @arg_list = ( $DATABASE_CMD, $database, $sqlfile);

  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
  if ($rc){
      offliner_warn("ERROR", "Delete command failed", $!);
  }
}

###############################################################################
#
# Subroutine  : delete_historical_data
#
# Description : Deletes data from the database. 
#
# Parameters  : None
#
###############################################################################
sub delete_historical_data {

  my @arg_list;
  my $start_date   = $_[0];  
  my $extract_date = $_[1];
  my $table_name   = $_[2];
  my $rc;
  my $del_command;
     
  
  get_summaries($table_name);
  # Read from the summary id's file file
  open SUMMARY_IDS, $summary_ids or offliner_exit("Metrica Offliner failed, cannot open file $summary_ids for read", $!, $EXIT_RC_FAILURE);
  while (<SUMMARY_IDS>) {
	  my ($summary_id) = split (/\|/,$_,1);
	  chomp $summary_id;
	  chop  $summary_id;   # Remove final pipe
	  if ($start_date eq ""){
		$del_command = "delete from $SUMM_PERIODS_TABLE where summaryid = \"$summary_id\" and startTime <= $extract_date;";
	  } 
	  else {
		$del_command = "delete from $SUMM_PERIODS_TABLE where summaryid = \"$summary_id\" and startTime BETWEEN $start_date AND $extract_date;";
	  }

	  writesql("$del_command");
	  writelog("\| INFO \| metrica_offliner \| $del_command\n");
	  @arg_list = ( $DATABASE_CMD, $sys_database, $sqlfile);

	  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
	  if ($rc){
	      offliner_warn("ERROR", "Delete command failed", $!);
	  }
  }
  close SUMMARY_IDS;
}

###############################################################################
#
# Subroutine  : restore
#
# Description : Restores archived data to the database. Optional agedate 
#               parameter. If agedate is omitted, all correct
#               files are restored
#
# Parameters  : None
#
###############################################################################
sub restore {

  my @files;                       # files for restoration
  my @zipped;                      # compressed files for restoration
  my $file;
  my ($name, $path, $suffix);

  # If agedate is not defined default to today in utime
  if(($opt_restore ne "") && ($opt_agedate eq "")) {
    $opt_agedate=time();
    writelog("\| DEBUG \| metrica_offliner \| Command : -restore \| default agedate $opt_agedate\n");
  }
  else {
    check_valid_date($opt_agedate) or offliner_exit
        ("Metrica Offliner failed due to agedate $opt_agedate in incorrect format", "", $EXIT_RC_SYNTAX);
    # convert from YYYYMMDD to utime
    $opt_agedate = get_utime($opt_agedate);
    writelog("\| DEBUG \| metrica_offliner \| Command : -restore -agedate \| $opt_agedate\n");
  }
  
  # Check Restore directory for compressed files and uncompress 
  if (defined $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} &&
              $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} ne "") {
     @zipped = glob("$restore_dir/*.ixd.*") or offliner_exit("Metrica Offliner failed: compression is on, but no compressed files exist in $restore_dir", $!, $EXIT_RC_FAILURE);
     foreach $file (@zipped) {
        ($name, $path, $suffix) = fileparse($file, '\.ixd.*');
        validate_restore_file($path, $name, $file, $opt_agedate);
       
	# Check for stop file and exit if it exists
	if( does_stop_file_exist() ) {
           writelog("\| INFO \| metrica_offliner \| Received stop signal, exiting...\n");       
	   last;
        }
     }
  }
  else {  
     @zipped = glob("$restore_dir/*.ixd") or offliner_exit("Metrica Offliner failed: compression is off, but no ixd files exist in $restore_dir", $!, $EXIT_RC_FAILURE);
     foreach $file (@zipped) {
        ($name, $path, $suffix) = fileparse($file, '\.ixd');
        validate_restore_file($path, $name, $file, $opt_agedate);
       
	# Check for stop file and exit if it exists
	if( does_stop_file_exist() ) {
           writelog("\| INFO \| metrica_offliner \| Received stop signal, exiting...\n");       
	   last;
        }
     }
  }
}    

###############################################################################
#
# Subroutine  : validate_restore_file
#
# Description : 
#
# Parameters  : None
#
###############################################################################  
   
sub validate_restore_file {   
 
  my $path         = $_[0]; 
  my $name         = $_[1];  
  my $file         = $_[2]; 
  my $opt_agedate  = $_[3];
  my $table;
  my $run_date;
  my $start_date;

  if ($name =~ /(^\w{1,}),{1}(\d{8})$/){
      $table = $1;
      $run_date = $2;
      if (check_valid_date($run_date)){
        restore_data($file, $path, $name, $table, $run_date, $opt_agedate);
      } 
      else {
        writelog("\| WARNING \| metrica_offliner \| Invalid date format for rundate for file $run_date. Ignoring \| $file\n");
      }
  }
  elsif ($name =~ /(^\w{1,}),{1}(\d{8})_{1}(\d{8})$/) {
      $table = $1;
      $start_date = $2;
      $run_date = $3;
      
      if (check_valid_date($start_date) || check_valid_date($run_date)){
        restore_data($file, $path, $name, $table, $run_date, $opt_agedate);
      } 
      else {
        writelog("\| WARNING \| metrica_offliner \| Invalid date format for startdate $start_date or rundate $run_date. Ignoring \| $file\n");
      }
  } 
  else {
      writelog("\| WARNING \| metrica_offliner \| Filename in incorrect format and shall be ignored. \| $file\n");        
  }
}

###############################################################################
#
# Subroutine  : restore_data
#
# Description : 
#
# Parameters  : None
#
###############################################################################  

sub restore_data {

  my $file        = $_[0]; 
  my $path        = $_[1];  
  my $name        = $_[2];   
  my $table       = $_[3];
  my $run_date    = $_[4]; 
  my $opt_agedate = $_[5];
  my $date_stamp;
  my $uncompressed_file;
  my $restore_command;
  my @arg_list;
  my $rc;
  my $database_for_table;

  # convert from YYYYMMDD to utime
  $date_stamp = get_utime($run_date);

  if ($date_stamp <= $opt_agedate){
  
    # uncompress
    if (defined $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} && 
                $ENV{'METRICA_DBSERV_OFFLINER_UNCOMPRESSION'} ne "") {
       writelog("\| INFO \| metrica_offliner \| $uncompression on file $file\n");
       @arg_list = ( $uncompression, $file);
       $rc = 0xffff & system("@arg_list >> $logfile 2>&1") ;
       if ($rc){
          offliner_warn("ERROR", "Uncompression on file $file failed", $!);
          return 0;
       }
    }

    $uncompressed_file = "$path$name\.ixd";
    $restore_command = "load from $uncompressed_file insert into $table;";
    writesql("$restore_command");
    writelog("\| INFO \| metrica_offliner \| $restore_command\n");
    if (lc $table eq lc $SUMM_PERIODS_TABLE) { # if it is the historical periods table
    	$database_for_table = $sys_database;
    } else {
    	$database_for_table = $database;    
    }
    @arg_list = ( $DATABASE_CMD, $database_for_table, $sqlfile);        
    $rc = 0xffff & system("@arg_list >> $logfile 2>&1");
    if ($rc){
      offliner_warn("WARNING", "Metrica Offliner: unsuccessful restoring of $uncompressed_file to table $table in database $database_for_table.\nPlease see $logfile for details.", $!);
      writelog("\| INFO \| metrica_offliner \| Continuing restoring any remaining files.\n");
    }
  }
}


###############################################################################
#
# Subroutine  : validate_date_order
#
# Description : Ensures that the enddate is after the startdate
#
# Parameters  : startdate, enddate
#
###############################################################################
sub validate_date_order {

  my $startdate   = $_[0];  
  my $enddate     = $_[1];

  # compare dates
  if ($startdate >= $enddate){
     return 0; #error
  }
  else {
     return 1; #no error
  }
}

###############################################################################
#
# Subroutine  : get_utime
#
# Description : Convert from YYYYMMDD to utime
#
# Parameters  : Date in YYYYMMDD format
#             
# Returns     : Date in utime format
#
###############################################################################
sub get_utime {

   my $year;
   my $month;
   my $monthday;
   my $utime;
   my $in_time=$_[0];

   chomp $in_time;
     
   $year = sprintf("%4d", substr($in_time,0,4));
   $month = sprintf("%1d", substr($in_time,4,2));
   $monthday =  sprintf("%1d", substr($in_time,6,2));
   $utime = timelocal(0,0,0, $monthday, ($month-1), $year);
   return $utime;
}

###############################################################################
#
# Subroutine  : get_format_date
#
# Description : Convert from utime to YYYYMMDD
#
# Parameters  : Date in utime format
#             
# Returns     : Date in YYYYMMDD format
#
###############################################################################
sub get_format_date {

  my $unformatted_date = $_[0];
  my $datestamp;
  my $formatted_date_stamp;
   
  $datestamp = localtime($unformatted_date);
  $formatted_date_stamp = sprintf("%04d%02d%02d", ($datestamp->year)+1900,($datestamp->mon)+1,$datestamp->mday);
  return $formatted_date_stamp;
 
}

###############################################################################
#
# Subroutine  : check_valid_date
#
# Description : 
#
# Parameters  : 
#             
# Returns     : 
#
###############################################################################
sub check_valid_date {

  my $date = $_[0];
  
  writelog("\| DEBUG \| metrica_offliner \| Verifying date $date\n");

  if ($date =~ /(^(19|20)\d{2})((0[1-9])|(1[0-2]))((0[1-9])|([1-2]\d)|(3[0-1]))$/) {
     return 1; #correct
  } else { 
     return 0; #error
  }
}

###############################################################################
#
# Subroutine  : get_offline
#
# Description : Extracts the offliner_tables table into a flat file.
#
# Parameters  : None
#
###############################################################################
sub get_offline {
  my @arg_list;                   # command arguments
  my $rc;                         # system command variable
  my $init_command;               # initialising command
  my $logtime = time();           # log time

  $init_command = "unload to $tables select * from offline_tables;";
  writesql("$init_command");

  writelog("\| INFO \| metrica_offliner \| $init_command\n");
  @arg_list = ( $DATABASE_CMD, $database, $sqlfile);
  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
  if ($rc){
    offliner_exit("Metrica Offliner failed with database error : Unable to access table offline_tables in database $database", $!, $EXIT_RC_FAILURE);
  }
}

###############################################################################
#
# Subroutine  : get_summaries
#
# Description : Extracts the summary_ids table into a flat file.
#
# Parameters  : None
#
###############################################################################
sub get_summaries {

  my $summary_table = $_[0];
  my @arg_list;                   # command arguments
  my $rc;                         # system command variable
  my $summ_config_cmd;               # initialising command
  my $logtime = time();           # log time

  $summ_config_cmd = "unload to $summary_ids select summaryid from $SUMM_CONFIG_TABLE where summarytable = \"$summary_table\";";
  writesql("$summ_config_cmd");

  writelog("\| INFO \| metrica_offliner \| $summ_config_cmd\n");
  @arg_list = ( $DATABASE_CMD, $admin_database, $sqlfile);
  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
  if ($rc){
    offliner_exit("Metrica Offliner failed with database error : Unable to access table $SUMM_CONFIG_TABLE in database $admin_database", $!, $EXIT_RC_FAILURE);
  } 
}


###############################################################################
#
# Subroutine  : show_version
#
# Description : Prints version to screen
#
# Parameters  : None
#
###############################################################################
sub show_version {

  my @arg_list;
  my $arg1 = ". $metrica_dir/DBServices/bin/metrica_dbserv_version.ksh\n";
  my $arg2 = "print_version_noheader";
  my $arg3 = "\"DBservices_r4.79.0.3:cset.041555:29:29\"";

  print_header;

  @arg_list = ( $arg1, $arg2, $arg3 );
    
  my $rc = 0xffff & system( "@arg_list");
  if ($rc){
    offliner_exit("Call to $arg1 failed");
  }
}


###############################################################################
#
# Subroutine  : writesql
#
# Description : Writes sql commands to an sql command file
#
# Parameters  : None
#
###############################################################################
sub writesql {
  open SQLFILE, ">$sqlfile" or offliner_exit("Metrica Offliner failed, cannot open $sqlfile for write", $!, $EXIT_RC_FAILURE);
  print SQLFILE "@_";
  close SQLFILE;
}

###############################################################################
#
# Subroutine  : make_tmp_files
#
# Description : Makes tmp command file and tmp offliner_tables file
#
# Parameters  : None
#
###############################################################################
sub make_tmp_files {
  unlink $sqlfile, $tables, $summary_ids, $results_file;
  
  my $permission_msg = "Please ensure that the directory has write permissions.";

  open (SQLFILE, ">$sqlfile") or offliner_exit("Metrica Offliner failed, cannot open $sqlfile for write. $permission_msg", $!, $EXIT_RC_FAILURE);
  close SQLFILE;

  open (TABLES, ">$tables") or offliner_exit("Metrica Offliner failed, cannot open $tables for write. $permission_msg", $!, $EXIT_RC_FAILURE);
  close TABLES;

  open (SUMMARY_IDS, ">$summary_ids") or offliner_exit("Metrica Offliner failed, cannot open $summary_ids for write. $permission_msg", $!, $EXIT_RC_FAILURE);
  close SUMMARY_IDS;
  
  open (RESULTS, ">$results_file") or offliner_exit("Metrica Offliner failed, cannot open $results_file for write. $permission_msg", $!, $EXIT_RC_FAILURE);
  close RESULTS;  
  
}

###############################################################################
#
# Subroutine  : offliner_warn
#
# Description : Writes warning messages.
#               Logs errors to screen and logfile.
#
# Parameters  : Logmessage, Return code, Loglevel
#
###############################################################################
sub offliner_warn {

  my $loglevel   =  $_[0];
  my $logmessage =  $_[1];
  my $returncode =  $_[2];
  my $logtime = time();

  print ("$logmessage \n");
  writelog("\: $loglevel \: metrica_offliner \| $logmessage \| $returncode\n");
}

###############################################################################
#
# Subroutine  : writelog
#
# Description : Logs messages to a logfile
#
# Parameters  : None
#
###############################################################################
sub writelog {

  if (-w $logfile){
      open LOGFILE, ">>$logfile" or die "Metrica Offliner failed, cannot open $logfile for appending :$!";
  } else {
      open LOGFILE, ">$logfile" or die "Metrica Offliner failed, cannot open $logfile for writing :$!";
  }
  print LOGFILE time(), "@_";
  close LOGFILE;
}

###############################################################################
#
# Subroutine  : write_tee
#
# Description : Logs messages to a logfile without time info and to stdout
#
# Parameters  : None
#
###############################################################################
sub write_tee {

  if (-w $logfile){
      open LOGFILE, ">>$logfile" or die "Metrica Offliner failed, cannot open $logfile for appending :$!";
  } else {
      open LOGFILE, ">$logfile" or die "Metrica Offliner failed, cannot open $logfile for writing :$!";
  }
  print LOGFILE time(), "\| INFO \| metrica_offliner \| @_";
  print "@_";
  close LOGFILE;
}


###############################################################################
#
# Subroutine  : logvars
#
# Description : Writes all env vars to the logfile. Only used for debug.
#               For use uncomment # logvars() Subroutine call.
#               Useful for debugging when run as a cronjob to locate
#               missing environment variables.
#
# Parameters  : None
#
###############################################################################
sub logvars {
  open LOGFILE, ">>$logfile";
    foreach (sort keys %ENV) {
      print LOGFILE "$_  =  $ENV{$_}\n";
    }
  close LOGFILE;
}


###############################################################################
#
# Subroutine  : clean_tmp_files
#
# Description : Removes tmp command file and tmp offliner_tables file
#
# Parameters  : None
#
###############################################################################
sub clean_tmp_files {
  unlink $sqlfile, $tables, $summary_ids, $results_file, $stop_file;
}


#***************************************************************************

=head2 offliner_exit 

 DESCRIPTION:
   Exits gracefully.
   Cleans tmp files.
   Logs errors to screen and logfile.
   Quits with exit code.

 PARAMETERS:
   $Logmessage - 
   $Return code - 
   $Exitcode

 RETURNS:
   Array with no empty data members

=cut

sub offliner_exit {

  my $logmessage =  $_[0];
  my $returncode =  $_[1];
  my $exitcode =  $_[2];
  my $logtime = time();

  clean_tmp_files();
  print ("$logmessage \n");
  writelog("\| ERROR \| metrica_offliner \| $logmessage \|: $returncode\n");
  exit $exitcode;
}





###############################################################################
#                              TIDY functionality                             #
###############################################################################



sub process_tidy {

    check_minimum_action_env();
    print_header();
    make_tmp_files();
    tidy();
    clean_tmp_files();
    exit $EXIT_RC_SUCCESS;

}


#***************************************************************************

=head2 tidy 

 DESCRIPTION:
   Tidies from the database. Optional starttime and endtime
   parameter. 
   
 PARAMETERS:
   None

 RETURNS:
    - 
    
=cut

sub tidy {
 
  if (!validate_tidy_options()) {
    offliner_exit("Metrica Offliner failed, error in usage", $!, $EXIT_RC_FAILURE);
  }

  # Since we are not using the Meta Service
  $objectid = idify_object($opt_object);
  
  get_elements();

  if ($tidyNCAction) {
    tidy_nc();
  }
  if ($tidyRawAction) {
    tidy_raw();
  }
  if ($tidyDailyAction) {
    tidy_daily();
  }
  if ($tidyWeeklyAction) {
    tidy_weekly();
  }
  if ($tidyMonthlyAction) {
    tidy_monthly();
  }
  if ($tidyPerformancedataAction) {
    tidy_performancedata();
  }
  if ($tidyAllAction) {
    tidy_all();
  }

 
}

#***************************************************************************

sub tidy_nc {

  writelog("\| AUDIT \| metrica_offliner \| Tidying nc data\n"); 

  my $get_tables_command  = "select systables.tabname from syscolumns, systables WHERE syscolumns.tabid = systables.tabid AND systables.tabname LIKE 'nc_%' AND syscolumns.colname LIKE '$objectid'";
  writelog("\| DEBUG \| metrica_offliner \| SQL $get_tables_command\n");
 
  my @lines = execute($database, $get_tables_command);
  my @rel_tables; 
  
  #
  # Store tables results
  #
  foreach my $line ( @lines ) {
        if ($line =~ /^tabname\s+(.*)$/) {
           my $tabname = $1;
           push @rel_tables, $tabname;
        }
   }
  
  #
  # Verifies tables
  #
  foreach my $table ( @rel_tables ) {
  
        writelog("\| DEBUG \| metrica_offliner \| Processing table $table\n");    

        if (!table_exists($database, $table)) {
           writelog("\| ERROR \| metrica_offliner \| Table $table does not exist\n");  
           pop @rel_tables, $table;              
         }
    
   }


  #
  # Find relations tables
  #
  my @nc_delete_tabs;
  my @nc_update_tabs;
  
  foreach my $table ( @rel_tables ) {
  
        writelog("\| DEBUG \| metrica_offliner \| Attempting to find relation entries for table $table\n");    
        
        if (find_relations_entry($admin_database, $table, $objectid)) {
           writelog("\| DEBUG \| metrica_offliner \| Adding table $table to the list of NC tables to delete from\n");    
           push @nc_delete_tabs, $table;      
         }
         else {
           if ($disable_update_functionality) {
              writelog("\| DEBUG \| metrica_offliner \| Adding table $table to the list of NC tables to update\n");    
              push @nc_update_tabs, $table;    
           }
         }
    
  }
  


  #
  # Delete nc 
  #
  writelog("\| INFO \| metrica_offliner \| Running delete on the following tables @nc_delete_tabs\n");
  foreach my $table ( @nc_delete_tabs ) {
     
         writelog("\| INFO \| metrica_offliner \| Processing delete on $table\n");
         
         foreach my $element ( @elementIDs ) {    
             my $select_SQL = "SELECT rowid FROM $table WHERE $objectid = '$element'";
             my $delete_SQL = "DELETE FROM $table WHERE $objectid = '$element'";


             writelog("\| DEBUG \| metrica_offliner \| Running SQL $select_SQL\n");  
             @lines = sqlfile_execute_unparsed($database, $select_SQL);
             my $numlines = @lines;

             if ($opt_verbose ne "") {
                write_tee("$numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");  
             } 
             else {
                writelog("\| DEBUG \| metrica_offliner \| $numlines entries in table $table meet the delete criteria for element $element $delete_msg\n");               
             }            
             
             if ($numlines > 0){
                 
                 if ($opt_nodelete eq "0") {
                    writelog("\| DEBUG \| metrica_offliner \| Running SQL $delete_SQL\n");               
                    execute_delete($delete_SQL);
                 }
                 else {
                    writelog("\| DEBUG \| metrica_offliner \| The -nodelete option is specified. Not running $delete_SQL\n");               
                 }
             }
         }  
  }

  #
  # Update nc 
  #
  my $nc_update_tabs_num = @nc_update_tabs;
  if ($disable_update_functionality) {
     if ($nc_update_tabs_num ne "0") {
          writelog("\| INFO \| metrica_offliner \| Running updates on the following tables @nc_update_tabs\n");
          foreach my $table ( @nc_update_tabs ) {
             
                 writelog("\| DEBUG \| metrica_offliner \| Processing update on $table\n");       
                 
                 foreach my $element ( @elementIDs ) {    
                     my $select_SQL = "SELECT rowid FROM $table WHERE $objectid = '$element'";
                     my $update_SQL = "UPDATE $table SET $objectid = 'null' WHERE $objectid = '$element'";

                     writelog("\| DEBUG \| metrica_offliner \| Running SQL $select_SQL\n");  
                     @lines = sqlfile_execute_unparsed($database, $select_SQL);
                     my $numlines = @lines;   
                     
                     if ($opt_verbose ne "") {
                        write_tee("$numlines entries in table $table meet the update criteria for element $element. $update_msg\n");  
                     } 
                     else {
                        writelog("\| DEBUG \| metrica_offliner \| $numlines entries in table $table meet the update criteria for element $element. $update_msg\n");               
                     }
                     
                     if ($numlines > 0){
                         
                         if ($opt_nodelete eq "0") {
                            writelog("\| INFO \| metrica_offliner \| Executing SQL $update_SQL\n");
                 
                            # Execute commented out for initial release.
                            execute_delete($update_SQL);
                         }
                         else {
                            writelog("\| DEBUG \| metrica_offliner \| The -nodelete option is specified. Not running $update_SQL\n");               
                         }
                     }   
                 }
            }
        } 
        else {
          writelog("\| INFO \| metrica_offliner \| No NC tables updates.\n");   
        }
   }  
   else {
      writelog("\| INFO \| metrica_offliner \| NC Update functionaity is not available in the current release\n");

   }

  # Tidy custom non-standard nc
  tidy_nonstandard_nc_configurations();

}


#***************************************************************************

sub tidy_performancedata() {

   writelog("\| AUDIT \| metrica_offliner \| Tidying performance data\n");

   tidy_monthly();
   tidy_weekly(); 
   tidy_daily();   
   tidy_raw();
}

#***************************************************************************

sub tidy_all() {

   writelog("\| AUDIT \| metrica_offliner \| Tidying all data\n");

   tidy_monthly();
   tidy_weekly();
   tidy_daily();
   tidy_raw();   
   tidy_nc();
}

#***************************************************************************

sub tidy_raw {

   writelog("\| AUDIT \| metrica_offliner \| Tidying raw data\n");

   my $date_column = "day";
   my $ext = "_raw";

   do_tidy($date_column, $ext);
}

#***************************************************************************

sub tidy_daily {

   writelog("\| AUDIT \| metrica_offliner \| Tidying daily data\n");

   my $date_column = "day";
   my $ext = "_day";

   do_tidy($date_column, $ext);
}


#***************************************************************************

sub tidy_weekly {

   writelog("\| AUDIT \| metrica_offliner \| Tidying weekly data\n");

   my $date_column = "week_of";
   my $ext = "_week";

   do_tidy($date_column, $ext);

}

#***************************************************************************

sub tidy_monthly {

   writelog("\| AUDIT \| metrica_offliner \| Tidying monthly data\n");

   my $date_column = "month_of";
   my $ext = "_month";

   do_tidy($date_column, $ext);
}

#***************************************************************************

sub do_tidy {

   my $date_column =  $_[0];
   my $ext = $_[1];

   my $whereDate = build_where_stmt ($date_column);
   my $tables_SQL = "select systables.tabname from syscolumns, systables where syscolumns.colname = '$objectid' and systables.tabid = syscolumns.tabid and systables.tabname like '%$ext'";
   my @lines = sqlfile_execute($database, $tables_SQL);
  
   my @merged = merge_aoa(@lines);
   foreach my $table ( @merged ) {     
         writelog("\| DEBUG \| metrica_offliner \| Processing $table\n");  
         
         foreach my $element ( @elementIDs ) {    
            my $select_SQL = "SELECT rowid FROM $table WHERE $objectid = '$element' $whereDate";
            my $delete_SQL = "DELETE FROM $table WHERE $objectid = '$element' $whereDate";

            writelog("\| DEBUG \| metrica_offliner \| Running SQL $select_SQL\n");  
            @lines = sqlfile_execute_unparsed($database, $select_SQL);
            my $numlines = @lines;
            
            if ($opt_verbose ne "") {
               write_tee("$numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");  
            } 
            else {
               writelog("\| DEBUG \| metrica_offliner \| $numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");               
            }            
            
            if ($numlines > 0){

                if ($opt_nodelete eq "0") {
                   writelog("\| DEBUG \| metrica_offliner \| Running SQL $delete_SQL\n");               
                   execute_delete($delete_SQL);
                }
                else {
                   writelog("\| DEBUG \| metrica_offliner \| The -nodelete option is specified. Not running $delete_SQL\n");               
                }
            }
        }
    }
    
    tidy_nonstandard_configurations($ext, $whereDate);
}

#***************************************************************************
sub build_where_stmt {

  my $dateColumn =  $_[0];
  my $startDate = $opt_startdate;
  my $endDate = $opt_enddate;
  my $whereStatement = "";

  # If rundate is not defined default to today in utime else convert to utime
  if(($startDate eq "") && ($endDate eq "")) {
    writelog("\| DEBUG \| metrica_offliner \| No startdate or enddate to apply.\n");
  }
  elsif(($startDate ne "") && ($endDate eq "")) {
    check_valid_date($startDate) or offliner_exit
       ("Metrica Offliner failed due to startdate $startDate in incorrect format", "", $EXIT_RC_SYNTAX);
    $startDate = get_utime($startDate);
    $whereStatement = "AND  $dateColumn >= $startDate ";
  }
  elsif(($startDate eq "") && ($endDate ne "")) {
    check_valid_date($endDate) or offliner_exit
       ("Metrica Offliner failed due to enddate $endDate in incorrect format", "", $EXIT_RC_SYNTAX);
    $endDate = get_utime($endDate);
    $whereStatement = "AND $dateColumn <= $endDate ";
  }
  elsif(($startDate ne "") && ($endDate ne "")) {
    check_valid_date($startDate) or offliner_exit
       ("Metrica Offliner failed due to startdate $startDate in incorrect format", "", $EXIT_RC_SYNTAX);
    $startDate = get_utime($startDate);
    check_valid_date($endDate) or offliner_exit
       ("Metrica Offliner failed due to enddate $endDate in incorrect format", "", $EXIT_RC_SYNTAX);
    $endDate = get_utime($endDate);
    check_startdate_enddate($startDate, $endDate) or offliner_exit
       ("Metrica Offliner failed due to startdate is later than enddate", "", $EXIT_RC_SYNTAX);
    $whereStatement = "AND $dateColumn >= $startDate AND $dateColumn <= $endDate ";
  }

   return $whereStatement;

}

#***************************************************************************
sub tidy_nonstandard_nc_configurations() {


   my @lines;
  
                
     if (check_config_file('METRICA_DBSERV_OFFLINER_NCFILE')) {
 
         my $ncfile =  $ENV{'METRICA_DBSERV_OFFLINER_NCFILE'};                    

         my (@AoA) = parse_tidy_configuration($ncfile);
         my ($i, $value);
         
         writelog("\| AUDIT \| metrica_offliner \| Tidying nonstandard nc configurations\n");
   
         for $i ( 0 .. $#AoA ) {

         my $obj = $AoA[$i][0];
         my $oid = idify_object($obj);

         if ($oid eq $objectid) {

            my $table = $AoA[$i][1];
            my $col = $AoA[$i][2];

            writelog("\| DEBUG \| metrica_offliner \| Processing $table\n");
          
            foreach my $element ( @elementIDs ) {    
               my $select_SQL = "SELECT rowid FROM $table WHERE $col = '$element'";
               my $delete_SQL = "DELETE FROM $table WHERE $col = '$element'";

               writelog("\| DEBUG \| metrica_offliner \| Running SQL $select_SQL\n");  
               @lines = sqlfile_execute_unparsed($database, $select_SQL);
               my $numlines = @lines;
               
               if ($opt_verbose ne "") {
                   write_tee("$numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");  
               } 
               else {
                    writelog("\| DEBUG \| metrica_offliner \| $numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");               
               }               
               
               if ($numlines > 0){

                   if ($opt_nodelete eq "0") {
                      writelog("\| DEBUG \| metrica_offliner \| Running SQL $delete_SQL\n");               
                      execute_delete($delete_SQL);
                   }
                   else {
                      writelog("\| DEBUG \| metrica_offliner \| The -nodelete option is specified. Not running $delete_SQL\n");               
                   }
               }
            }
         }
      } 
   } 
}

#***************************************************************************

sub tidy_nonstandard_configurations() {

   my $ext =  $_[0];
   my $where = $_[1];
   
   my @lines;
  
                
     if (check_config_file('METRICA_DBSERV_OFFLINER_NCFILE')) {
 
         my $ncfile =  $ENV{'METRICA_DBSERV_OFFLINER_NCFILE'};                    

         my (@AoA) = parse_tidy_configuration($ncfile);
         my ($i, $value);
         
         writelog("\| AUDIT \| metrica_offliner \| Tidying performance tables from nonstandard configurations\n");
   
         for $i ( 0 .. $#AoA ) {

         my $obj = $AoA[$i][0];
         my $oid = idify_object($obj);

         if ($oid eq $objectid) {

            my $nctable = $AoA[$i][1];
            my $col = $AoA[$i][2];
            my $keyid = find_access_key($admin_database, $nctable);
            
            if ($keyid ne "0") {

              my $tables_SQL = "select systables.tabname from syscolumns, systables where syscolumns.colname = '$keyid' and systables.tabid = syscolumns.tabid and systables.tabname like '%$ext'";
              my @systables = sqlfile_execute($database, $tables_SQL);
            
              my @systables_flattened = merge_aoa(@systables);
              foreach my $table ( @systables_flattened ) {     
                    writelog("\| DEBUG \| metrica_offliner \| Processing $table\n");  
                    
                    foreach my $element ( @elementIDs ) {    
                    
                         # Get access_key values
                         my $keymapping_SQL = "select distinct($keyid) from $nctable where $col = '$element'";
                         my @keyvalues = sqlfile_execute($database, $keymapping_SQL);
                         
                         my @keyvalues_flattened = merge_aoa(@keyvalues);
                         
                         my $where_key_clause = " WHERE 1 = 0 ";
                         
                         my $i = 1;
                         my $last = @keyvalues_flattened;
                         
                         foreach my $keyvalue ( @keyvalues_flattened ) { 
                            if ($i eq "1") {
     			       $where_key_clause = " WHERE $keyid IN \('$keyvalue' ";
     			    }  
     			    elsif ($i eq $last) {
     			        $where_key_clause =  "$where_key_clause, '$keyvalue' \)"
     			    }  
     			    else {
     			    	$where_key_clause =  "$where_key_clause, '$keyvalue' "
     			    }
     			    
     			    $i++;

                         }

                         my $select_SQL = "SELECT rowid FROM $table $where_key_clause $where";
                         my $delete_SQL = "DELETE FROM $table $where_key_clause $where";

                         writelog("\| DEBUG \| metrica_offliner \| Running SQL $select_SQL\n");  
                         @lines = sqlfile_execute_unparsed($database, $select_SQL);
                         my $numlines = @lines;

                         if ($opt_verbose ne "") {
                             write_tee("$numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");  
                         } 
                         else {
                             writelog("\| DEBUG \| metrica_offliner \| $numlines entries in table $table meet the delete criteria for element $element. $delete_msg\n");               
                         }

                         if ($numlines > 0){

                             if ($opt_nodelete eq "0") {
                                writelog("\| DEBUG \| metrica_offliner \| Running SQL $delete_SQL\n");               
                                execute_delete($delete_SQL);
                             }
                             else {
                                writelog("\| DEBUG \| metrica_offliner \| The -nodelete option is specified. Not running $delete_SQL\n");               
                             }
                         }
                         

 
                     }
               }
            }
            else {
                 writelog("\| WARNING \| metrica_offliner \| No access_key in lc_relations for $nctable. Not running tidy for nonstandard configuration.\n");               
            }
         }
      }  
   } 
}


#***************************************************************************

=head2  validate_tidy_options

 DESCRIPTION:
   Validate conmbinations or parameters for tidy usage

 PARAMETERS:
   None
   
 RETURNS:
   1 or 0

=cut

sub validate_tidy_options
{

  my $returnVal = 0;
  my $tableGiven = 0;
  my $fileGiven = 0;  

  my @tabletypes = split(/,/,join(',',@opt_tidy));  
  foreach $tabletype (@tabletypes) {
      writelog("\| DEBUG \| metrica_offliner \| Tabletype is $tabletype\n");      
      $tableGiven = 1;   
  }
  
  
  foreach $tabletype ( @tabletypes ) {

     SWITCH: for ($tabletype) {
                 /^nc/               && do { $tidyNCAction = 1; last; };
                 /^raw/              && do { $tidyRawAction = 1; last; };
                 /^daily/            && do { $tidyDailyAction = 1; last; };
                 /^weekly/           && do { $tidyWeeklyAction = 1; last; };
                 /^monthly/          && do { $tidyMonthlyAction = 1; last; };
                 /^performancedata/  && do { $tidyPerformancedataAction = 1; last; };   
                 /^all/              && do { $tidyAllAction = 1; last; };      
                 die "Unknown value for form variable tabletype: `$tabletype'";
     }  
  }
                  
 # Performancedata and another action not allowed   
 if (($tidyPerformancedataAction eq "1")  && 
     (($tidyRawAction || $tidyDailyAction || 
       $tidyWeeklyAction || $tidyMonthlyAction || 
       $tidyAllAction || $tidyNCAction) eq "1")) {  
       
     offliner_warn("ERROR", "-tidy performancedata and another tidy table type not allowed", $EXIT_RC_SYNTAX);
     return $returnVal;
 }

 # All and another action not allowed   
 if (($tidyAllAction eq "1")  && 
     (($tidyRawAction || $tidyDailyAction || 
       $tidyWeeklyAction || $tidyMonthlyAction || 
       $tidyPerformancedataAction || $tidyNCAction) eq "1")) {
       
     offliner_warn("ERROR", "-tidy all and another tidy table type not allowed", $EXIT_RC_SYNTAX);       
     return $returnVal;
 }
 
 # Nc and another action not allowed    
 if (($tidyNCAction eq "1")  && 
     (($tidyRawAction || $tidyDailyAction || 
       $tidyWeeklyAction || $tidyMonthlyAction || 
       $tidyPerformancedataAction || $tidyAllAction) eq "1")) {   
     offliner_warn("ERROR", "-tidy nc and another tidy table type not allowed", $EXIT_RC_SYNTAX);       
     return $returnVal;
 }
    
 # No tidy action parameter was given - not allowed 
 if ($tableGiven ne "1") {   
     offliner_warn("ERROR", "No tidy table type specified", $EXIT_RC_SYNTAX);            
     return $returnVal;
 }

 # NC tidy action parameter with dates - not allowed 
 if ((($tidyNCAction  || $tidyAllAction ) eq "1") &&
     (($opt_startdate  || $opt_enddate ) ne "")) {  
     offliner_warn("ERROR", "Cannot use date selection when tidying nc", $EXIT_RC_SYNTAX);       
     return $returnVal;
 }     
 
 if ((is_array_empty(@opt_elements)) && ($opt_file eq "")) {
     offliner_warn("ERROR", "Either the -elements or the -file parameter must be specified", $EXIT_RC_SYNTAX);       
     return $returnVal;
 }

 if (!(is_array_empty(@opt_elements)) && ($opt_file ne "")) {
     offliner_warn("ERROR", "The -elements or the -file parameter cannot both be specified", $EXIT_RC_SYNTAX);       
     return $returnVal;
 }

 if ($opt_object eq "") {
     offliner_warn("ERROR", "An object must be specified", $EXIT_RC_SYNTAX);       
     return $returnVal;
 } elsif (($opt_object eq "nc") || ($opt_object eq "nc_id")) {
     offliner_warn("ERROR", "An object of type $opt_object cannot be specified", $EXIT_RC_SYNTAX);       
     return $returnVal; 
 }

 if (defined $opt_nodelete){
     $opt_nodelete = 1; #on (no deletion)
     $delete_msg = "No delete shall be executed.";
     $update_msg = "No updates shall be executed.";
     
 } 
 else {
     $opt_nodelete = 0; #off (deletion)
     $delete_msg = "Executing deletes.";
     $update_msg = "Executing updates";
 }
  

 $returnVal = 1;
 return $returnVal;

}

#***************************************************************************

sub get_elements(){

  my $elementGiven;

  if ($opt_file ne "") {
      $elementGiven = process_element_file();
  } else { 
      my @elements = split(/,/,join(',',@opt_elements));  
      foreach $element (@elements) {
           writelog("\| DEBUG \| metrica_offliner \| Element is $element\n");    
           push @elementIDs, $element;
           $elementGiven = 1;       
      }
  }
  
  if (!($elementGiven)) {
      offliner_exit ("Metrica Offliner failed due to no element given", "", $EXIT_RC_SYNTAX);   
  }
  

}

#***************************************************************************

sub idify_object {

       my ($obj) = @_;
       my $obj_idifier = "_id";   
 
       # Needed to map object to column
       my $objID = "$obj$obj_idifier";
       writelog("\| DEBUG \| metrica_offliner \| Object is \"$obj\", mapping to column name \"$objID\"\n");           
       
       return $objID;
} 

#***************************************************************************

sub is_array_empty {

    my @arr = @_;
    my $returnVal = 1;
        
    foreach my $val ( @arr ) {
      $returnVal = 0; # Empty
    }
    
    return $returnVal;

}


#***************************************************************************

sub process_element_file {

  my $hasElements = 0;
   
  # Read from the elements file
    open ELEMENTS, $opt_file or offliner_exit("Metrica Offliner failed, cannot open file $opt_file for read", $!, $EXIT_RC_FAILURE);
    while (<ELEMENTS>) {

   my $element = $_;
   chomp $element;       
   writelog("\| DEBUG \| metrica_offliner \| Reading element from element file $element \n");    
        push @elementIDs, $element;
        $hasElements = 1; 
   
    } #while (<ELEMENTS>)   
    close ELEMENTS; 
    
    return $hasElements;
 
}


#***************************************************************************

=head2 chomp_values 

 DESCRIPTION:
   Remove empty array entries

 PARAMETERS:
   $self - The class object (implicit parameter).
   @data - Data array

 RETURNS:
   Array with no empty data members

=cut

sub chomp_values
{
    
    my (@data) = @_;

    return grep(!/^\s*$/, @data);
}

#***************************************************************************

=head2  execute

 DESCRIPTION:
   Use dbaccess to execute an SQL statement

 PARAMETERS:
   $database - Database name
   $sql      - SQL statement

 RETURNS:
   Lines returned as a list

=cut

sub execute
{
   
   my ($database, $sql) = @_;
   my $command;

   $command = sprintf "/usr/bin/echo \"%s\" | \$INFORMIXDIR/bin/dbaccess %s 2>&1", $sql, $database;
   my @lines = `$command`;

   return chomp_values(@lines);
}

#***************************************************************************


sub execute_delete {

  my @arg_list;
  my $del_command   = $_[0];  
  my $rc;
   
  writesql("$del_command");
  writelog("\| INFO \| metrica_offliner \| $del_command\n");
  @arg_list = ( $DATABASE_CMD, $database, $sqlfile);

  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
  if ($rc){
      offliner_warn("ERROR", "Delete command failed", $!);
  }
}


#***************************************************************************

=head2  sqlfile_execute

 DESCRIPTION:
   Use dbaccess to execute an SQL statement, output to a temp file, parse the results

 PARAMETERS:
   $database - Database name
   $sql      - SQL statement

 RETURNS:
   Results as an array of arrays

=cut

sub sqlfile_execute
{
  
  my ($database, $sql) = @_;
  my $parse = 1;
  
  my @results = sqlfile_execute_parsable($database, $sql, $parse);
  
  return (@results);
}

#***************************************************************************

=head2  sqlfile_execute_unparsed

 DESCRIPTION:
   Use dbaccess to execute an SQL statement, output to a temp file, unparse the results

 PARAMETERS:
   $database - Database name
   $sql      - SQL statement

 RETURNS:
   Results as an array

=cut

sub sqlfile_execute_unparsed
{
  
  my ($database, $sql) = @_;
  my $parse = 0;
  
  my @results = sqlfile_execute_parsable($database, $sql, $parse);
  
  return (@results);
}



#***************************************************************************

=head2  sqlfile_execute_parsable

 DESCRIPTION:
   Use dbaccess to execute an SQL statement, output to a temp file, parse the results optionally

 PARAMETERS:
   $database - Database name
   $sql      - SQL statement
   $parse    - 1 parse, 0 unparse

 RETURNS:
   Results as an array

=cut

sub sqlfile_execute_parsable
{
  
  my ($database, $sql, $parse) = @_;
  my @execution_results;
  my $rc;
  
  my $wrapped = wrapsql($sql, $results_file);
  writesql("$wrapped");

  writelog("\| DEBUG \| metrica_offliner \| $wrapped\n");
  my @arg_list = ($DATABASE_CMD, $database, $sqlfile);
  $rc = 0xffff & system( "@arg_list >>$logfile 2>&1");
  if ($rc){
    writelog("\| ERROR \| metrica_offliner \| Metrica Offliner error when executing \"$wrapped\" \n");
    return (@execution_results);
  }
  if ($parse) {
    @execution_results = parse_results($results_file);
  } else {
    @execution_results = get_results($results_file);  
  }
  
  return (@execution_results);
}

#***************************************************************************

=head2  parse_tidy_configuration

 DESCRIPTION:
   Open the specified tidy configuration file and returns the configuration

 PARAMETERS:
   $file     - The results file

 RETURNS:
    @nsObjectArray - Array of non standard NC configurations   
    
=cut

sub parse_tidy_configuration
{
  
  my ($file) = @_;
  my(@nsObject, @nsObjectArray);
  my $delim = "\\,";
  my $keep = 0;  

  # Read from the nonstandard elements configuration file
  open CONFIG, $file or offliner_exit("Metrica Offliner failed, cannot open $file for read", $!, $EXIT_RC_FAILURE);
  while (<CONFIG>) {  
    my $line = $_;
    @nsObject = parse_line("$delim", $keep, $line); 
    my @emptied_nsObject = reduce_array(@nsObject);
    push @nsObjectArray, [@emptied_nsObject];
  }
  
  return(@nsObjectArray); 
   
}


#***************************************************************************

=head2  parse_results

 DESCRIPTION:
   Open the specified results file and return the results

 PARAMETERS:
   $file     - The results file

 RETURNS:
   Lines parsed 

=cut

sub parse_results
{
  
  my ($file) = @_;
  my(@results, @allresults);
  my $delim = "\\|";
  my $keep = 0;  

  # Read from the offline table file
  open RESULTS, $file or offliner_exit("Metrica Offliner failed, cannot open $file for read", $!, $EXIT_RC_FAILURE);
  while (<RESULTS>) {  
    my $line = $_;
    @results = parse_line("$delim", $keep, $line); 
    my @emptied_results = reduce_array(@results);
    push @allresults, [@emptied_results];
  }
  
  return(@allresults); 
   
}

#***************************************************************************

=head2  get_results

 DESCRIPTION:
   Open the specified results file and simply return the results

 PARAMETERS:
   $file     - The results file

 RETURNS:
   Lines  

=cut

sub get_results
{
  
  my ($file) = @_;
  my(@results);
 

  # Read from the offline table file
  open RESULTS, $file or offliner_exit("Metrica Offliner failed, cannot open $file for read", $!, $EXIT_RC_FAILURE);
  while (<RESULTS>) {  
    my $line = $_;
    push @results, $line;
  }
  
  my @emptied_results = reduce_array(@results);
  return(@emptied_results); 
   
}


#***************************************************************************

sub reduce_array
{
  my (@full_array) = @_;
  my(@reduced_array);

  foreach my $array_element(@full_array){  
     chomp $array_element;
     if (length($array_element)) {
        push (@reduced_array, $array_element) 
     } 
  }

  return(@reduced_array);   
}

#***************************************************************************

=head2  wrapsql

 DESCRIPTION:
   Wraps sql commands to unload to a results file

 PARAMETERS:
   $sql      - The sql command
   $file     - The results file

 RETURNS:
   $wrapped_sql - The wrapped sql command

=cut

sub wrapsql {

  
  my ($sql, $file) = @_;
  
  my $wrapped_sql = "unload to $file $sql";  

  return $wrapped_sql;
}



#***************************************************************************

=head2  get_index_column_details

 DESCRIPTION:
   This method is used to translate the column id parameter that is stored
   in the index system catalog.
   
   Subroutine unused in current version.   

 PARAMETERS:

   $database - Database name
   $table    - Table the column belongs to
   $colno    - Column number to translate

 RETURNS:
   Column name

=cut

sub get_index_column_details
{
   
   my ($database, $table, $colno) = @_;
   my $retValue = "";
  
   #
   # Find the column name based on the colid
   #
   my @lines = execute($database, "SELECT cols.colname FROM syscolumns cols,systables tab WHERE cols.tabid = tabs.tabid AND tabs.tabname = '$table' AND cols.colno = abs($colno);");

   foreach my $line ( @lines ) {
      if ($line =~ /^colname\s+(.*)$/) {
         my $colname = $1;

         if ($colno > 0) {
            $retValue = $colname;
         }
         else {
            $retValue = "$colname DESC";
         }
      }
   }

   if ($retValue eq "") {
      die "Could not translate column number: $table, $colno";
   }

   return $retValue;
}

#***************************************************************************

=head2  get_index_details

 DESCRIPTION:
   This method is used to retrieve the index details for a given table.
   
   Subroutine unused in current version.		

 PARAMETERS:
   $database - Database name
   $table     - SQL statement

 RETURNS:
  Hash references keyed by indexname
  1) Type of index
  2) Index columns
  3) Index dbspace

=cut

sub get_index_details
{
   
   my ($database, $table) = @_;
   my $idxname;
   my @idxname;
   my %idxcolumns;
   my %idxdbspace;
   my %idxtype;

   #
   # Get the indexes associated with this table
   #
   my @lines = execute($database, "SELECT idx.idxname, 'END', idx.indexkeys, idx.idxtype, frag.dbspace FROM sysindices idx, systables tab, OUTER sysfragments frag WHERE idx.tabid = tab.tabid AND idx.tabid = frag.tabid AND idx.idxname = frag.indexname AND frag.fragtype = 'I' AND tab.tabname = '$table';");

   #printf "DEBUG:\n%s", join("",@lines);

   foreach my $line ( @lines ) {
      # Necessary for ".." operator
      $_ = $line;

      if ($line =~ /^idxname\s+/ .. /^\(constant\)\s+/) {
         if ($line =~ /^idxname\s+(.*)$/) {
            @idxname = ($1);
         }
         elsif ($line =~ /^\s+(.*)$/) {
            push @idxname, $1;
         }
         elsif ($line =~ /^\(constant\)\s+/) {
            $idxname = join("",@idxname);
         }
      }
      elsif ($line =~ /^indexkeys\s+(.*)$/) {
    #
    # Index keys have the following format:
    #
    # 1 [1], -2 [1]
    # ^-------^-------- These are the column numbers
    # Note: A negative value indicates that this is a descending index
    #
         my $indexKeys = $1;
    my @columns = ();

    foreach my $key ( split(",", $indexKeys) ) {
       if ($key =~ /^\s*(\S+)\s+\[/) {
               my $colno = $1;
          push @columns, get_index_column_details($database, $table, $colno); 
       }
       else {
               die "Index key has unexpected format: $indexKeys";
       }
    }

         $idxcolumns{$idxname} = \@columns;
      } 
      elsif ($line =~ /^idxtype\s+(.*)$/) {
         if ($1 eq "U") {
            $idxtype{$idxname} = "UNIQUE";
         }
         else {
            $idxtype{$idxname} = "";
         }
      }
      elsif ($line =~ /^dbspace\s+(.*)$/) {
         $idxdbspace{$idxname} = $1;
      }
   }

   return (\%idxtype, \%idxcolumns, \%idxdbspace);
}

#***************************************************************************

=head2  get_index_SQL

 DESCRIPTION:
   This method is used to recover the information regarding all table
   indexes involving this column.
   2 list references are returned. One list contains SQL statements to
   drop the indexes, the second contains SQL statements to create the
   indexes.
   
   Subroutine unused in current version.

 PARAMETERS:
   $database - database name
   $table    - Table name
   $column   - column name

 RETURNS:
   1) Array reference to "DROP INDEX" SQL statements
   2) Array reference to "CREATE INDEX" SQL statements
   

=cut

sub get_index_SQL
{
   
   my ($database, $table, $column) = @_;
   my %indexData;
   my @drop = ();
   my @create = ();

   #
   # Get the indexes associated with this table
   #
   my ($idxtype, $idxcolumns, $idxdbspace) = get_index_details($database, $table);

   foreach my $index ( keys %{$idxcolumns} ) {
      if (grep(/^$column\s*/, @{$idxcolumns->{$index}}) > 0) {
         push @drop,   "SET LOCK MODE TO WAIT;";
         push @drop,   sprintf "DROP INDEX %s;", $index;

         push @create, "SET PDQPRIORITY 80;";
         push @create, sprintf "CREATE %s INDEX %s ON %s (%s) IN %s;", $idxtype->{$index}, $index, $table, join(",",@{$idxcolumns->{$index}}), $idxdbspace->{$index};
      }
   }

   return (\@drop, \@create);
}

#***************************************************************************

=head2  table_exists

 DESCRIPTION:
  Ensure that the table exists

 PARAMETERS:
   $database - database name
   $table    - Table name

 RETURNS:
   1 or 0

=cut

sub table_exists
{   
   my ($database, $table) = @_;
   my $returnVal = 0;
   
   my $table_exists_SQL = "SELECT tab.tabname FROM systables tab WHERE tab.tabtype = 'T' AND tab.tabname = '$table'";

   my @lines = execute($database, $table_exists_SQL);

   foreach my $line ( @lines ) {
      if ($line =~ /^tabname\s+/) {
         writelog("\| DEBUG \| metrica_offliner \| Table $table exists\n");    
         $returnVal = 1;
      }
   }

   return $returnVal;
}
#***************************************************************************

=head2  find_relations_entry

 DESCRIPTION:
  Queries the lc_relations table in the admin db

 PARAMETERS:
   
   $admin_database - admin database name
   $table          - Table name
   $objectid       - The objectid    
   
 RETURNS:
   1 or 0

=cut

sub find_relations_entry
{
   
   my ($admin_database, $table, $objectid) = @_;
   my $returnVal = 0;
   my $related_tables;
   my @merged;
   
   writelog("\| DEBUG \| metrica_offliner \| Finding relations entries for table $table\n");    
   
   my $relations_SQL = "SELECT distinct(nc_tabname) FROM lc_relations rel WHERE rel.nc_tabname = '$table' AND rel.access_key = '$objectid'";

   my @lines = sqlfile_execute($admin_database, $relations_SQL);
  
   @merged = merge_aoa(@lines);
   foreach my $result ( @merged ) {
      
            writelog("\| DEBUG \| metrica_offliner \| Objecttype $objectid is an access key on table $table\n"); 
            $returnVal = 1;
            
   }
    
   return $returnVal;
}
#***************************************************************************

=head2  find_access_key

 DESCRIPTION:
  Queries the lc_relations table in the admin db to get an access key

 PARAMETERS:
   
   $admin_database - admin database name
   $table          - Table name
   
 RETURNS:


=cut

sub find_access_key
{
   
   my ($admin_database, $table) = @_;
   my $returnVal = 0;
   my $related_tables;
   my @merged;
   my $key = 0;
   
   writelog("\| DEBUG \| metrica_offliner \| Finding relations entries for table $table\n"); 
   
   my $accesskeys_SQL = "SELECT distinct(access_key) FROM lc_relations rel WHERE rel.nc_tabname = '$table' ";

   my @lines = sqlfile_execute($admin_database, $accesskeys_SQL);
  
   @merged = merge_aoa(@lines);
   my $nummerged = 0;
   foreach my $result ( @merged ) {  
        $nummerged++; 
        writelog("\| DEBUG \| metrica_offliner \| Access key on table $table is $result\n"); 
   }
    
   if ($nummerged eq "1") {
       $key = $merged[0];
       writelog("\| INFO \| metrica_offliner \| Access key is $key\n");       
   } 
   elsif ($nummerged eq "0") {
       writelog("\| ERROR \| metrica_offliner \| No access key on table $table\n");
   }
   else {
       writelog("\| ERROR \| metrica_offliner \| More than one access key on table $table\n");
   }
   
   return $key;
}


#***************************************************************************

=head2  merge_aoa

 DESCRIPTION:
  Merges arrays of array into a single results array

 PARAMETERS:
   
   @AoA            - Array Of Arrays
   
 RETURNS:
   @merged_array   - Array Of results

=cut

sub merge_aoa
{
   
   my (@AoA) = @_;
   my @merged_array;
   my ($i, $j, $aref, $value);
   
   for $i ( 0 .. $#AoA ) {
   $aref = $AoA[$i];
   for $j ( 0 .. $#{$aref} ) {
       $value = $AoA[$i][$j];
       push (@merged_array, $value);
   }
    }  
    
   return @merged_array;
}

#***************************************************************************

=head2  table_column_type

 DESCRIPTION:
   Returns the type of a specified table column
   Note there is only logic to spot "integer" and opaque type names 
   like "utime".

 PARAMETERS:
   $database - database name
   $table    - Table name
   $column   - column name

 RETURNS:
   Column type

=cut

sub table_column_type
{
   
   my ($database, $table, $column) = @_;

   my @lines = execute($database, "SELECT col.coltype, xtd.name FROM syscolumns col,systables tab,OUTER sysxtdtypes xtd WHERE col.tabid = tab.tabid AND col.extended_id = xtd.extended_id AND tab.tabtype = 'T' AND tab.tabname = '$table' AND col.colname = '$column'");
   my $typeid = "notExist";
   my $xtdType;

   foreach my $line ( @lines ) {
      if ($line =~ /^coltype\s+(.*)$/) {
         $typeid = $1;
      }
      elsif ($line =~ /^name\s+(.*)$/) {
         $xtdType = $1;
      }
   }

   #
   # If the typeid is >= 256 then "not null" condition is set
   #
   if ($typeid >= 256) {
      $typeid -= 256;
   }

   #
   # Translate the typeid
   #
   if ($xtdType eq "utime") {
      return "utime";
   }
   elsif ($typeid eq 2) {
      return "integer";
   }
   else {
      return "typeid = $typeid";
   }
}

#***************************************************************************

sub check_startdate_enddate {

  my $startdate   = $_[0];  
  my $enddate     = $_[1];

  # compare dates
  if ($startdate > $enddate){
     return 0; #error
  }
  else {
     return 1; #no error
  }
}

#***************************************************************************
=head2  does_stop_file_exist

 DESCRIPTION:
	Checks to see if the  a predefined file  exists

 PARAMETERS:
  none
 RETURNS:
    0 - if the file does not exists
    1 - if the file exists

=cut

sub does_stop_file_exist {
  if (-f $stop_file) {
     return 1;
   }
  else {
     return 0;
   }
}

#***************************************************************************

=head2  stop

 DESCRIPTION:
  Causes the offliner to stop after it has processed it current job.  This function
	creates a predefined file that the function does_stop_file_exist detects.

 PARAMETERS:
  none
 RETURNS:
     none

=cut

sub stop{
  open HANDLE, ">$stop_file";
  close HANDLE;
}

