#!/bin/ksh
###############################################################################
# @(#) SysManagement/delivery/unix/bin/metrica_appserveradmin.ksh SysManagement_r4.79.0.9:cset.008500:38:38 08/27/07 @(#)
#
#                   COPYRIGHT NOTICE
#
#   This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
# 
#   Copyright  2005 Vallent Software Systems UK. All rights reserved
#
# Title:        metrica_appserveradmin.ksh
# Authors:      Patrick Wood / Alan Greasley / James Richardson
#               Shane O'Rourke
# Description:  Appserver admin
#
###############################################################################


#---------------------------------
#      Global variables
#---------------------------------

PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
IAS_INIT_DELAY=30
START_TIMEOUT=300    # Need a timeout for these wait loops, in seconds
STOP_TIMEOUT=120     # Need a timeout for these wait loops, in seconds
STOP_PTN_TIMEOUT=60  # AJG - Partition shouldn't take long
RESTART_TIMEOUT=500  # Need a timeout for these wait loops, in seconds
DELAY=10
DEVELOPER_MODE=false
DEBUG_MODE=false
TMP_FILE=/tmp/metrica_appserveradmin.out.$$
PARTITION_HEADER="Partition Name"
DETAILS_HEADER="Partition Details"
STANDARD_SERVICES="EJB Container; Naming Service; JTS; JSS"
WEB_SERVICES="Web Container"  
PARTITIONS_DIR="$APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/" 
PROPERTIES_FILE="$APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/properties/server.properties"
URI_DIR="$APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/conf/"
URI_FILE="UriMapFile.properties"
WAR_DIR="$APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard/wars/"
SERVICES_PROPERTY="metrica.static.partitions.list"
CLONE_SCRIPT="metrica_cloneadmin.pl"
OS_TYPE=`uname -s`
DROP_SPW_TMP_FILE=/tmp/spwrappers_to_drop.$$

#---------------------------------
#      Start Stop codes
#---------------------------------

integer RUNNING=0
integer STARTING=1
integer STOPPING=3
integer STOPPED=4

#---------------------------------
#      Error codes
#---------------------------------

integer ERR_USAGE=101
integer ERR_ENV=102
integer ERR_TIMEOUT=103
integer ERR_NOAPPSERVER=104
integer ERR_PTN_STOP_FAILED=110


#---------------------------------
#      Return codes
#---------------------------------

integer SUCCESS=0
integer FAILURE=1

#---------------------------------
#      Onstat return codes
#---------------------------------

integer INFORMIX_ON_LINE=5

#---------------------------------
#      Setup and Validation
#---------------------------------

# Set up common variables
#
. metrica_install_common.ksh

# Check environment variables
#
[[ -z "$APPSERVER_DIR" ]] && $ECHO "APPSERVER_DIR not defined" && exit $ERR_ENV
[[ -z "$APPSERVER_MGMT_PORT" ]] && $ECHO "APPSERVER_MGMT_PORT not defined" && exit $ERR_ENV
[[ -z "$APPSERVER_NAME" ]] && $ECHO "APPSERVER_NAME not defined" && exit $ERR_ENV
[[ -z "$METRICA_DIR" ]] && $ECHO "METRICA_DIR not defined" && exit $ERR_ENV
[[ -z "$METRICA_PERFORMANCE_DB" ]] && $ECHO "METRICA_PERFORMANCE_DB not defined" && exit $ERR_ENV

# The following variables can be optionally over-ridden (testing?)
#
[[ -z "$APPSERVER_AUTH_FILE" ]] && APPSERVER_AUTH_FILE=$APPSERVER_DIR/.bes_auth
[[ -z "$APPSERVER_LOG_FILE" ]] && APPSERVER_LOG_FILE=$APPSERVER_DIR/startup.log

# The following variables are explicitly set
#
EJBPARTITION_NAME=${EJBPARTITION_NAME:-standard}
WEBPARTITION_NAME=${WEBPARTITION_NAME:-standard} # historical reasons
APPSERVER_STD_ARGS="-mgmtport $APPSERVER_MGMT_PORT -server $APPSERVER_NAME"
JAVA_HOME=${JAVA_HOME:-/usr/j2se}

# Make sure appserver binaries are on path
#
export PATH=$PATH:$APPSERVER_DIR/bin





###############################################################################
# UTILITY FUNCTIONS
###############################################################################


###################################################################
#
# Function    : cleanup
#
# Description : Clean up tmp file
#
###################################################################

cleanup(){
    [ "$DEBUG_MODE" = "false" ] && rm -f $TMP_FILE
}

###################################################################
#
# Function    : log
#
# Description : Log to screen & tmp file. 
#               Please log carefully when using ksh as I/O
#               functions can be interfered with by logmessages,
#
###################################################################

log(){
    $ECHO $* | tee -a $TMP_FILE
}

###################################################################
#
# Function    : is_cloned
#
# Description : Check to see if any partitions have been cloned
#
###################################################################

is_cloned()
{
    RETURN="FALSE"

    my_web_partitions="`get_web_partitions`"      
    for my_partition in $my_web_partitions
    do   
        RETURN="TRUE"
    done       
}

###################################################################
#
# Function    : check_directory
#
# Description : Checks to see if the directory specified exists
#
###################################################################

check_directory()
{
   my_dir="$1" 
 
   if [ ! -d "$my_dir" ]; then
     RETURN="FALSE"
   else
     RETURN="TRUE"
   fi
}

###################################################################
#
# Function    : get_property
#
# Description : Retrieves the value from a named properties file 
#               for the specified property
#
###################################################################

get_property()
{

   my_property_name="$1"
   my_property_file="$2"

   # Get second field
   #
   awk -F"=" '
   /^[ \t]*'"$my_property_name"'[ \t]*=[ \t]*/ {
       print $2
       exit
   }
   ' $my_property_file  | xargs echo
   
}


###################################################################
#
# Function    : validinteger
#
# Description : Return success if string consists only of
#               numbers
#
###################################################################

validinteger () { 
   case "$1" in
       *[!0-9]*)   RETURN="FALSE";;  # string contains invalid chars.
       *)          RETURN="TRUE";;  # only digits from 0-9
   esac
}



###############################################################################
# START
###############################################################################


###################################################################
#
# Function    : start_all
#
# Description : Starts the appserver and all defined partitions. 
#               First start the Appserver itself. Then start partitions.
#               The call to start partitions will first start the EJB 
#               Partition and then any the WEB Partitions, if separate 
#               from the EJB Partition (i.e. if partition cloning has 
#               taken place). Chain the commands to ensure that the start 
#               of one is preceded by the success of the previous 
#               (success = RUNNING = 0)
#
###################################################################

start_all()
{
  start_server && sleep $DELAY && start_partitions

  # Return error code of last routine to run (only going to be RUNNING unless
  # something has timed out and we are in trouble - error message will have
  # been printed
  return $?
}

###################################################################
#
# Function    : start_server
#
# Description : Starts the appserver
#
###################################################################

start_server()
{
  integer STAT
  if [ "$DEVELOPER_MODE" = "true" ] ; then
    DEVELOPER_MODE=false
    STAT=$STOPPED
  else
    status_server
    STAT=$?
  fi

  if [[ $STAT -eq $RUNNING ]] ; then
    log "Appserver already running"
    # If it's already up, return
    return $RUNNING
  elif [[ $STAT -eq $STARTING ]] ; then
    log "Appserver starting"
  else
   if [[ -z "$JSS_REMOVE_SKIP" ]];
   then
      log "Proceeding to remove jss storage files"
      if [ -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard/jss_storage* ];
      then
        log "Removing jss storage files ..."
        rm -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard/jss_storage* 2> /dev/null
      else
        log "AppServer jss storage files not found , skipping jss storage deletion"
      fi
    else
      log "Skipping jss storage deletion"
    fi

    # 56645
    onstat - >/dev/null 2>/dev/null
    ONSTAT_RC=`echo $?`
    if [[ -z "$METRICA_HOUSEKEEPING_SKIP_DROPTEMPSPRS" && $ONSTAT_RC -eq $INFORMIX_ON_LINE ]];
    then
      log "Dropping SP wrappers ..."

      # Retrieve SP wrappers
      echo "DROP TABLE tmp_spwrappers_to_drop" | dbaccess $METRICA_PERFORMANCE_DB >/dev/null 2>/dev/null
      echo "SELECT procname FROM sysprocedures WHERE procname LIKE '%\_sprrepwrapper\_%' INTO TEMP tmp_spwrappers_to_drop;UNLOAD TO '$DROP_SPW_TMP_FILE' SELECT * FROM tmp_spwrappers_to_drop;" | dbaccess $METRICA_PERFORMANCE_DB >/dev/null 2>/dev/null
		
      # Drop wrappers
      if [[ -s $DROP_SPW_TMP_FILE ]];
      then
        while read wrapper 
        do
          echo "DROP PROCEDURE ${wrapper%!(|)}" | dbaccess $METRICA_PERFORMANCE_DB >/dev/null 2>/dev/null
        done < $DROP_SPW_TMP_FILE
        rm -f $DROP_SPW_TMP_FILE >/dev/null 2>/dev/null
      else
        log "No wrappers to remove."
      fi
    else
      # METRICA_HOUSEKEEPING_SKIP_DROPTEMPSPRS is set (or informix is down...)
      log "Skipping SPR wrappers removal"
    fi

    # If it's not already starting, start the appserver	
    log "Starting appserver ..."
    nohup $APPSERVER_DIR/bin/ias >> $APPSERVER_DIR/startup.log 2>&1 &
  fi

  # Extra sleep for Server Start
  log -n "Waiting for AppServer to start ..."
  sleep $IAS_INIT_DELAY
  DELAY_TOTAL=$IAS_INIT_DELAY
  TIMED_OUT=0

  # Now loop until it's up - hmm, we don't want to loop forever, though!
  while [[ $STAT -ne $RUNNING ]] ; do
    log -n "."
    let DELAY_TOTAL+=$DELAY
    sleep $DELAY
    [[ $DELAY_TOTAL -ge $START_TIMEOUT ]] && TIMED_OUT=2 && break
    status_server SILENT
    STAT=$?
    [[ $STAT -gt $STARTING ]] && TIMED_OUT=1 && break
  done

  if [[ $TIMED_OUT -eq 1 ]] ; then
    log " start failed"
    return $STOPPED
  elif [[ $TIMED_OUT -eq 2 ]] ; then
    log " timed out"
    return $ERR_TIMEOUT
  else
    log " started"
    return $RUNNING
  fi
}

###################################################################
#
# Function    : start_partitions
#
# Description : Starts the partitions. Only starts the web 
#               partitions if the EJB (standard) partition has
#               been started successfully and if web partitions
#               exist.
#
###################################################################

start_partitions()
{

    # Start standard partition
    start_ejb_partition
    STAT=$?
    [[ $STAT -ne $RUNNING ]] && return $STAT
    
    # Start of web partitions if cloned    
    is_cloned 
    my_is_cloned="$RETURN"   
    if [ "$my_is_cloned" = "TRUE" ]; then  
        start_web_partitions
        STAT=$?
        [[ $STAT -ne $RUNNING ]] && return $STAT
    fi
    
    return $RUNNING    

}

###################################################################
#
# Function    : start_ejb_partition
#
# Description : Starts the standard partition
#
###################################################################

start_ejb_partition() 
{
    my_ejb_partitions="`get_ejb_partitions`"  
    for my_partition in $my_ejb_partitions
    do      
        start_partition $my_partition 
        STAT=$?
        break # should only be one
    done  
    
    return $STAT
}

###################################################################
#
# Function    : start_web_partitions
#
# Description : Starts web partitions. If one fails to start 
#               it quits out.
#
###################################################################

start_web_partitions()
{
    my_web_partitions="`get_web_partitions`"      
    for my_partition in $my_web_partitions
    do   
        start_partition $my_partition 
        STAT=$?
        [[ $STAT -ne $RUNNING ]] && return $STAT        
    done  
    
    return $STAT
}


###################################################################
#
# Function    : start_named_partition
#
# Description : Starts a particular named partitions
#
###################################################################

start_named_partition()
{   
    my_named="$1" 	
    
    is_valid_partition $my_named  
    my_is_valid="$RETURN" 
    

    if [ "$my_is_valid" = "TRUE" ]; then  
       
        # Start the EJB partition if necessary
        if [ "$my_named" != "$EJBPARTITION_NAME" ]; then            
            start_ejb_partition
            STAT=$?
            [[ $STAT -ne $RUNNING ]] && return $STAT  
        fi 
        
        start_partition $my_named 
        STAT=$?  
        
    else
        echo "Error : Routine start_named_partition(). Partition $my_named does not exist" && 
        return -1
    fi
      
    return $STAT  
      
}



###################################################################
#
# Function    : start_partition
#
# Description : Starts an individual specified partition
#
###################################################################

start_partition()
{
 
  PNAME=$1

  integer STAT

  if [ "$DEVELOPER_MODE" = "true" ] ; then
    STAT=$STOPPED
  else
    status_partition $1
    STAT=$?
  fi

  if [[ $STAT -eq $RUNNING ]] ; then
    # If it's already up, return
    #
    log "Partition $PNAME already running"
    return $RUNNING
  elif [[ $STAT -eq $STARTING ]] ; then
    log "Partition $PNAME already starting"
  else
    # If it's not already starting, start the EJB Partition
    #
    STATUS=`iastool -start $APPSERVER_STD_ARGS -partition $PNAME < $APPSERVER_AUTH_FILE | tee -a $TMP_FILE | tail -1`
    log "${STATUS##*password: } ..."
  fi

  log -n "Waiting for partition to start ..."
  DELAY_TOTAL=0
  TIMED_OUT=0

  # Now loop until it's up - hmm, we don't want to loop forever, though!
  #
  while [[ $STAT -ne $RUNNING ]] ; do
    log -n "."
    DELAY_TOTAL=`expr $DELAY_TOTAL + $DELAY`
    sleep $DELAY
    [[ $DELAY_TOTAL -ge $START_TIMEOUT ]] && TIMED_OUT=2 && break
    status_partition $1 SILENT
    STAT=$?
    [ $STAT -gt $STARTING ] && TIMED_OUT=1 && break
  done

  if [[ $TIMED_OUT -eq 1 ]] ; then
    log " start failed"
    return $STOPPED
  elif [[ $TIMED_OUT -eq 2 ]] ; then
    log " timed out"
    return $ERR_TIMEOUT
  else
    log " started"
    return $RUNNING
  fi
}



###############################################################################
# RESTART
###############################################################################


###################################################################
#
# Function    : restart_all
#
# Description : Restart full appserver system (inc sub-processes)
#
###################################################################

restart_all()
{
  # First restart the appserver itself
  # Then make sure the EJB Partition is started
  # Then make sure any Web Partition is started
  # Chain the commands to ensure that the start of one is preceded by 
  # the success of the previous (success = RUNNING = 0)
 
  restart_server && sleep $DELAY && start_partitions

  # Return error code of last routine to run (only going to be RUNNING unless
  # something has timed out and we are in trouble - error message will have
  # been printed
  return $?
}


###################################################################
#
# Function    : restart_server
#
# Description : Restart Server, Stops all processes first.
#
###################################################################

restart_server()
{
  log "Restarting appserver (using stop / start) ..."

  stop_all

  start_server

  return $?
}

###################################################################
#
# Function    : restart_partitions
#
# Description : Restarts the partitions. Only restarts the web 
#               partitions if the EJB (standard) partition has
#               been restarted successfully and if web partitions
#               exist.
#
###################################################################

restart_partitions()
{

    # Start standard partition
    restart_ejb_partition
    STAT=$?
    [[ $STAT -ne $RUNNING ]] && return $STAT    
    return $RUNNING    

}

###################################################################
#
# Function    : restart_ejb_partition
#
# Description : Restarts the standard partition. The web partitions
#               must also be restarted in this case if any exist.
#
###################################################################

restart_ejb_partition() 
{

    restart_web_partitions
    STAT=$?
    [[ $STAT -ne $RUNNING ]] && return $STAT   

    my_ejb_partitions="`get_ejb_partitions`"  
    for my_partition in $my_ejb_partitions
    do      
        restart_partition $my_partition 
        STAT=$?
        break # can only be one
    done  
    
    return $STAT
}

###################################################################
#
# Function    : restart_web_partitions
#
# Description : Starts web partitions. If one fails to restart 
#               it quits out.
#
###################################################################

restart_web_partitions()
{
    STAT="$RUNNING"

    # Start of web partitions if cloned    
    is_cloned 
    my_is_cloned="$RETURN"   
    if [ "$my_is_cloned" = "TRUE" ]; then  
       my_web_partitions="`get_web_partitions`"      
       for my_partition in $my_web_partitions
       do   
           restart_partition $my_partition 
           STAT=$?
           [[ $STAT -ne $RUNNING ]] && return $STAT        
       done  
    fi

    return $STAT
}


###################################################################
#
# Function    : restart_named_partition
#
# Description : Starts a particular named partitions
#
###################################################################

restart_named_partition()
{   
    my_named="$1" 	
    
    is_valid_partition $my_named  
    my_is_valid="$RETURN" 
    
    if [ "$my_is_valid" = "TRUE" ]; then  
        # Restart the EJB partition
        if [ "$my_named" = "$EJBPARTITION_NAME" ]; then            
            retstart_ejb_partition
            STAT=$?
        else 
            restart_partition $my_named 
            STAT=$?
        fi
    else
        echo "Error : Routine restart_named_partition(). Partition $my_named does not exist" && 
        return -1
    fi
      
    return $STAT  
      
}


###################################################################
#
# Function    : restart_partition
#
# Description : Restarts a partition
#
###################################################################

restart_partition()
{
  log "Restarting partition (using stop / start) ..."

  stop_partition $1
  RETCODE=$?

  if [ $RETCODE -eq $ERR_PTN_STOP_FAILED ]; then
     log "Cannot stop the partition"
     return $RETCODE
  fi

  start_partition $1

  return $?
}



###############################################################################
# STOP
###############################################################################

###################################################################
#
# Function    : stop_all
#
# Description : Stop appserver
#
###################################################################

stop_all() {
  integer STAT
  if [ "$DEVELOPER_MODE" = "true" ] ; then
    STAT=$RUNNING
  else
    status_server
    STAT=$?
  fi

# backing up existing Apache core dump files before appserver shutdown 
  backup_core_file

  if [[ $STAT -eq $STOPPED ]] ; then
      sleep 1
  else

     if [[ $STAT -eq $STOPPING ]] ; then
    log "Appserver already stopping"
  else
    STATUS=`iastool -stop $APPSERVER_STD_ARGS < $APPSERVER_AUTH_FILE | tee -a $TMP_FILE | tail -1`
    log "${STATUS##*password: } ..."
  fi

     DELAY_TOTAL=0
     TIMED_OUT=0

  log -n "Waiting for AppServer to stop ..."
  DELAY_TOTAL=0
  TIMED_OUT=0
  # Now loop until it's down - hmm, we don't want to loop forever, though!
  while [[ $STAT -ne $STOPPED ]] ; do
    log -n "."
    let DELAY_TOTAL+=$DELAY
    sleep $DELAY
    [[ $DELAY_TOTAL -ge $STOP_TIMEOUT ]] && TIMED_OUT=1 && break
    status_server SILENT
    STAT=$?
  done

  if [[ $TIMED_OUT -ne 0 ]] ; then
    DEBUG_MODE=true
    log " timed out"
  else
    log " stopped"
  fi

  fi 

  BES_STAT=$(metrica_processadmin.pl -status | egrep '(Apache|Osagent|Partition|Ias)' | grep -v grep)
  if [ -n "$BES_STAT" ]; then
    DEBUG_MODE=true
    log "Stray Appserver processes ... shutting down appserver"    
    metrica_processadmin.pl -status | grep Apache    | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
    metrica_processadmin.pl -status | grep Osagent   | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
    metrica_processadmin.pl -status | grep Partition | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
    metrica_processadmin.pl -status | grep Ias       | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
  fi
# Remove httpd core dump files generated on shutdown of Appserver -workaround for Borland/HP issue
  remove_httpd_core_file
  log " OK"

  return $STOPPED  # OK
}


###################################################################
#
# Function    : stop_partitions
#
# Description : Stops the partitions. Only stops the web 
#               partitions if the EJB (standard) partition has
#               been stoped successfully and if web partitions
#               exist.
#
###################################################################

stop_partitions()
{
         
    # Stop standard partition
    stop_ejb_partition
    STAT=$?
    if [ "$STAT" -ne $STOPPED ]; then      
	  echo "Error : Cannot stop all partitions."
          return $STAT 
    fi   
    
    return $STOPPED    
}

###################################################################
#
# Function    : stop_ejb_partition
#
# Description : This stops the ejb partition but stops the web
#               partitions first.
#
###################################################################

stop_ejb_partition() 
{
    stop_web_partitions
    STAT=$?
    if [ "$STAT" -ne $STOPPED ]; then      
         echo "Error : Cannot stop all web partitions."
         return $STAT 
    fi  

    my_ejb_partitions="`get_ejb_partitions`"  
    for my_partition in $my_ejb_partitions
    do      
        stop_partition $my_partition 
        STAT=$?
        
        if [ "$STAT" -ne $STOPPED ]; then      
	     echo "Error : Cannot stop ejb partition."
             return $STAT 
        fi  
        
        break # should only be one
    done  
    
    return $STAT
}

###################################################################
#
# Function    : stop_web_partitions
#
# Description : Stops web partitions. If one fails to stop 
#               it quits out.
#
###################################################################

stop_web_partitions()
{

    STAT="$STOPPED"

    # Start of web partitions if cloned    
    is_cloned 
    my_is_cloned="$RETURN"   
    if [ "$my_is_cloned" = "TRUE" ]; then  
       my_web_partitions="`get_web_partitions`"      
       for my_partition in $my_web_partitions
       do   
           stop_partition $my_partition 
           STAT=$?
           
           if [ "$STAT" -ne $STOPPED ]; then
                 echo "Error : Cannot stop web partition $my_partition."
	         return $STAT 
           fi  
         
       done  
    fi

    return $STAT
}

###################################################################
#
# Function    : stop_named_partition
#
# Description : Stops a particular named partitions. If the 
#               named partition is the EJB partition then 
#               stop_ejb_partition is called instead.
#
###################################################################

stop_named_partition()
{   
    my_named="$1" 	
    
    is_valid_partition $my_named  
    my_is_valid="$RETURN" 
    
    if [ "$my_is_valid" = "TRUE" ]; then  
        # Stop the EJB partition
        if [ "$my_named" = "$EJBPARTITION_NAME" ]; then            
            stop_ejb_partition
            STAT=$?
        else 
            stop_partition $my_named   
            STAT=$?
        fi
    else
        echo "Error : Routine stop_named_partition(). Partition $my_named does not exist" && 
        return -1
    fi
      
    return $STAT  
      
}



###################################################################
#
# Function    : stop_partition
#
# Description : Stops a partition
#
###################################################################

stop_partition() {
  integer STAT

  PNAME=$1
  HARD_KILL=false

     #
     # Get partition status
     #
     status_partition $PNAME
     STAT=$?

     #
     # Use iastool to stop the partition
     #
     if [[ $STAT -eq $STOPPED ]] ; then
      sleep 1
  else

     if [[ $STAT -eq $STOPPING ]] ; then
         log "Partition $PNAME already stopping"
     else
         STATUS=`iastool -stop $APPSERVER_STD_ARGS -partition $PNAME < $APPSERVER_AUTH_FILE | tee -a $TMP_FILE | tail -1`
         log "${STATUS##*password: } ..."
     fi

     DELAY_TOTAL=0
     TIMED_OUT=0

     # Now loop until it's down - hmm, we don't want to loop forever, though!
     #
     while [[ $STAT -ne $STOPPED ]] ; do
         log -n "."
         let DELAY_TOTAL+=$DELAY
         sleep $DELAY
         [[ $DELAY_TOTAL -ge $STOP_PTN_TIMEOUT ]] && TIMED_OUT=1 && break
         # status_partition $1 SILENT
         status_partition $PNAME SILENT        
         STAT=$?
     done

     if [[ $TIMED_OUT -ne 0 ]] ; then
       DEBUG_MODE=true
       log " timed out"
     else
       log " stopped"
     fi

  fi 
      

  #
  # Make absolutely sure the partition is stopped
  #
  # log -n "Ensuring partition process $PNAME is stopped ... "
  PTN_STAT=$(metrica_processadmin.pl -status | grep Partition | grep $PNAME | grep -v grep)

  #
  # If a partition process still exists, kill the appserver processes
     #
     if [ -n "$PTN_STAT" ]; then
       DEBUG_MODE=true

       #  Does full kill of Appserver processes
    log "Stray $PNAME partition process ... shutting down appserver"
       metrica_processadmin.pl -status | grep Apache    | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
       metrica_processadmin.pl -status | grep Osagent   | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
       metrica_processadmin.pl -status | grep Partition | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE
       metrica_processadmin.pl -status | grep Ias       | metrica_processadmin.pl -kill 2>&1 | cat >> $TMP_FILE


       #
       # Start the appserver fresh
       #
       start_server
    START_STAT=$?

    if [[ $START_STAT -ne $RUNNING ]] ; then
        log "Error, unable to start appserver correctly."
        exit $START_STAT
     fi
    
    HARD_KILL=true

    
  else
     log "Partition $PNAME fully stopped"
  fi


  # Stop all partitions if we had a hard kill
  [ "$HARD_KILL" = "true" ] && stop_partitions

  return $STOPPED  # OK
}



###############################################################################
# STATUS ROUTINES
###############################################################################


###################################################################
#
# Function    : status_ejb_partition
#
# Description : Gets the status of the ejb (standard) partition
#
###################################################################

status_ejb_partition() 
{
    my_ejb_partitions="`get_ejb_partitions`"  
    for my_partition in $my_ejb_partitions
    do      
        status_partition $my_partition 
        STAT=$?
        break # should only be one
    done  
    
    return $STAT
}

###################################################################
#
# Function    : status_web_partitions
#
# Description : Gets status of all web (non-standard) partitions
#               Returns the non-RUNNING status if any of the web
#               partitions are not running
#
###################################################################

status_web_partitions()
{
    RETURN_STAT=$RUNNING

    my_web_partitions="`get_web_partitions`"      
    for my_partition in $my_web_partitions
    do   
        status_partition $my_partition 
        STAT=$?
        
        # Give status of all 
        if [ "$STAT" -ne $RUNNING ]; then
           RETURN_STAT=$STAT
        fi         
                
    done
    
    return $RETURN_STAT
    
}


###################################################################
#
# Function    : status_named_partition
#
# Description : Gets status of a particular named partitions
#
###################################################################

status_named_partition()
{   
    my_named="$1" 	
    
    STAT=$STOPPED
    
    is_valid_partition $my_named  
    my_is_valid="$RETURN" 
    
    if [ "$my_is_valid" = "TRUE" ]; then  
        status_partition $my_named 
        STAT=$?
        [[ $STAT -ne $STOPPED ]] && return $STAT    

    else
        echo "Error : Routine status_named_partition(). Partition $my_named does not exist" && 
        return -1
    fi
    
    return $STAT
        
}

###################################################################
#
# Function    : status_partitions
#
# Description : Gets status of the ejb and web partitions
#
###################################################################

status_partitions()
{   
    # Defaults
    EJB_STAT=$RUNNING
    WEB_STAT=$RUNNING    

    # Get status of standard partition
    status_ejb_partition
    if [ $? -ne $RUNNING ]; then 
       EJB_STAT=$STOPPED
    fi
    
    # Get status of web partitions if cloned    
    is_cloned 
    my_is_cloned="$RETURN"   
    if [ "$my_is_cloned" = "TRUE" ]; then  
        status_web_partitions
        if [ $? -ne $RUNNING ]; then 
           WEB_STAT=$STOPPED
        fi
    fi

    if [[ $EJB_STAT -ne $RUNNING || $WEB_STAT -ne $RUNNING ]]; then      
        return $STOPPED      
    fi
    
    return $RUNNING    
}

###################################################################
#
# Function    : status_all
#
# Description : Gets the appserver status and the status of all 
#               the partitions
#               Only return running if all three are up
#
###################################################################

status_all()
{   

    # Defaults
    SERVER_STAT=$RUNNING
    PART_STAT=$RUNNING  

    status_server
    if [ $? -ne $RUNNING ]; then 
       SERVER_STAT=$STOPPED
    fi    
    
    status_partitions
    if [ $? -ne $RUNNING ]; then 
       PART_STAT=$STOPPED
    fi 
    
    if [[ $SERVER_STAT -ne $RUNNING || $PART_STAT -ne $RUNNING ]]; then  
        return $STOPPED      
    fi   
    
    return $RUNNING
}


###################################################################
#
# Function    : status_server
#
# Description : Get status of appserver
#
###################################################################

status_server()
{
  SILENT=$1
  [[ -z $SILENT ]] && log -n "Checking AppServer status ... "
  STATUS=`iastool -ping $APPSERVER_STD_ARGS < $APPSERVER_AUTH_FILE | tee -a $TMP_FILE | tail -1`

  if [[ ${STATUS%Starting} != "$STATUS" ]] ; then
      [[ -z $SILENT ]] && log "Starting"
      return $STARTING
  elif [[ ${STATUS%Running} != "$STATUS" ]] ; then
      [[ -z $SILENT ]] && log "Running"
      return $RUNNING
  elif [[ ${STATUS%Stopping} != "$STATUS" ]] ; then
      [[ -z $SILENT ]] && log "Stopping"
      return $STOPPING
  else
      [[ -z $SILENT ]] && log "Stopped"
      return $STOPPED
  fi
}


###################################################################
#
# Function    : status_partition
#
# Description : Get status of the ejbpartition
#
###################################################################

status_partition()
{
   
  PNAME=$1 
  SILENT=$2
    
  [[ -z $SILENT ]] && log -n "Checking Status of Partition $PNAME ... "
  STATUS=`iastool -ping $APPSERVER_STD_ARGS -partition $PNAME < $APPSERVER_AUTH_FILE | tee -a $TMP_FILE | tail -1`

  if [[ ${STATUS%Starting} != $STATUS ]]; then
      [[ -z $SILENT ]] && log "Starting"
      return $STARTING
  elif [[ ${STATUS%Running} != $STATUS ]]; then
      [[ -z $SILENT ]] && log "Running"
      return $RUNNING
  elif [[ ${STATUS%Stopped} != "$STATUS" ]] ; then
      [[ -z $SILENT ]] && log "Stopped"
      return $STOPPED
  else
      [[ -z $SILENT ]] && log "Unavailable ... presumed Stopped"
      return $STOPPED
  fi
}




###############################################################################
# Partition Control functionality 
###############################################################################


###################################################################
#
# Function    : is_valid_partition
#
# Description : Checks to see if a parameter is a valid partition
#
###################################################################

is_valid_partition() 
{   
    my_named="$1" 	
    RETURN="FALSE"
    
    my_all_partitions="`get_partitions`"  
    for my_partition in $my_all_partitions
    do      
        if [ "$my_partition" = "$my_named" ]; then     
            RETURN="TRUE"
        fi
    done      
}

###################################################################
#
# Function    : list_services
#
# Description : List information about the current services defined
#               as defined by the SERVICES_PROPERTY property.
#
###################################################################

list_services()
{  
   my_services="`get_property $SERVICES_PROPERTY $PROPERTIES_FILE`"    
   echo $my_services
}

###################################################################
#
# Function    : get_partitions
#
# Description : Check the services to see if it is a partition
#
###################################################################

get_partitions()
{

   my_all_services="`list_services`"	
   
   let count=0 
   for my_service in $my_all_services
   do
 
      # Set up full path to partitions directory     
      my_path="$PARTITIONS_DIR$my_service"
      
      # Check is service a partition
      check_directory $my_path  
      my_is_partition="$RETURN"  
                        
      if [ "$my_is_partition" = "TRUE" ]; then           
         my_partitions[$count]=$my_service
         # Increment counter
         let count=count+1         
      fi
    
   done 
   
   echo "${my_partitions[*]}"
}


###################################################################
#
# Function    : list_partitions
#
# Description : Lists the defined partitions and also finds the
#               max length of the partition name for formatting
#               purposes
#
###################################################################

list_partitions()
{

   # Get Partitions List
   my_partitions="`get_partitions`"
      
   # Get MAX length
   let my_max_title=${#PARTITION_HEADER}
       
   for my_partition in $my_partitions
   do      
      if (( ${#my_partition} > ${#PARTITION_HEADER} )); then
         let my_max_title=${#my_partition}
      fi 
   done
   

   # Print Header
   print_partition_header $my_max_title 
   is_cloned
   my_is_cloned="$RETURN"

   if [ "$my_is_cloned" = "FALSE" ]; then
       STANDARD_SERVICES="$STANDARD_SERVICES, Web Container"
   fi
   
   # Print Partitions
   for my_partition in $my_partitions
   do    
      print_partition $my_max_title $my_partition   
   done

}


###################################################################
#
# Function    : print_partition_header
#
# Description : Prints the header used in -listpartitions in a
#               nice format
#
###################################################################

print_partition_header() 
{
  
   my_max_title="$1" 

   let my_buffer=$my_max_title-${#PARTITION_HEADER}
   my_space_buffer="  "
   
   echo
   let j=0    
   while (( $j < $my_buffer ))
   do
     my_space_buffer="$my_space_buffer "
     let j=j+1
   done   
   echo "$PARTITION_HEADER $my_space_buffer $DETAILS_HEADER"      
  
  
   # Header lines for partition
   let k=0    
   let len=${#PARTITION_HEADER}
   while (( $k < $len ))
   do  
     my_partition_line_buffer="$my_partition_line_buffer-"
     let k=k+1
   done   

   # Header lines for details
   let l=0    
   let len=${#DETAILS_HEADER}  
   while (( $l < $len ))
   do     
     my_details_line_buffer="$my_details_line_buffer-"
     let l=l+1
   done      
   echo "$my_partition_line_buffer $my_space_buffer $my_details_line_buffer"     
     
}


###################################################################
#
# Function    : get_ejb_partitions
#
# Description : Gets all EJB partitions
#
###################################################################

get_ejb_partitions()
{
   
   # Get Partitions List
   my_partitions="`get_partitions`" 
   
   let count=0    
   for my_partition in $my_partitions
   do      
   if [ "$my_partition" = "$EJBPARTITION_NAME" ]; then            
       ejb[$count]=$my_partition 
       let count=count+1
   fi
   done
   
   echo "${ejb[*]}"   
          
}


###################################################################
#
# Function    : get_web_partitions
#
# Description : Gets all WEB partitions
#
###################################################################

get_web_partitions() 
{
   
   # Get Partitions List
   my_partitions="`get_partitions`" 
   
   let count=0    
   for my_partition in $my_partitions
   do      
   if [ "$my_partition" != "$EJBPARTITION_NAME" ]; then            
       web[$count]=$my_partition 
       let count=count+1
   fi
   done
   
   echo "${web[*]}"      
          
}


###################################################################
#
# Function    : print_partition
#
# Description : Prints the partitions in -listpartitions in a
#               nice format
#
###################################################################

print_partition()
{
  
   my_max_title="$1" 
   my_partition="$2"
   my_description=" "
   
   # DEBUG
   # echo "print_partition:$my_partition"

   # Assign a description  
   if [ "$my_partition" = "$EJBPARTITION_NAME" ]; then            
       my_description="$STANDARD_SERVICES"            
   else 
       my_description="$WEB_SERVICES"    
   fi     
  
   # Work out buffering
   let my_buffer=$my_max_title-${#my_partition}  
   my_space_buffer="  "
   let i=0    
   while (( $i < $my_buffer ))
   do
       my_space_buffer="$my_space_buffer "
       let i=i+1
   done   
   
   # Print out formatted output   
   echo "$my_partition $my_space_buffer $my_description"      
   
}

###################################################################
#
# Function    : clone_partition
#
# Description : Clones a webpartition
#
###################################################################
clone_partition()
{

    my_count=$1
    
    validinteger $my_count  
    my_is_valid="$RETURN" 
    
    if [ "$my_is_valid" = "TRUE" ]; then  

       # Stop all
       stop_all
   
       integer CLONE_STAT

       # For all partitions
       let l=0   
       while (( $l < $my_count ))
       do     
          # Call clone script to handle partition cloning
          $CLONE_SCRIPT -clone
          CLONE_STAT=$?
    
          if [[ $CLONE_STAT -eq $SUCCESS ]] ; then
             log "Partition cloned successfully."
          else
             log "Failed to clone partition." && return $CLONE_STAT
          fi
 
          let l=l+1
       done 

       # Start all
       start_all
      
    else 
    
       log "Error clone number $myclonecount is not a valid integer."
       log "Failed to clone partition." && return -1
      
    fi
    
    verify_urimappings
      
    return $CLONE_STAT
   
}  

###################################################################
#
# Function    : drop_partitions
#
# Description : Stops all services before dropping a webpartition.
#               This function validated that it is a valid WEB 
#               partition that was specified.
#
###################################################################
drop_partitions()
{
    # Set default
    my_flag="FALSE"
    RETCODE=0

    # Read Partitions
    let count=0
    while [ ! -z "$1" ]
    do 
       my_partitions[$count]=$1               
       let count=count+1
       shift
    done

    # Validate Partitions
    for my_ptn in ${my_partitions[*]}
    do
        is_valid_partition $my_ptn  
        my_is_valid="$RETURN" 
        
        if [ "$my_is_valid" = "TRUE" ]; then  
        
           # Checking for the EJB partition
	   if [ "$my_named" = "$EJBPARTITION_NAME" ]; then            
	        log "Error : Cannot drop standard partition." && return -1
           fi   
        else
           log "Error : Routine drop_partitions(). Partition $my_ptn does not exist" && 
           return -1
        fi
        my_flag="TRUE"
    done
    
    # All partitions specified are valid   
    if [ "$my_flag" = "TRUE" ]; then
    
        # Stop all
        stop_all
 
        integer DROP_STAT
        
        for my_ptn in ${my_partitions[*]}
        do
        
           # Call clone script to handle partition dropping
           $CLONE_SCRIPT -drop $my_ptn
           DROP_STAT=$?
   
           if [[ $DROP_STAT -eq $SUCCESS ]] ; then
              echo "Partition $my_ptn dropped successfully."
           else
              echo "Failed to drop partition $my_ptn." && RETCODE=1
           fi
        done
    fi     
        
         
    # Start all
    start_all

    # Return 0 if all were successful
    return $RETCODE

}



###############################################################################
# DEPLOYMENT
###############################################################################


###################################################################
#
# Function    : deploy_modules
#
# Description : Deploy comma-separated list of archives to the 
#               AppServer
#
###################################################################

deploy_modules()
{
  # Args 1) Comma-separated list of modules
  #      2) Deployment CLASSPATH 
  [[ -n $2 ]] && CPARG="-cp \"$2\""
  STATUS=$( (iastool -deploy $CPARG -jars $1 $APPSERVER_STD_ARGS -partition $EJBPARTITION_NAME -noverify -nostubs  < $APPSERVER_AUTH_FILE; echo "___$?") | tee -a $TMP_FILE | tail -2 )

  log "Status: ${STATUS%___*} ..."
  verify_urimappings

  return ${STATUS##*___}
}

###################################################################
#
# Function    : undeploy_modules
#
# Description : Remove comma-separated list of archives from the 
#               AppServer
#
###################################################################

undeploy_modules()
{
  # Args 1) Comma-separated list of modules
  RETCODE=0

  # Execute the AppServer undeploy command for each JAR
  for JARFILE in $( echo $1 | tr ',' ' ' ) ; do
      BASEJARFILE=$( basename $JARFILE )
      STATUS=$( iastool -undeploy -jar $BASEJARFILE $APPSERVER_STD_ARGS -partition $EJBPARTITION_NAME < $APPSERVER_AUTH_FILE; echo "___$?" | tee -a $TMP_FILE | tail -2 )

      log "$BASEJARFILE: ${STATUS%___*} ..."

  # Return general error status of 1 if something goes wrong
  [[ ${STATUS##*___} -ne 0 ]] && RETCODE=1
  done
  
  verify_urimappings

  return $RETCODE
}


###################################################################
#
# Function    : update_database_properties
#
# Description : A number of properties contain the database username 
#               and password. This function updates them.
#
###################################################################

update_database_properties()
{
  DBUSERNAME=$1
  DBPASSWORD=$2

  (echo "UPDATE propertyservice_property SET value = '$DBUSERNAME' WHERE property_name IN ('cs_metricaAdminUser','cs_metricaPerformanceUser','cs_metricaSystemUser')" | dbaccess $METRICA_ADMINISTRATION_DB) > /dev/null 2>&1
  (echo "UPDATE propertyservice_property SET value = '$DBPASSWORD' WHERE property_name IN ('cs_metricaAdminPassword','cs_metricaPerformancePassword','cs_metricaSystemPassword')" | dbaccess $METRICA_ADMINISTRATION_DB) > /dev/null 2>&1
}


###################################################################
#
# Function    : create_datasources
#
# Description : Create JNDI defs archive for deployment
#               Construct DAR containing META-INF/jndi-definitions.xml
#               file, constructed from .dsd files in given directory ($1).
#
###################################################################

create_datasources()
{

  DIRECTORY=$1
  DBUSERNAME=$2
  DBPASSWORD=$3
  OUTPUT_PATH=`dirname $4`
  OUTPUT_FILE=`basename $4`
  CURRDIR=`pwd`

  # $DIRECTORY will contain many .dsd files e.g.
  #   MetricaPerformance.dsd:
  #     vendor;prop1;typeA;abc
  #     driver;prop2;typeB;123
  #
  #   MetricaAdministration.dsd:
  #     vendor;prop3;typeC;$INFORMIXDIR
  #     driver;prop4;typeD;$METRICA_PERFORMANCE_DB
  #
  # A file jndi-definitions.xml is produced from these .dsd files. NB
  #   - visitransact-datasource property child-elements are produced from
  #     lines in .dsd beginning with 'vendor', and driver-datasource property
  #     child-elements are produced from lines in .dsd beginning with 'driver'.
  #   - one visitransact-datasource element and one driver-datasource element
  #     is produced per .dsd file.
  #   - all visitransact-datasource elements come before driver-datasource elements.
  #   - DataSource names are produced from name of .dsd file.
  #   - Environment variables in .dsd are evaluated before being added to
  #     jndi-definitions.xml.
  #
  # The jndi-definitions.xml file produced from the above .dsd file examples would be:
  #
  #   <?xml version="1.0"?>
  #   <!DOCTYPE jndi-definitions PUBLIC "-//Borland Corporation//DTD JndiDefinitions//EN" "http://www.borland.com/devsupport/appserver/dtds/jndi-definitions.dtd">
  #
  #   <jndi-definitions>
  #
  #     <!-- All Borland DS defs first, then Informix DS defs, as per DTD -->
  #
  #     <visitransact-datasource>
  #       <jndi-name>serial://datasources/metrica/MetricaPerformance</jndi-name>
  #       <driver-datasource-jndiname>serial://datasources/metrica/DriverMetricaPerformance</driver-datasource-jndiname>
  #       <property>
  #         <prop-name>prop1</prop-name>
  #         <prop-type>typeA</prop-type>
  #         <prop-value>abc</prop-value>
  #       </property>
  #     </visitransact-datasource>
  #     <visitransact-datasource>
  #       <jndi-name>serial://datasources/metrica/MetricaAdministration</jndi-name>
  #       <driver-datasource-jndiname>serial://datasources/metrica/DriverMetricaAdministration</driver-datasource-jndiname>
  #       <property>
  #         <prop-name>prop3</prop-name>
  #         <prop-type>typeC</prop-type>
  #         <prop-value>/apps/informix/ids.2000</prop-value>
  #       </property>
  #     </visitransact-datasource>
  #     <driver-datasource>
  #       <jndi-name>serial://datasources/metrica/DriverPerformance</jndi-name>
  #       <datasource-class-name>com.informix.jdbcx.IfxConnectionPoolDataSource</datasource-class-name>
  #       <log-writer>False</log-writer>
  #       <property>
  #         <prop-name>prop2</prop-name>
  #         <prop-type>typeB</prop-type>
  #         <prop-value>123</prop-value>
  #       </property>
  #     </driver-datasource>
  #     <driver-datasource>
  #       <property>
  #         <prop-name>prop4</prop-name>
  #         <prop-type>typeD</prop-type>
  #         <prop-value>metrica_performance</prop-value>
  #       </property>
  #     </driver-datasource>
  #   </jndi-definitions>
  #
  # jndi-definitions.xml is then placed into a DAR and deployed.
  #

  # JNDI_DEFINITIONS will be the final XML file: jndi-definitions.xml.
  #
  get_tmp_file
  JNDI_DEFINITIONS="$RETURN"

  # VISITRANSACT_DATASOURCE will contain the
  # visitransact-datasource element produced from a .dsd file.
  #
  get_tmp_file
  VISITRANSACT_DATASOURCE="$RETURN"

  # DRIVER_DATASOURCE will contain the
  # driver-datasource element produced from a .dsd file.
  #
  get_tmp_file
  DRIVER_DATASOURCE="$RETURN"

  # VISITRANSACT_DATASOURCE_TMP will be used to contain
  # property elements to be added to VISITRANSACT_DATASOURCE.
  #
  get_tmp_file
  VISITRANSACT_DATASOURCE_TMP="$RETURN"

  # DRIVER_DATASOURCE_TMP will be used to contain
  # property elements to be added to DRIVER_DATASOURCE.
  #
  get_tmp_file
  DRIVER_DATASOURCE_TMP="$RETURN"

  # Some temporary files.
  #
  get_tmp_file
  DSD_TMP="$RETURN"

  get_tmp_file
  DSD_TMP2="$RETURN"

  # A temporary directory.
  #
  get_tmp_dir
  TMPDIR="$RETURN"

  # Add beginning of jndi-definitions.xml and jndi-definitions element.
  #
  cat <<END > $JNDI_DEFINITIONS
<?xml version="1.0"?>
<!DOCTYPE jndi-definitions PUBLIC "-//Borland Corporation//DTD JndiDefinitions//EN" "http://www.borland.com/devsupport/appserver/dtds/jndi-definitions.dtd">

<jndi-definitions>

<!-- All Borland DS defs first, then Informix DS defs, as per DTD -->

END

  cp $DIRECTORY/*.dsd $TMPDIR
  chmod u+w $TMPDIR/*.dsd

  for DSD in `ls $TMPDIR/*.dsd`
  do

    # Add user and password properties to .dsd.
    # NB use single quotes instead of double quotes,
    # so that any $s in $DBUSERNAME and $DBPASSWORD
    # are not later interpreted as environment variables.
    #
    $ECHO 'driver;user;String;$DBUSERNAME' >> $DSD
    $ECHO 'driver;password;String;$DBPASSWORD' >> $DSD

    # Evaluate fields containing environment variables in the .dsd.

    # DSD_TMP is the .dsd file with each
    # line as the argument to an echo command.
    #
    $M_AWK ' { print "echo \"" $0 "\"\n" } ' $DSD > $DSD_TMP

    . $DSD_TMP > $DSD_TMP2
    mv $DSD_TMP2 $DSD_TMP

    # Add beginning of next visitransact-datasource and
    # driver-datasource element for this DataSource.
    #
    cat <<END >> $VISITRANSACT_DATASOURCE
  <visitransact-datasource>
    <jndi-name>serial://datasources/metrica/`basename $DSD .dsd`</jndi-name>
    <driver-datasource-jndiname>serial://datasources/metrica/Driver`basename $DSD .dsd`</driver-datasource-jndiname>
END

    cat <<END >> $DRIVER_DATASOURCE
  <driver-datasource>
    <jndi-name>serial://datasources/metrica/Driver`basename $DSD .dsd`</jndi-name>
    <datasource-class-name>com.informix.jdbcx.IfxConnectionPoolDataSource</datasource-class-name>
    <log-writer>False</log-writer>
END

    # Construct property elements from .dsd file, adding
    # properties from lines beginning with 'vendor' to the
    # visitransact-datasource element, and properties from lines
    # beginning with 'driver' to the driver-datasource element.
    #
    rm $VISITRANSACT_DATASOURCE_TMP
    rm $DRIVER_DATASOURCE_TMP

    $M_AWK -F";" \
      -v vendor="$VISITRANSACT_DATASOURCE_TMP" \
      -v driver="$DRIVER_DATASOURCE_TMP" '

      /^vendor/ {
        print "      <property>" >> vendor
        print "        <prop-name>" $2 "</prop-name>" >> vendor
        print "        <prop-type>" $3 "</prop-type>" >> vendor
        print "        <prop-value>" $4"</prop-value>" >> vendor
        print "      </property>" >> vendor
      }

      /^driver/ {
        print "      <property>" >> driver
        print "        <prop-name>" $2 "</prop-name>" >> driver
        print "        <prop-type>" $3 "</prop-type>" >> driver
        print "        <prop-value>" $4"</prop-value>" >> driver
        print "      </property>" >> driver
      }

    ' $DSD_TMP

    # Add property elements to visitransact-datasource
    # and driver-datasource elements.
    #
    cat $VISITRANSACT_DATASOURCE $VISITRANSACT_DATASOURCE_TMP > $DSD_TMP
    mv $DSD_TMP $VISITRANSACT_DATASOURCE

    cat $DRIVER_DATASOURCE $DRIVER_DATASOURCE_TMP > $DSD_TMP
    mv $DSD_TMP $DRIVER_DATASOURCE

    # Close visitransact-datasource and driver-datasource elements.
    #
    cat <<END >> $VISITRANSACT_DATASOURCE
  </visitransact-datasource>
END

    cat <<END >> $DRIVER_DATASOURCE
  </driver-datasource>
END

  done

  # Add all visitransact-datasource and driver-datasource
  # elements to jndi-definitions element.
  #
  cat $JNDI_DEFINITIONS $VISITRANSACT_DATASOURCE > $DSD_TMP
  mv $DSD_TMP $JNDI_DEFINITIONS

  cat $JNDI_DEFINITIONS $DRIVER_DATASOURCE > $DSD_TMP
  mv $DSD_TMP $JNDI_DEFINITIONS

  # Close jndi-definitions element.
  #
  cat <<END >> $JNDI_DEFINITIONS
</jndi-definitions>
END

  # Put jndi-definitions.xml into a DAR.
  #

  mkdir -p $TMPDIR/META-INF

  mv $JNDI_DEFINITIONS $TMPDIR/META-INF/jndi-definitions.xml

  cd $TMPDIR
  jar cvf $OUTPUT_FILE META-INF

  # Copy DAR to given output file.
  #
  if [ ! -a $OUTPUT_PATH ] ; then
    mkdir -p $OUTPUT_PATH
  fi

  if [ -a $OUTPUT_PATH/$OUTPUT_FILE ] ; then
    rm -rf $OUTPUT_PATH/$OUTPUT_FILE
  fi

  cp $OUTPUT_FILE $OUTPUT_PATH

  # Deploy DAR.
  #
  cd $CURRDIR
  LOG "Deploying metrica-datasources.dar to $EJBPARTITION_NAME on $APPSERVER_NAME..."
  stop_ejb_partition
  RETCODE=$?
  cd $TMPDIR

  if [ $RETCODE -eq $ERR_PTN_STOP_FAILED ]; then
     log "Cannot stop the partition"
     return $RETCODE
  fi

  deploy_modules $OUTPUT_FILE "$METRICA_DIR/lib/ext/ifxjdbc.jar:$METRICA_DIR/lib/ext/ifxjdbcx.jar" $EJBPARTITION_NAME
  start_ejb_partition

  return 0
}


###################################################################
#
# Function    : verify_urimappings
#
# Description : Writes the ware list into the UriMapFile.properties
#               properties file.
#
###################################################################

verify_urimappings()
{

    URI_TMP=$URI_DIR\/$URI_FILE.tmp

    $ECHO "#                                                                 "> $URI_TMP
    $ECHO "# File: urimapping file                                           ">> $URI_TMP
    $ECHO "#                                                                 ">> $URI_TMP
    $ECHO "# Entries in this file map URI's to WebClusters (CORBA instances) ">> $URI_TMP 
    $ECHO "# configured in the WebClusters.properties file.                  ">> $URI_TMP
    $ECHO "#                                                                 ">> $URI_TMP 
    $ECHO "#                                                                 ">> $URI_TMP


    # Get list of wars
    let count=0 
    for x in `ls $WAR_DIR/*.war 2> /dev/null`
    do
        my_war=`basename $x .war`
        $ECHO "/$my_war = cluster1" >> $URI_TMP
        $ECHO "/$my_war/* = cluster1" >>$URI_TMP
    done

    my_stamp=`date '+%d%b%Y_%H:%M'`
    mv $URI_DIR\/$URI_FILE $URI_DIR\/$URI_FILE\.$my_stamp
    mv $URI_TMP $URI_DIR\/$URI_FILE
        
}

###################################################################
#
# Function    :	backup_core_file 
#
# Description : function to back up core files in apache directory
#
###################################################################

backup_core_file()
{
    # check 2 different directories for core dump files an backup if they exist
    if [ "$OS_TYPE" = "HP-UX" ]; then
        if [ -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/core ]; then
            mv $APPSERVER_DIR/var/servers/$APPSERVER_NAME/core $APPSERVER_DIR/var/servers/$APPSERVER_NAME/core.backup 2> /dev/null
            log "Unix core found in the following location: $APPSERVER_DIR/var/servers/$APPSERVER_NAME .It has been renamed to ensure it is not overwritten."
	fi
	if [ -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/core ]; then
            mv $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/core $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/core.backup  2> /dev/null
            log "Unix core found in the following location: $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2 .It has been renamed to ensure it is not overwritten."
	fi
fi

}

###################################################################
#
# Function    : remove_httpd_core_file
#
# Description : Function to remove core dump files generated by httpd
#		This is a tidy-up for a Borland/HP bug in which core
#		dump files are generated by Apache on shutdown
#
###################################################################

remove_httpd_core_file()
{
# If OS is HP-UX check for any BES apache generated core dump files and remove
    if [ "$OS_TYPE" = "HP-UX" ]; then
        REMOVE_FLAG=0
        if [ -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/core ]; then
                PROCESSNAME_CORE=`file $APPSERVER_DIR/var/servers/$APPSERVER_NAME/core | awk {'print $5'}`
                if [ "$PROCESSNAME_CORE" = "'httpd'" ]; then
                        rm -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/core  2> /dev/null
			REMOVE_FLAG=1
                fi
        fi
        if [ -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/core ]; then
                PROCESSNAME_CORE=`file $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/core | awk {'print $5'}`
                if [ "$PROCESSNAME_CORE" = "'httpd'" ]; then
                      rm -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/apache2/core  2> /dev/null
		      REMOVE_FLAG=1
                fi
        fi
        if [ $REMOVE_FLAG -eq 1 ]; then
                log "Removing temporary files created during system shutdown"
        fi
   fi
}

###############################################################################
# GENERAL
###############################################################################


###################################################################
#
# Function    : usage
#
# Description : Show correct command-line syntax for -help
#
###################################################################

usage()
{
  [[ $1 = "ERROR" ]] && $ECHO -n "Error: "

  $ECHO ""  
  $ECHO "Usage: $PROGNAME {                                               "
  $ECHO "       -h | -help | --help                                       "
  $ECHO "       -v | -version | --version                                 "
  $ECHO "} | {                                                            "
  $ECHO "       {-start | -stop  | -restart | -status}                    "  
  $ECHO "       [ EJB | WEB | PARTITIONS | ALL | <partition> ]            "  
  $ECHO "} | {                                                            "
  $ECHO "       -listpartitions                                           "   
  $ECHO "} | {                                                            "
  $ECHO "       -clone [<count>]                                          "   
  $ECHO "} | {                                                            "
  $ECHO "       -drop <partitions>                                         "     
  $ECHO "} | {                                                            "
  $ECHO "       -create_ds <directory> -dbusername <username> -dbpassword <password> -o <file>"
  $ECHO "}"
  $ECHO ""  

  # Exit with defined error code if incorrect syntax
  # Otherwise, user asked for help, so exit normally
  [[ $1 = "ERROR" ]] && exit $ERR_USAGE
}


###################################################################
#
# Function    : longhelp
#
# Description : Show detailed command-line usage
#
###################################################################

longhelp()
{
  $ECHO ""
  $ECHO "Options:"
  $ECHO ""  
  $ECHO "      -h | -help | --help         Display help information        "
  $ECHO "      -v | -version | --version   Display version information     "
  $ECHO "      -start                      Starts appserver or partitions  "
  $ECHO "      -stop                       Stops appserver or partitions   " 
  $ECHO "      -restart                    Restarts appserver or partitions"
  $ECHO "      -status                     Status of appserver or partitions"
  $ECHO "" 
  $ECHO "                   EJB          - The standard partition          "
  $ECHO "                   WEB          - All web partitions              "
  $ECHO "                   PARTITIONS   - The EJB and WEB partitions      "
  $ECHO "                   ALL          - The Appserver and all partitions"   
  $ECHO "                   <partition>  - A particular partition          "
  $ECHO "" 
  $ECHO "      -listpartitions             Lists all the partitions.       " 
  $ECHO "      -clone [<count>]            Clones web partitions           "   
  $ECHO "      -drop <partitions>          Drops web partitions            "   
  $ECHO "                                  Multiple web partitions can be  "  
  $ECHO "                                  specified by giving a space delimited"   
  $ECHO "                                  list, -drop web1 web2 web3      "  
  $ECHO "      -create_ds <directory> -dbusername <username> -dbpassword <password> -o <file>"
  $ECHO "                                  Update Deployed Datasources     "
  $ECHO ""
}

###################################################################
#
# Function    : version
#
# Description : Show version information
#
###################################################################

version()
{
  if [ -f $METRICA_DIR/Installer/bin/metrica_version.ksh ]; then
    . $METRICA_DIR/Installer/bin/metrica_version.ksh
    print_version "$PROGNAME" "SysManagement_r4.79.0.9:cset.008500:38:38"
  else
    $ECHO "SysManagement_r4.79.0.9"
  fi
}



###############################################################################
# SCRIPT MAIN
###############################################################################
#
# Start the script
#

trap cleanup 0

[[ $# -eq 0 ]] && usage ERROR

while [[ $# -gt 0 ]]; do
  case $1 in
      "-start") # start option
	  if [ "`echo $2 | cut -c1`" = '-' -o $# -lt 2 ] ; then
	      start_all
	  else
	      shift
	      case "$1" in
		  EJB) start_ejb_partition ;;
		  WEB) start_web_partitions ;;
		  SVR) start_server ;;
		  PARTITIONS) start_partitions;;
		  ALL) start_all ;;
                  *) start_named_partition $1 ;;
	      esac
	  fi
          RETCODE=$?
          [ $RETCODE -ne 0 ] && DEBUG_MODE=true
	  exit $RETCODE
	  ;;
      "-stop") # stop option
	  if [ "`echo $2 | cut -c1`" = '-' -o $# -lt 2 ] ; then
	      stop_all
	  else
	      shift
	      case "$1" in
                EJB)
                  stop_ejb_partition
                  RETCODE=$?

                  if [ $RETCODE -eq $ERR_PTN_STOP_FAILED ]; then
                     log "Cannot stop the partition"
                  fi
                  ;;

                WEB)
                  stop_web_partitions
                  RETCODE=$?

                  if [ $RETCODE -eq $ERR_PTN_STOP_FAILED ]; then
                     log "Cannot stop the web partitions"
                  fi
                  ;;
                PARTITIONS)
                  stop_partitions
                  RETCODE=$?

                  if [ $RETCODE -eq $ERR_PTN_STOP_FAILED ]; then
                     log "Cannot stop the partitions"
                  fi
                  ;;
                SVR) 
		  stop_all
                  RETCODE=$?
	       	;;
                ALL) 
		  stop_all
                  RETCODE=$?
	       	;;
		*) stop_named_partition $1 
		  RETCODE=$?
		  
		  if [ $RETCODE -eq $ERR_PTN_STOP_FAILED ]; then
		     log "Cannot stop the partition"
                  fi
                ;;
	      esac
	  fi
	  exit $RETCODE
	  ;;
      "-restart") # restart option
	  if [ "`echo $2 | cut -c1`" = '-' -o $# -lt 2 ] ; then
	      restart_all
	  else
	      shift
	      case "$1" in
		  EJB) restart_ejb_partition ;;
		  WEB) restart_web_partitions ;;
		  SVR) restart_server ;;
		  PARTITIONS) restart_partitions ;;		  		  
		  ALL) restart_all ;;
		  *) restart_named_partition $1 ;;
	      esac
	  fi
          RETCODE=$?
          [ $RETCODE -ne 0 ] && DEBUG_MODE=true
	  exit $RETCODE
	  ;;
      "-status") # print appserver status
	  if [ "`echo $2 | cut -c1`" = '-' -o $# -lt 2 ] ; then
	      status_all
	      #
	      # BUG-39555
	      # ---------
	      # When appserver is down iastool throws an exception
	      # whose return code should not be returned to the Start/Stop
	      # service
	      #
	      exit $SUCCESS 	      
	  else
	      shift
	      case "$1" in
		  EJB) status_ejb_partition ;;
		  WEB) status_web_partitions ;;
		  SVR) status_server ;;
		  PARTITIONS) status_partitions ;;		  
		  ALL) status_all ;;
		  *) status_named_partition $1 ;;
	      esac
	  fi
	  exit $? 
	  ;;
      "-create_ds")
	  DIRECTORY=$2
	  shift 2

	  # Remaining arguments are expected to be something like:
	  #  -dbusername metrica -dbpassword metrica123
	  #  -o /apps/metrica/DataSources/data/metrica-datasources.dar

	  while [[ $# -gt 0 ]]; do
	      case $1 in
		  "-dbusername") DBUSERNAME=$2;;
		  "-dbpassword") DBPASSWORD=$2;;
		  "-o") OUTPUT=$2;;
		  *) usage ERROR;;
	      esac
	      shift 2
	  done

	  #
	  # BUG-39905
	  # ---------
	  # Check that the appserver is running before
	  # attempting to deploy anything
	  #
	  status_server
	  if [ $? -ne $RUNNING ]; then
             $ECHO "Appserver is not running"
	     exit $ERR_NOAPPSERVER
          fi

	  create_datasources $DIRECTORY $DBUSERNAME $DBPASSWORD $OUTPUT

	  if [ $? -ne 0 ]; then
             $ECHO "Cannot create and deploy datasources"
	     exit $?
          fi

	  update_database_properties $DBUSERNAME $DBPASSWORD

	  exit $?
	  ;;
      "-deploy") # Deploy a module or modules (comma-separated) to
	  shift	 # the relevant partition(s) of the AppServer
		 #  $1 - File list (comma-separated)
		 #  $2 - Classpath
	  [[ $# -lt 2 ]] && usage ERROR
		 deploy_modules $1 "$2" 
      RETCODE=$?
      [ $RETCODE -ne 0 ] && DEBUG_MODE=true
	  exit $RETCODE
	  ;;
      "-undeploy") # Remove a module or modules (comma-separated) from 
	  shift	   # the relevant partition(s) of the AppServer
		   #  $1 - File list (comma-separated)
	  [[ $# -lt 1 ]] && usage ERROR
		  undeploy_modules $1

      RETCODE=$?
      [ $RETCODE -ne 0 ] && DEBUG_MODE=true
	  exit $RETCODE
	  ;;  
      "-listpartitions") # List the partitions defined 
	  list_partitions
          RETCODE=$?
	  exit $RETCODE	  
	  ;;        
      "-clone") # List the partitions defined 
	  if [ "`echo $2 | cut -c1`" = '-' -o $# -gt 2 ] ; then
	      usage ERROR
	  elif [ "`echo $2 | cut -c1`" = '-' -o $# -eq 1 ] ; then     
	      clone_partition 1
	      exit $?	
	  elif [ "`echo $2 | cut -c1`" = '-' -o $# -eq 2 ] ; then     
	      clone_partition $2
	      exit $?	   
	  fi
	  ;;  
      "-drop") # Drops the specified partition
	  if [ "`echo $2 | cut -c1`" = '-' -o $# -lt 2 ] ; then
	      echo "No partition specified"
	      usage ERROR
	  else   
	      let count=0
              while [ ! -z "$2" ]
              do
                  drops_array[$count]=$2               
                  let count=count+1
                  shift
              done
              drop_partitions ${drops_array[@]}
              exit $?
	  fi
          ;;	  
      "-v"|"-version"|"--version") # print version of this script
	  version
	  ;;
      "-h"|"-help"|"--help") # Print detailed usage
          version
	  usage NORMAL
	  longhelp
	  ;;
      "-dev") # Hidden Param - skips status test - USE WITH CARE!
	  DEVELOPER_MODE=true
	  ;;
      "-debug") # Hidden Param - leaves tmp file for perusal
	  DEBUG_MODE=true
	  ;;
      *) # Default to print usage
	  usage ERROR
	  ;;
  esac
  shift
done

# Default exit code of 0 (success)
exit 0
