#**************************************************************************
# @(#) SysManagement/delivery/unix/bin/metrica_lap.ksh SysManagement_r4.79.0.9:cset.008506:4:4 09/28/07 @(#)
#
# Filename: metrica_lap.ksh 
#
#                  COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
# Copyright  2005 Vallent Software Systems UK. All rights reserved 
#
#
# Description:
# -------------------------------
#  This script runs the IBM LAP tool to prompt the user to accept
#  the product license if they have not accepted it before.
#
#
# Parameters:
# -------------------------------
#  -logfile:  The file to log the output to, as well as logging to stdout.
#
#  -silent:   Specify that LAP should run in silent mode, meaning the user is
#             not prompted to accept the license; it is silently accepted on
#             their behalf.
#
#
# Exit Value:
# -------------------------------
#  0 - returned if the user has accepted the license.
#  1 - returned if the user has not accepted the license, or an error occurred.
#
#**************************************************************************

#
# Assign aliases for the external programs we will use internally
#
_PRINTF=/usr/bin/printf
_DATE=/usr/bin/date
_LN=/usr/bin/ln
_RM=/usr/bin/rm

#
# Assign aliases for the external programs we will use internally
#
DIRNAME=$(dirname $0)

#
# Retrieve the MPM release number
#
if [ -f ${DIRNAME}/metrica_release.ksh ]; then
    . ${DIRNAME}/metrica_release.ksh
else
    lap_exit "Could not find file ${DIRNAME}/metrica_release.ksh"
fi

[ -z "${MPM_LICENSE_RELEASE}" ] && lap_exit "MPM_LICENSE_RELEASE is not set"

#
# Set the LAP variables and make them read-only
#
typeset -r LAP_DIR=${DIRNAME}/lap
typeset -r LAP_CLASSPATH=${LAP_DIR}/lapapp.jar:${METRICA_DIR}/Installer/bin/lap/lapapp.jar
typeset -r LAP_LICENSE_DIR=${DIRNAME}/license
typeset -r LAP_STATUS_DIR=${METRICA_DIR}/etc/license/${MPM_LICENSE_RELEASE}
typeset -r LAP_IBM_STATUS_DIR=${LAP_STATUS_DIR}/license
typeset -r METRICA_LICENSE_DIR=${METRICA_DIR}/license

#---------------------------------------------------------------
# Print a DEBUG message
#---------------------------------------------------------------
lap_print_debug()
{
    if [ -n "${M_DEBUG}" ]; then
        lap_print_msg "DEBUG" "$1"
    fi
}

#---------------------------------------------------------------
# Print an INFO message
#---------------------------------------------------------------
lap_print_info()
{
    lap_print_msg "INFO" "$1"
}

#---------------------------------------------------------------
# Exit with an error code and print a FATAL message
#---------------------------------------------------------------
lap_exit()
{
    lap_print_msg "FATAL" "$1"
    exit 1
}

#---------------------------------------------------------------
# Internal print message
#---------------------------------------------------------------
lap_print_msg()
{
    LOG_LEVEL=$1
    MESSAGE=$2
    ${_PRINTF} "[%s] %s: %s\n" "`${_DATE}`" "${LOG_LEVEL}" "${MESSAGE}"
    if [ ! -z "${LAP_LOG}" ]; then
        ${_PRINTF} "[%s] %s: %s\n" "`${_DATE}`" "${LOG_LEVEL}" "${MESSAGE}" >> ${LAP_LOG}
    fi
}

#---------------------------------------------------------------
# Executes the License Acceptance Process Tool.
# If the user does not accept the license, then installation must cease.
#
# Passing '-silent' as a parameter to this function will launch LAP in silent mode.
#---------------------------------------------------------------
execute_lap()
{
    LAP_SILENT_PARAM=
    [ "$1" = "-silent" ] && LAP_SILENT_PARAM="-t 5"

    [ -z "${METRICA_DIR}" ] && lap_exit "METRICA_DIR is not set"
    [ ! -d "${METRICA_DIR}" ] && lap_exit "METRICA_DIR ${METRICA_DIR} is not a directory"

    # Source the JAVA environment script
    [ ! -f ${METRICA_DIR}/setup_java.ksh ] && lap_exit 1 "${METRICA_DIR}/setup_java.ksh not found"
    . ${METRICA_DIR}/setup_java.ksh

    # Ensure that JAVA_HOME is set
    [ -z "${JAVA_HOME}" ] && lap_exit 1 "JAVA_HOME is not set"
    lap_print_debug "LAP: JAVA_HOME set to ${JAVA_HOME}"

    # Point to the JVM
    LAP_JAVA_VM=${JAVA_HOME}/bin/java

    # Ensure that the JVM exists and is executable
    [ ! -f ${LAP_JAVA_VM} ] && lap_exit 1 "Could not find JVM in ${JAVA_HOME}"
    [ ! -x ${LAP_JAVA_VM} ] && lap_exit 1 "JVM ${LAP_JAVA_VM} is not executable"
    lap_print_debug "LAP: Using JVM ${JAVA}"

    # Execute LAP
    ${LAP_JAVA_VM} -classpath ${LAP_CLASSPATH} com.ibm.lex.lapapp.LAP -l ${LAP_LICENSE_DIR} -s ${LAP_STATUS_DIR} -text_only ${LAP_SILENT_PARAM}
    LAP_STATUS=$?
    lap_print_debug "LAP: Return value of LAP was ${LAP_STATUS} (9 is accepted, 3 is declined)"

    # Exit if the license was not accepted
    [ ${LAP_STATUS} -ne 9 ] && lap_exit "License was not accepted (${LAP_STATUS})"

    lap_print_info "LAP: License is accepted"

    # When we have finished, create a link to the new license directory
    # This is performed so the user documentation can state one location
    # there the licenses are stored and don't need to be updated each release.
    create_symbolic_link ${LAP_IBM_STATUS_DIR} ${METRICA_LICENSE_DIR}
}

#---------------------------------------------------------------
# Creates a symbolic link between 2 files/directories
#
# First parameter is the the source (existing entity)
# Second parameter is the the target (link to create)
#---------------------------------------------------------------
create_symbolic_link()
{
    lap_print_debug "LAP: creating symbolic link, source [$1] target[$2]"
    if [ -d $2 ]; then
        lap_print_debug "LAP: Removing existing link target $2"
	${_RM} $2
    fi
    ${_LN} -sf $1 $2
}


#---------------------------------------------------------------
# 
#                 M A I N   P R O G R A M
# 
#---------------------------------------------------------------
while [[ -n $1 ]]
do 
    case "$1" in 

        # logfile option
        "-logfile") 
            LAP_LOG=$2
            shift 
            if [ -z "$LAP_LOG" ]; then 
                lap_exit "must specify a file after -logfile option"
            fi 
        ;; 

        # silent option
        "-silent") 
            LAP_EXTRA_PARAM="-silent"
        ;; 

    esac 
    shift 
done 

execute_lap ${LAP_EXTRA_PARAM}

exit 0
