#!/usr/bin/env perl

############################################################
#
# 3rd party packages
#
use strict;
use Getopt::Long;
use Pod::Usage;
use FileHandle;

############################################################
#
# Global variables
#
my $FUNCTION       = "Metrica Process Administration";
my $COPYRIGHT      = "Copyright  2000-2005 Vallent Software Systems";
my $REPOSITORY_STR = "SysManagement_r4.79.0.9:cset.006333:18:18";  # This value is set by repository

my $helpf; 
my $versionf; 
my $manf; 
my $statusf; 
my $stopf; 
my $killf; 
my $catchallf;

my $platform;
my $platform_arch ;
my $uname;


############################################################
#
# Global objects
#
my $PROCESS_TABLE = new MetricaProcessTable($ENV{LOGNAME});
my $VERSION       = new MetricaVersion($REPOSITORY_STR);

############################################################
#
# Main Program
#

#
# Check for environment variables
#
die "LOGNAME undefined"       unless defined($ENV{LOGNAME});
die "APPSERVER_DIR undefined" unless defined($ENV{APPSERVER_DIR});
die "METRICA_DIR undefined"   unless defined($ENV{METRICA_DIR});

#
# Option processing
#
GetOptions('help' => \$helpf, 
           'version' => \$versionf, 
           'man' => \$manf, 
           'status' => \$statusf, 
           'stop' => \$stopf, 
           'kill' => \$killf, 
           'preparetostop|start|restart' => \$catchallf) 
   or pod2usage(-exitval => 2);

#
# Option handling 
#
if (@ARGV != 0) {
   pod2usage(-exitval => 2, -message => "Invalid parameter.");
} 
elsif ($statusf) {
   $PROCESS_TABLE->status;
} 
elsif ($stopf) {
   $PROCESS_TABLE->stop->status;
} 
elsif ($killf) {
   $PROCESS_TABLE->kill->status;
} 
elsif ($helpf) {
   pod2usage(-exitval => 0, -verbose => 1);
} 
elsif ($manf) {
   pod2usage(-exitval => 0, -verbose => 2);
} 
elsif ($versionf) {
   printf "%s\n%s\n%s\n", $FUNCTION, $COPYRIGHT, $VERSION->version;
} 
elsif ($catchallf) {
   pod2usage(-exitval => 1, -message => "Start/Stop parameter not implemented");
} 
else {
   pod2usage(-exitval => 2, -message => "Invalid parameter.");
}

#
# Cleanup
#
exit 0;

############################################################
#
# MetricaVersion package
#
package MetricaVersion;

#-----------------------------------------------------------
#
# constructor method
#
# Parameters:
#  $repositoryStr : This is the string returned by the repository 
#                   SysManagement_r4.79.0.9:cset.006333:18:18 variable
#
# Returns:
#  Object reference
#
sub new {
   my $class = shift;
   my ($repositoryStr) = @_;
   my $self = {}; bless $self, $class;

   #
   # Translation hash for the package state
   #
   my %state;
   $state{r} = "Released product";
   $state{d} = "Developement";
   $state{a} = "Developement testing";
   $state{b} = "Functional testing";

   #
   # Analyse the version string from the repository
   #  SysManagement_r4.0.0.latest:cset.004216:5:5
   #
   if ($repositoryStr =~ /(\w+)_(\w)(\d+)\.(\d+)\.(\d+)\.([^:]+):/) {
      $self->{subsystem} = $1;
      $self->{state}     = $state{$2};
      $self->{major}     = $3;
      $self->{minor}     = $4;
      $self->{patch}     = $5;
      $self->{checkpt}   = $6;
   } 
   else {
      $self->{invalidStr} = $repositoryStr;
   }

   return $self;
}

#-----------------------------------------------------------
#
# version method
#
# Parameters:
#  none
#
# Returns:
#  Correctly formatted version string
#
sub version {
   my $self = shift;

   if (!defined($self->{invalidStr})) {
      return sprintf "%s Version %s.%s Patch %s (Checkpoint: %s, State: %s)", $self->{subsystem}, $self->{major}, $self->{minor}, $self->{patch}, $self->{checkpt}, $self->{state}; 
   }
   else {
      die "Version unknown :  $self->{invalidStr}";
   }
}

############################################################
#
# MetricaProcessTable package
#
package MetricaProcessTable;

#-----------------------------------------------------------
#
# constructor method
#
# Parameters:
#  $owner - This is the userid whose processes are printed 
#           using the "ps" command
#
# Returns:
#  Object reference
#
sub new {
   my $class = shift;
   my ($owner) = @_;
   
   my $self = {	
   	'hp_ps'   => \&hp_ps,	
   	'svr5_ps' => \&svr5_ps,	
   	'bsd_ps'  => \&bsd_ps	
   }; 
   
   
   bless $self, $class;
   $self->{owner} = $owner;   
   
   my $appserverdir_len = 0;
   my $appservername_len = 0; 
   my $danger_flag = 0;
   my $unix_path_length = 35;

   #
   # APPSERVER_DIR
   #
   if ((!defined $ENV{APPSERVER_DIR})||($ENV{APPSERVER_DIR} eq "")) {
      $danger_flag=1; 
   }
   else {
      my $dir=$ENV{APPSERVER_DIR};
      $appserverdir_len=length($dir);
   }

   #
   # APPSERVER_NAME
   #
   if ((!defined $ENV{APPSERVER_NAME})||($ENV{APPSERVER_NAME} eq "")) {
      $danger_flag=1; 
   }
   else {
      my $name=$ENV{APPSERVER_NAME};
      $appservername_len=length($name);
   }
   
   # Total Length
   my $total_length = ($appserverdir_len + $appservername_len);
   
   # The max length of the combined $APPSERVER_DIR and $APPSERVER_NAME path needs to be 35 so the partition
   # name is in the path outputted using /usr/bin/ps on Sun 
   
   if ($total_length > $unix_path_length ) {
      $danger_flag=1;    
   }
   
   #
   # Run the uname command
   #   
   my $uname = `uname -a`;
   die "metrica_processadmin.ksh : Error running uname command" if $?;
   
   chomp $uname;
   if ($uname =~ "SunOS") {

       # If ucb/ps is NOT defined and path is too long -> EXIT
       if (!(-x "/usr/ucb/ps") && ($danger_flag eq "1")) {
              print "Fatal Error : \n";
              print "The unix command /usr/ucb/ps is not present and\n";
              print "the path lengths are dangerously long. Please have\n";
              print "a unix amdinistrator to install /usr/ucb/ps on this\n";
              print "Solaris machine.\n";
              exit ;
       }
       # If ucb/ps is NOT defined and path NOT too long ->  use bin/ps 
       elsif (!(-x "/usr/ucb/ps") && ($danger_flag eq "0")) {
                $self->{ps} = "svr5_ps";
       }
       # Otherwise -> use ucb/ps       
       else{
             $self->{ps} = "bsd_ps";
       }
   }
   elsif ($uname =~ "HP-UX") {
       $self->{ps} = "hp_ps";
   }
   elsif ($uname =~ "Linux") {
       $self->{ps} = "svr5_ps";
   }
   else {
      $self->{ps} = "svr5_ps";
   }   
   return $self;
}

#-----------------------------------------------------------
#
# process_table method
#
# Parameters:
#  none
#
# Returns:
#  An array of hash references is returned. 
#  See the result of the ps method for more details.
#
# Notes:
#  This method retrieves a filtered list of metrica processes
#  The identification of the various subsystems and the 
#  kill signals are configured here.
#
sub process_table {
   my $self = shift;
   my @listing = ();

   #
   # Start/Stop services (These should not be killed by this script)
   #
   push @listing, $self->filter_ps("COMMAND", ["rmiregistry|".$ENV{METRICA_DIR}."/StartStop"], 
                     {SUBSYSTEM => "Start/Stop", KILL_SIGNAL => "TERM"});

   #
   # IAS 
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{APPSERVER_DIR}."/bin/ias"], 
                     {SUBSYSTEM => "Ias", KILL_SIGNAL => "9"});

   #
   # Partition 
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{APPSERVER_DIR}."/bin/partition"], 
                     {SUBSYSTEM => "Partition", KILL_SIGNAL => "9"});

   #
   # Osagent 
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{APPSERVER_DIR}."/bin/osagent"], 
                     {SUBSYSTEM => "Osagent", KILL_SIGNAL => "USR2"});                   
                     
   #
   # Apache 
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{APPSERVER_DIR}."/bin/apache2"], 
                     {SUBSYSTEM => "Apache", KILL_SIGNAL => "USR2"});                     

   #
   # Common services
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/CS"], 
                     {SUBSYSTEM => "Common Services",  KILL_SIGNAL => "TERM"});

   #
   # DB Services
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/DBServices"], 
                     {SUBSYSTEM => "DB Services", KILL_SIGNAL => "TERM"});

   #
   # Reporting
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/Reporting"], 
                     {SUBSYSTEM => "Reporting", KILL_SIGNAL => "TERM"});

   #
   # Grouping Service
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/GroupingService"], 
                     {SUBSYSTEM => "Grouping Service", KILL_SIGNAL => "TERM"});

   #
   # Alarm Management
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/AlarmManagement"], 
                     {SUBSYSTEM => "Alarm Management", KILL_SIGNAL => "TERM"});

   #
   # Loaders
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/DBLoader|metrica_load"], 
                     {SUBSYSTEM => "DB Loader", KILL_SIGNAL => "TERM"});

   #
   # System Management
   #
   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/Installer"], 
                     {SUBSYSTEM => "System Management", KILL_SIGNAL => "TERM"});

   push @listing, $self->filter_ps("COMMAND", [$ENV{METRICA_DIR}."/utils"], 
                     {SUBSYSTEM => "System Management", KILL_SIGNAL => "TERM"});

   #
   # Unknown processes (These should be displayed but not stopped)
   #
   push @listing, $self->filter_ps("COMMAND", ["java"], 
                     {SUBSYSTEM => "UNKNOWN", KILL_SIGNAL => "KILL"});

   $self->remove_duplicates("PID", @listing);
}

#-----------------------------------------------------------
#
# filter_ps method
#
# Parameters:
#  $field      - The process hash key that filtering is applied to. 
#                Is normally the "COMMAND" key
#  $filtersRef - Reference to a list of filters (regular expression strings)
#  $presetRef  - Reference to a hash of preset process hash values.
#                This parameter is passed transparently to the ps method.
#  %param      - Additional parameters used by this method
#                $param{children} : boolean value controlling if child
#                                   processes are also retrieved
#
# Returns:
#  An array of hash references is returned. 
#  See the result of the ps method for more details.
#
# Notes:
#  The process_table method does not call the ps method directly.
#  Instead it uses this method to filter the results.
#
sub filter_ps {
   my $self = shift;
   my ($field, $filtersRef, $presetRef, %param) = @_;
   my @filters = @{$filtersRef};
   my %preset = %{$presetRef};
   my @listing = ();
   my $pattern="-xism:";
   my $array_count;
    
   $param{children} = 1 unless defined($param{children});

   # Point to the function as specified by $self->{ps}
   my $process = $self->{$self->{ps}}; 
	 my $numelements = @filters;
	 foreach (@filters) {
            delete @filters[$array_count] if /$pattern/;
            $array_count=$array_count+1;
	}
	$numelements = @filters;
if ($numelements ne "0"){
   foreach my $row ( $self->$process(%preset) ) {
      my $okay = 1;
      push @listing, $row;

      #
      # Apply filters
      #
      foreach my $filter ( @filters ) {
         if (!($row->{$field} =~ /$filter/)) {
            pop @listing;
            $okay = 0;
            last;
         }
      }


      #
      # For all non appserver processes
      #
      if (!($row->{COMMAND} =~ $ENV{APPSERVER_DIR})) {

      #
      # Add child processes
      #
      if ($okay && $param{children}) {

	 #
	 # Recursively check the child processes
	 #
         push @listing, $self->filter_ps("PPID", [qr/^$row->{PID}$/], $presetRef);
         }
      }
   }
   }

   return $self->remove_duplicates("PID", @listing);
}

#-----------------------------------------------------------
#
# remove_duplicates method
#
# Parameters:
#  $uniqueKey - The hash key that makes the hash reference
#               array unique
#  @hashList  - An array of hash references 
#
# Returns:
#  An array of hash references is returned with duplicate
#  hashes removed.
#
# Notes:
#  1) The first unique hash is kept, all other hashes containing
#     duplicate values of the unique key are discarded.
#  2) The returned array is sorted based on the value of the unique key
#
sub remove_duplicates {
   my $self = shift;
   my ($uniqueKey, @hashList) = @_;
   my %uniqueHash;
   my @uniqueList = ();

   foreach my $row ( reverse @hashList ) {
      $uniqueHash{$row->{$uniqueKey}} = $row;
   }

   foreach my $key ( sort(keys %uniqueHash) ) {
      push @uniqueList, $uniqueHash{$key};
   }

   return @uniqueList;
}

#-----------------------------------------------------------
#
# svr5_ps method
#
# Parameters:
#  %preset - This hash is used to define preset values
#            in the returned process hash list.
#            Typically used to define the SUBSYSTEM and 
#            KILL_SIGNAL hash values.
#
# Returns:
#  An array of hash references is returned. 
#  Each array element represents a metrica process and has 
#  the following hash keys:
#
#   $element->{UID}
#   $element->{PID}
#   $element->{PPID}
#   $element->{SUBSYSTEM}
#   $element->{COMMAND}
#   $element->{KILL_SIGNAL}
#
# Notes:
#  It is assumed that all metrica processes have been started
#  via the Start/Stop function. This is important because the
#  command pathname is being used to identify the various
#  processes.
#
#  This method utilises the default SVR5 program "/usr/bin/ps".
#  A restriction this imposes on our script is a limit on the
#  character width of the CMD field.
#
sub svr5_ps {
   my $self = shift;
   my (%preset) = @_;
   my @listing = ();
   #
   # Run the ps command
   #
   my @ps = `ps -u $self->{owner} -f`;
   die "metrica_processadmin.ksh : Error running ps command" if $?;

   #
   # Parse the command result
   #
   foreach my $line ( @ps ) {
      my %data;
      my @row = split(/\s+/, $line);

      $data{UID}         = $row[1];
      $data{PID}         = $row[2];
      $data{PPID}        = $row[3];
      $data{SUBSYSTEM}   = "UNKNOWN";
      $data{KILL_SIGNAL} = "TERM";

      if ($row[7] eq "?") {
         $data{COMMAND} = join(" ", @row[9 .. $#row]);
      } 
      else {
         $data{COMMAND} = join(" ", @row[8 .. $#row]);
      }

      #
      # Preset column values (Possibe overwrite)
      #
      @data{keys %preset} = values %preset;

      #
      # Exclude this perl script from the listing
      #
      push @listing, \%data if $data{PID} != $$ && !($data{COMMAND} =~ /$0/);
   }    

   return @listing;
}
#-----------------------------------------------------------
#
# hp_ps method
#
# Parameters:
#  %preset - This hash is used to define preset values
#            in the returned process hash list.
#            Typically used to define the SUBSYSTEM and 
#            KILL_SIGNAL hash values.
#
# Returns:
#  An array of hash references is returned. 
#  Each array element represents a metrica process and has 
#  the following hash keys:
#
#   $element->{UID}
#   $element->{PID}
#   $element->{PPID}
#   $element->{SUBSYSTEM}
#   $element->{COMMAND}
#   $element->{KILL_SIGNAL}
#
# Notes:
#  It is assumed that all metrica processes have been started
#  via the Start/Stop function. This is important because the
#  command pathname is being used to identify the various
#  processes.
#
#  This method utilises the default HP program "/usr/bin/ps".
#  Giving the x option should mean no restriction on the
#  length of process descriptions
#

sub hp_ps {
   my $self = shift;
   my (%preset) = @_;
   my @listing = ();
   
   #
   # Run the ps command
   #
   my @ps = `ps -u $self->{owner} -fx`;
   die "metrica_processadmin.ksh : Error running ps command" if $?;

   #
   # Parse the command result
   #
   foreach my $line ( @ps ) {
      my %data;
      my @row = split(/\s+/, $line);

      $data{UID}         = $row[1];
      $data{PID}         = $row[2];
      $data{PPID}        = $row[3];
      $data{SUBSYSTEM}   = "UNKNOWN";
      $data{KILL_SIGNAL} = "TERM";

      if ($row[7] eq "?") {
         $data{COMMAND} = join(" ", @row[9 .. $#row]);
      } 
      else {
         $data{COMMAND} = join(" ", @row[8 .. $#row]);
      }

      #
      # Preset column values (Possibe overwrite)
      #
      @data{keys %preset} = values %preset;

      #
      # Exclude this perl script from the listing
      #
      push @listing, \%data if $data{PID} != $$ && !($data{COMMAND} =~ /$0/);
   }    

   return @listing;
}


#-----------------------------------------------------------
#
# bsd_ps method
#
# Parameters:
#  %preset - This hash is used to define preset values
#            in the returned process hash list.
#            Typically used to define the SUBSYSTEM and 
#            KILL_SIGNAL hash values.
#
# Returns:
#  An array of hash references is returned. 
#  Each array element represents a metrica process and has 
#  the following hash keys:
#
#   $element->{UID}
#   $element->{PID}
#   $element->{PPID}
#   $element->{SUBSYSTEM}
#   $element->{COMMAND}
#   $element->{KILL_SIGNAL}
#
# Notes:
#  It is assumed that all metrica processes have been started
#  via the Start/Stop function. This is important because the
#  command pathname is being used to identify the various
#  processes.
#
#  This method utilises the default BSD program "/usr/ucb/ps".
#


sub bsd_ps {
   my $self = shift;
   my (%preset) = @_;
   my @listing = ();
   my $uid = getpwnam($self->{owner});
   
   #
   # Run the ps command
   #
   my @ps = `/usr/ucb/ps -axlww | grep $uid |grep -v grep | grep -v axlww`;
   die "metrica_processadmin.ksh : Error running /usr/ucb/ps command" if $?;

   #
   # Parse the command result
   #
   foreach my $line ( @ps ) {
      my %data;
      $line =~ s/^\s+//;
      my @row = split(/\s+/, $line);

      $data{UID}         = $row[1];
      $data{PID}         = $row[2];
      $data{PPID}        = $row[3];
      $data{SUBSYSTEM}   = "UNKNOWN";
      $data{KILL_SIGNAL} = "TERM";

      $data{COMMAND} = join(" ", @row[0 .. $#row]);

      #
      # Preset column values (Possibe overwrite)
      #
      @data{keys %preset} = values %preset;

      #
      # Exclude this perl script from the listing
      #
      #push @listing, \%data if $data{PID} != $$ && !($data{COMMAND} =~ /$0/);
	push @listing, \%data if $data{PID} != $$ && !($data{COMMAND} =~ /$0/) && ($data{UID} eq "$uid");
   }    

   return @listing;
}

#-----------------------------------------------------------
#
# pause method
#
# Parameters:
#  $somethingToKill - Boolean value indicating whether to
#                     perform a pause
#
# Returns:
#  Object reference
#
sub pause {
   my $self = shift;
   my ($somethingToKill) = @_;

   if ($somethingToKill) {
      print("Waiting 5 seconds ...\n");
      sleep 5;
   } 
   else {
      print("Nothing to kill ...\n");
   }

   return $self;
}

#-----------------------------------------------------------
#
# status method
#
# Parameters:
#  none
#
# Returns:
#  Object reference
#
# Notes:
#  Prints a list of metrica processes to the screen
#
sub status {
   my $self = shift;

   printf "%s; %5s; %5s; %s; %s; %s\n", 
      "UID", "PID", "PPID", "SUBSYSTEM", "COMMAND", "KILL_SIGNAL";

   foreach my $process ( $self->process_table ) {
      printf "%s; %5s; %5s; %s; %s; %s\n", 
         $process->{UID}, 
         $process->{PID}, 
         $process->{PPID}, 
         $process->{SUBSYSTEM}, 
         $process->{COMMAND},
         $process->{KILL_SIGNAL}
         ;
   }

   return $self;
}

#-----------------------------------------------------------
#
# stop method
#
# Parameters:
#  none
#
# Returns:
#  Object reference
#
# Notes:
#  Sends the configured signal to all processes except those
#  marked UNKNOWN.
#
sub stop {
   my $self = shift;
   my $somethingToKill = 0;

   foreach my $process ( $self->process_table ) {
      if (($process->{SUBSYSTEM} ne "Start/Stop") && ($process->{SUBSYSTEM} ne "UNKNOWN")) {
         $somethingToKill = 1;
         printf "Sending %s to %s\n", $process->{KILL_SIGNAL}, $process->{COMMAND};
         kill("$process->{KILL_SIGNAL}", $process->{PID});
      }
   }

   $self->pause($somethingToKill);

   return $self;
}

#-----------------------------------------------------------
#
# kill method
#
# Parameters:
#  none
#
# Returns:
#  Object reference
#
# Notes:
#  This method reads data from the program standard input.
#  The assumption is that it is formatted according to the
#  output of the status method.
#
#  All processes are sent the configured signal
#
sub kill {
   my $self = shift;
   my $somethingToKill = 0;

   my $file = new FileHandle('-') or die("Cannot open STDIN");

   while (my $line = $file->getline()) {
      chomp($line);
      my @process = split(/\s*;\s*/, $line);

      my $uid       = $process[0];
      my $pid       = $process[1];
      my $ppid      = $process[2];
      my $subsystem = $process[3];
      my $command   = $process[4];
      my $signal    = $process[5];

      if ($uid ne "UID") {
         $somethingToKill = 1;
         printf "Sending %s to %s\n", $signal, $command;
         kill("$signal", $pid);
      }
   }
   $file->close;

   $self->pause($somethingToKill);

   return $self;
}

__END__

=head1 CHANGE HISTORY

 Date        Author         Comments
 ====        ======         ========
 2004-04-16  Mark O'Connor  Initial draft.
 2005-04-11  Shane O'Rourke  Introducing platform specific ps.
                             Specific killing of BES processes

=head1 GLOSSARY

=head1 ASSOCIATED DOCUMENTS

=head2 Referenced documents

=over

=item NGN System Management Subsystem Design Specification 

L<http://notesdoc/872569EE0064948E/(ADCWebOpenLinkLatestVD)?OpenAgent&ddHID=17102001-Q4XO-CNSR&LinkType=VD&Open=>

=item High Level Design System Management  

L<http://notesdoc/872569EE0064948E/(ADCWebOpenLinkLatestVD)?OpenAgent&ddHID=17102001-Q4XO-CNSR&LinkType=VD&Open=>

=back

=head2 Other related documents

=over

=item NGN System Management Start Stop Low Level Design 

L<http://notesdoc/872569EE0064948E/(ADCWebOpenLinkLatestVD)?OpenAgent&ddHID=02272002-3FH3-2TYV&LinkType=VD&Open=>

=item NGN System Management Start Stop Command Line Interface Specification 

L<http://notesdoc/872569EE0064948E/(ADCWebOpenLinkLatestVD)?OpenAgent&ddHID=05032002-837O-MG4M&LinkType=VD&Open=>

=back

=head1 INTRODUCTION

This document describes the command line interface to the metrica process 
administration script.
This script is used to both display running metrica processes and provides
options to kill these processes.

This command is designed to be used as a trouble-shooting tool and to be 
called from the Start / Stop service, ensuring the metrica system has
been properly shutdown.

=head2 Document Scope

This document describes the metrica process administration script 
introduced as part of the NGN Largo-1 project.
The scope of this function was initially limited to recognising and 
killing Borland appserver processes.

It is assumed that all metrica processes are started via the Start/Stop
function. 

The current version can recognise all but 3 java processes started by
the performance manager application.
These unrecognised processes are displayed with an UNKNOWN classification
and will not be killed using the B<-stop> option (See below).

=head2 Document Conventions

In the command line syntax the use of curly brackets {} implies compulsory
entieis, square brackets [] implies optional entities.  
Bars | denote choices.

=head2 Outstanding issues

An outstanding issue is how to manage the B<metrica_load> process on an
environment which has multiple instances of Performance Manager installed.
Care must be taken when using the B<-stop> option in such cases.

Another issue is how to properly identify java processes that belong
to the metrica application.
The following processes cannot currently be recognised due to the  field width restriction
of the B<ps> command.

=over

=item StartStopServer

 /usr/java1.3.1/bin/../bin/sparc/native_threads/java 
  -DMETRICA_DIR=/apps/inst1/metrica 
  -Djava.security.policy=/apps/inst1/metrica/StartStop/data/rmi_policy 
  -classpath /apps/inst1/metrica/StartStop/lib/SystemManagement.jar 
  com.adc.metrica.systemmanagement.startstop.StartStopServer 
  -start 
  -propertyfile /apps/inst1/metrica/StartStop/data/startstop.properties

=item NotificationService

 /usr/java1.3.1/jre/bin/../bin/sparc/native_threads/java 
  -server
  -classpath ...
  -DOF.Install.Dir=/apps/inst4/openfusion 
  -DOF.Domains.URL=file:/apps/inst4/openfusion/domains 
  -DOF.Node.URL=file:/apps/inst4/openfusion/domains/OpenFusion/localhost 
  -DOF.Domain.URL=file:/apps/inst4/openfusion/domains/OpenFusion 
  -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORB 
  -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB 
  -Dvbroker.orb.dynamicLibs=com.prismt.orb.portable.visibroker.Init 
  -Dvbroker.agent.port=30106 
  -DORBservices=com.prismt.orb.portable.visibroker.Init 
  com.prismt.openfusion.orb.Service 
  file:/apps/inst4/openfusion/domains/OpenFusion/localhost/NotificationService/NotificationService.xml

=item ReportTaskExecutorServer

 usr/java1.3.1/bin/../bin/sparc/native_threads/java 
  -cp ...
  -DORBagentPort=30106 
  -Xms20m -Xmx100m -DSVCnameroot=namingservice 
  -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB 
  -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORBSingleton 
  -Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl 
  -Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl 
  -Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl 
  com.adc.metrica.reporting.reportservice.reportserver.reporttaskexecutor.server.ReportTaskExecutorServer

=back

It has been suggested that all java invocations would include an extra 
parameter indicating the subsystem name.

=head1 FUNCTIONAL DESCRIPTION

=head2 Functional overview

The purpose of this function is to observe the component processes that implement
the Metrica Performance Manager product.
Occasionally it is necessary to kill off these processes in situations where the
application may not have been shutdown in the correct order.

This function uses the Unix B<ps> command to detect the processes executing
on the host machine.
The process listing is then filtered, classified and displayed
to the user.
Each process is assigned a recommend B<kill> signal; this signal can be
optionally used to shutdown the process.

This function is configured to be called by the Start/Stop service. 

B<Use of the unix ps command>

The B<ps> command differs from platform to platform. The following command syntax 
is common between Solaris and HPUX

  ps -u metrica -f

It must be noted however that under HPUX the I<COMMAND> column has a default
limit of 64 characters, whereas on Solaris this limit is 80. This has been 
extended for use on HP-UX 11.11 + through the use of ps -u <user> fx. 
This places a restriction on the identification of processes with very long 
command paths (Such as java processes).

=head2 Usage Scenarios

=over

=item Print help messages to the screen

The abbreviated help messages are printed by issuing the follow command

  metrica_processadmin.pl -help

=item Return script version

The following command returns the version and copyright information of the script

  metrica_processadmin.pl -version

Output:

  Metrica Process Administration
  Copyright  2000-2005 Vallent Software Systems
  SysManagement Version 4.0 Patch 0 (Checkpoint: 1, State: Released product)

=item Print a list of metrica processes

The following command returns a list of metrica processes

  metrica_processadmin.pl -status

The following is an example output

 UID;   PID;  PPID; SUBSYSTEM; COMMAND; KILL_SIGNAL
 ngndev;  2195; 27824; Start/Stop; /usr/java1.3.1/bin/../bin/sparc/native_threads/rmiregistry; TERM
 ngndev;  2223;     1; UNKNOWN; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -DMETRICA_DIR=/apps/inst1/m; KILL
 ngndev;  2351;  2350; Ias; /apps/inst1/BES5.1/bin/ias; 9
 ngndev;  2373;  2351; Osagent; /apps/inst1/BES5.1/bin/osagent -serversig -sig int; USR2
 ngndev;  2397;  2351; Osagent; /apps/inst1/BES5.1/bin/osagent -serversig -sig int; USR2
 ngndev;  2398;  2351; Partition; /apps/inst1/BES5.1/bin/partition -server bes-triffid-inst1 -partition standard; 9
 ngndev;  2739;  2223; Common Services; /bin/ksh /apps/inst1/metrica/CS/bin/metrica_selfreportingserviceadmin.ksh -star; TERM
 ngndev;  2742;  2739; Common Services; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -Dvbroker.agent.port=30106; TERM
 ngndev;  2891;     1; UNKNOWN; /usr/java1.3.1/jre/bin/../bin/sparc/native_threads/java -server -classpath /app; KILL
 ngndev;  2991;  2223; Common Services; /bin/ksh /apps/inst1/metrica/CS/bin/metrica_nfserviceadmin.ksh -start; TERM
 ngndev;  2994;  2991; Common Services; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -DORBInitRef=NotificationSe; TERM
 ngndev;  3061;  2223; Common Services; /bin/ksh /apps/inst1/metrica/CS/bin/metrica_eventserviceadmin.ksh -start; TERM
 ngndev;  3064;  3061; Common Services; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -Dvbroker.agent.port=30106; TERM
 ngndev;  3141;  2223; Common Services; /bin/ksh /apps/inst1/metrica/CS/bin/metrica_schedulingserviceadmin.ksh -start; TERM
 ngndev;  3144;  3141; Common Services; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -Dvbroker.agent.port=30106; TERM
 ngndev;  3675;  2223; Alarm Management; /bin/ksh /apps/inst1/metrica/AlarmManagement/bin/metrica_alarmserviceadmin.ksh; TERM
 ngndev;  3678;  3675; Alarm Management; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -Dvbroker.agent.port=30106; TERM
 ngndev;  3825;     1; UNKNOWN; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -cp :/apps/inst1/metrica/ex; KILL
 ngndev;  4013;     1; DB Loader; metrica_load -ruleset 1030111120 -perfdb metrica_performance -admindb metrica_a; TERM
 ngndev;  4016;     1; DB Loader; metrica_load -ruleset 1030111120 -perfdb metrica_performance -admindb metrica_a; TERM
 ngndev;  4018;  2223; DB Loader; /bin/ksh /apps/inst1/metrica/DBLoader/bin/metrica_lcmadmin.ksh -start; TERM
 ngndev;  4019;  4018; DB Loader; /usr/java1.3.1/bin/../bin/sparc/native_threads/java -Dvbroker.agent.port=30106; TERM

If a process' subsystem cannot be determined the text B<UNKNOWN> will appear in the
subsystem column of the report.

=item Kill the list of metrica processes

The list of processes returned by the B<-status> option will be killed by running
the following command

  metrica_processadmin.pl -stop

It is intended that this command is called by the Start/Stop service after shutting
down the appserver. It is not intended to be a replacement for the script
F<metrica_admin.ksh>.

B<Note:> This option will not stop unknown processes or processes involved in the
operation of the Start/Stop service.

=item Kill a selected list of metrica processes

This command is very similar to the B<-stop> option. 
The Unix B<grep> command can be used to choose which processes are killed.

The following example kills the unknown processes.

  metrica_processadmin.pl -status | grep UNKNOWN | metrica_processadmin.pl -kill

Alternatively the user could save the process list in a file, edit it and then submit
it to the program as follows:

  metrica_processadmin.pl -status > process_list.txt
  vi process_list.txt
  metrica_processadmin.pl -kill < process_list.txt

=back

=head1 SYNOPSIS

metrica_processadmin.pl {-help | -version | -status | -stop | -kill}

The purpose of this function is to observe the component processes that implement
the Metrica Performance Manager product.

=head2 Command arguments

=over 8

=item B<-help>

Print a help message and exit.

=item B<-version>

Print the script copyright and version information.

=item B<-status>

Prints a listing of active metrica processes. 

B<Note:> The recommend signal to be used when killing each process is also printed

=item B<-stop>

This option sends the recommended kill signal to each process in the B<-status>
listing.

B<Note:> UNKNOWN and Start/Stop processes are never signalled.

=item B<-kill>

Kill metrica processes submitted on STDIN. The following example can be used
to kill the Start/Stop processes.

  metrica_processadmin.pl -status | grep "Start/Stop"  | metrica_processadmin.pl -kill

=back

=head2 Return codes

This script returns one of the following return codes:

=over

=item [0]

The command has been correctly specified and executed.
Results are printed on STDOUT.

=item [1]

The command has been correctly specified however the script has been 
asked to execute a Start/Stop service parameter that is unhandled.
The message I<Start/Stop parameter not implemented> is printed on STDOUT.
Usage details are printed on STDOUT.

=item [2]

The command has been incorrectly specified.
An error message is printed on STDERR.
Usage details are printed on STDERR.

=back

=head1 CONFIGURATION

=head2 Environment variables

This function requires the following environment variables to be set correctly.

=over

=item LOGNAME

This is used to determine for which userid the processes are displayed.

=item APPSERVER_DIR

This is used to determine which processes belong to the Borland appserver.

=item METRICA_DIR

This is used to determine which processes belong to various Metrica sub-systems.

=back

=head1 APPENDICES

=cut

