###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/EndStamp/metrica_update_endstamp_task.pl SysManagement_r4.79.0.9:cset.007769:8:8 08/25/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
# 
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
#
# Title:			metrica_update_endstamp_task.pl    
# Author:			Eugene Stuart    
#
# Description:      Reads list of NC table from file.
#					Checks whether tables have re-parented objects that require
#					updating (that is, whether the endstamp column values
#					if there are duplicate (object_id, endstamp) rows).
#					If so, updates endstamp.
#
##############################################################################
=head1 Application Class

 DESCRIPTION:
   The purpose of the class is to contain the main application logic for adding/
updating the endtime column.

=cut

#---------------------------------------------------------------------------
#       PACKAGE NAME
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#   Core Perl Libraries
#---------------------------------------------------------------------------
use strict;
use FileHandle;
use Getopt::Long;
use lib "$ENV{METRICA_DIR}/Installer/bin/lib";
use Application;

# Force output after every line (keeps stderr and stdout in sync)
autoflush STDOUT 1;
autoflush STDERR 1;

#---------------------------------------------------------------------------
#   Program Libraries
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PRIVATE GLOBAL CONSTANTS
#---------------------------------------------------------------------------
my $OK           = "OK";
my $TIMESTAMP_COL = "timestamp";
my $ENDSTAMP_COL  = "endstamp";
# -- bugfix 50585 - change column default from 2147483647 to 0
my $ENDSTAMP_DEFAULT=0;
# -- bugfix 50585 - the value given to the current object
my $ENDSTAMP_CURRENT=2147483647;
my $REPARENTED_PREFIX = "/tmp/Reparented_";
my $UPDATE_FILE_PREFIX = "/tmp/Update_";

#---------------------------------------------------------------------------
#       PUBLIC GLOBAL CONSTANTS
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PRIVATE GLOBAL VARIABLES
#---------------------------------------------------------------------------
my $USEFUL_FNS = new Application();
my %param = ();
main(\%param);
my $log;

#---------------------------------------------------------------------------
#       PUBLIC GLOBAL VARIABLES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PUBLIC SUBROUTINES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       CLASS METHOD SUBROUTINES
#---------------------------------------------------------------------------

#***************************************************************************

=head2 new 

 DESCRIPTION:
   This constructor parses the configuration file and initialises the 
   object data structures

 PARAMETERS:
   $class    - The name of the class (implicit parameter).

 RETURNS:
   The class object.

=cut

sub new
{
    my $class = shift;
    my $self = {};
    bless $self, $class;

    return $self;
}

#***************************************************************************
## Logs date and message to log file
##
## Note: first parameter is a valid FileHandle:: reference
#
sub log_msg {
	my ($file_handle, $msg) = @_;

	#printf("$msg ");

	my $msg_time = scalar(localtime(time));
	$file_handle->print("$msg_time - $msg");
}


#***************************************************************************
## Runs one or more sql scripts on an informix database
sub RunScript 
{
   my ($database, $filename) = @_;
   my $command;

   $command = sprintf "\$INFORMIXDIR/bin/dbaccess %s < %s 2>&1", $database, $filename;
   my @output = `$command`;

   return ($? >> 8);
}


#***************************************************************************
## Get the access key for nc_table
#
#  Returns a reference to a hash of NC table access keys
#
sub GetAccessKeys {
	my ($admin_database,$perf_database) = @_;

	log_msg($log, "Getting access keys for NC tables from database [$admin_database]\n");

	# -- Get the contents of the lc_relations table
	my $relations_SQL = "select unique nc_tabname,access_key from lc_relations where nc_tabname like 'nc_%'";
	
	log_msg($log, "$relations_SQL\n");
	my @lines = $USEFUL_FNS->execute($admin_database, $relations_SQL);
	my $tab="";
	my $access="";
        my %nc_table_access = ();

	foreach my $line ( @lines ) {
		if ($line =~ /^nc_tabname\s+(.*)$/) {
			$tab = $1;
		} elsif ($line =~ /^access_key\s+(.*)$/) {
			$access = $1;
		}
		if ( $tab && $access) {
			# Add only valid columns to the hash map
			if ($USEFUL_FNS->columnExists($perf_database, $tab,$access)){
			
				if ( !$nc_table_access{$tab} ) {
			       		$nc_table_access{$tab} = $access;
			        }
				else {
                                        printf( "WARNING: Multiple access keys [ %s, %s ] found in lc_relations for table %s. Using %s for updates.\n",$nc_table_access{$tab}, $access,$tab,$nc_table_access{$tab}); 
				}
			}
			else {
                                printf( "WARNING: Table/column [%s.%s] found in lc_relations does not exist in the performance database.\n",$tab,$access); 
			}
			
			$tab="";
			$access="";
		} 
	}

	if (%nc_table_access) {
		log_msg($log, "Retrieved access keys for NC tables\n");
	}

	return \%nc_table_access;
}



#***************************************************************************
## Get the access key for a particular nc_table.
## This sub is used when the access key is not present in lc_relations.
#
#  Returns a string representing access keys
#
sub GetAccessKeyFromSysCatalog {
	my ($database, $table, $timestamp_col) = @_;
	
	# -- Get the indexes associated with this table
	my ($idxtype, $idxcolumns_REF, $idxdbspace) = $USEFUL_FNS->getIndexDetails($database, $table);

	# -- return value
	my $access_key = "";

	# -- Iterate indexes until find one with timestamp in it
	# -- Then return the column in index that is not called
	# -- timestamp or endstamp
	foreach my $index ( keys %{$idxcolumns_REF} ) {
		if (grep(/^$timestamp_col\s*/, @{$idxcolumns_REF->{$index}}) > 0) {
			foreach my $col ( @{$idxcolumns_REF->{$index}} ) {
				if ( ($col ne $ENDSTAMP_COL) && ($col ne $timestamp_col) ) {
					# -- found access key
					$access_key = $col;
				}
			}
			if ( $access_key ) {
				#-- no  need to process any more indexes for this table
				last;
			}
		}
	}

	# -- return
	return $access_key;
}


#***************************************************************************
## Unload the access_key and timestamp to file of Reparented
#  network entity (cell, bsc, msc .....)
#
sub UnloadReparentedObject {
	my ($filename, $database, $nc_table, $nc_access_key, $timestamp_col) = @_;

	my $retVal="";

	# -- remove old reparented file, if exists
	unlink $filename;

	log_msg($log, "Unloading from table [$nc_table] into file [$filename]\n");

	# -- unload, grouping by id and timestamp and 
	# -- placing timestamp in ascending order within group

	my $unload_SQL = "unload to $filename select $nc_access_key, $timestamp_col from $nc_table where $nc_access_key in (select $nc_access_key from $nc_table group by $nc_access_key having count(*) >1) order by $nc_access_key, $timestamp_col"; 

	log_msg($log, "$unload_SQL\n");
	$retVal = $USEFUL_FNS->execute($database, $unload_SQL);
	
	log_msg($log, "Unloading returned [$retVal]\n");

	return $retVal;
}


#***************************************************************************
## Parse the unloaded file containing reparented network elements.
##
## bugfix 50585 -	When all other updates are complete, check for endstamp
##					columns with 0, update them to ENDSTAMP_CURRENT
#
sub CreateEndstampUpdateScript {
#	my $self = shift;
	my ($update_file, $reparented_file, $nc_table, $access_col, $endstamp_col, $timestamp_col) = @_;

	log_msg($log, "Creating update script [$update_file] for table [$nc_table] using data file [$reparented_file]\n"); 

	# -- previous line in source file
	my $prev_nc_access_id = "";
	my $prev_nc_timestamp = "";

	# -- current line in source file
	my $curr_nc_access_id = "";
	my $curr_nc_timestamp = "";

	# -- marks start of new reparented object 
	my $new_id = 1;

	# -- update line in target SQL script 
	my $update_SQL = "";
	my $endstamp; 
	
	# -- create update script for writing
	unlink $update_file;
	my $script_fh = new FileHandle($update_file, "w")
                                or die "Cannot open file for writing: $update_file";
	
	# -- open the reparented file that was unloaded
	my $reparent_fh = new FileHandle($reparented_file, "r");
	if ( !defined $reparent_fh ) {
		log_msg($log, "WARNING:  Cannot open file for reading: $reparented_file\n");
		log_msg($log, "\tCheck that the access_key [$access_col] in lc_relations is spelled correctly\n");
		log_msg($log, "WARNING:  table [$nc_table] will not be updated.");
		print "WARNING:  table [$nc_table] will not be updated. Could not retrieve reparented cells\n";
		close $script_fh;
		# -- EXIT script
		return -1;
	}

	# -- go faster stripes
	$script_fh->print("SET PDQPRIORITY 80;\n");

	# -- process reparented file line by line
	# -- create update script, setting endstamp column.
	# -- Note: reparented file is ordered by access_key, timestamp (ascending)
	if ( -z $reparented_file )
	{
		my $update_defaults_SQL="update $nc_table set $timestamp_col=-abs($timestamp_col), $endstamp_col=$ENDSTAMP_CURRENT where $endstamp_col=$ENDSTAMP_DEFAULT;\n";

		$script_fh->print("$update_defaults_SQL");

		close $reparent_fh;
		close $script_fh;

		log_msg($log, "Created update script\n");
	}
	else
	{
	while (<$reparent_fh>) {	
		($curr_nc_access_id, $curr_nc_timestamp) = split (/\|/,$_,3);
		$endstamp = $curr_nc_timestamp - 1;
		if ( ($curr_nc_access_id eq $prev_nc_access_id)) {
			if ($new_id) {
				# -- use abs() to ensure we are not simply togggling value
				# -- on a subsequent run of script
				$update_SQL .= "update $nc_table set $timestamp_col=-abs($timestamp_col), $endstamp_col=$endstamp where $access_col='$prev_nc_access_id' and $timestamp_col=$prev_nc_timestamp;\n";
				# -- reset new_id
				$new_id = 0;
			} else {
				# -- add update statement for previous record
				$update_SQL .= "update $nc_table set $endstamp_col=$endstamp where $access_col='$prev_nc_access_id' and $timestamp_col=$prev_nc_timestamp;\n";
			}
		} else {
			$new_id=1;
			# -- have got new id, so write SQL for records for this reparented object to script
			if ($prev_nc_access_id) {
				$update_SQL .= "update $nc_table set $endstamp_col=$ENDSTAMP_CURRENT where $access_col='$prev_nc_access_id' and $timestamp_col=$prev_nc_timestamp;\n";
			}
			$script_fh->print("$update_SQL");
			# -- reset update statement
			$update_SQL = "";
		}
		# -- store id and timestamp
		$prev_nc_access_id = $curr_nc_access_id;
		$prev_nc_timestamp = $curr_nc_timestamp;
	}

	# -- write SQL string of last records with last id to update script
	$update_SQL .= "update $nc_table set $endstamp_col=$ENDSTAMP_CURRENT where $access_col='$prev_nc_access_id' and $timestamp_col=$prev_nc_timestamp;\n";
	$script_fh->print("$update_SQL");

	# -- bugfix 50585
	# -- these rows represent all non-reparented objects - that is, single rows
	# -- and have endstamp=0
	my $update_defaults_SQL="update $nc_table set $timestamp_col=-abs($timestamp_col), $endstamp_col=$ENDSTAMP_CURRENT where $endstamp_col=$ENDSTAMP_DEFAULT;\n";
	$script_fh->print("$update_defaults_SQL");

	close $reparent_fh;
	close $script_fh;

	log_msg($log, "Created update script\n");

	# ZZZZ we probably do not need the reparenting file for audit purposes.
	#unlink $reparented_file;
	}
}


#***************************************************************************
## Update the value of the endstamp column
## 
#
sub UpdateEndstampColumn {
	my ($database, $reparent_file_prefix, $update_file_prefix, 
		$nc_tables_REF, $nc_access_keys_REF) = @_;
	my $update_script;
	my $reparented_file;
	my $retVal = -1;

	# -- dereference the input parameters 
	my @nc_tables = @{$nc_tables_REF};	
	my %nc_access_keys = %{$nc_access_keys_REF};

	print "\n";  #new line before begin of updates

	# -- create update script and run.
	foreach my $tab ( @nc_tables ) {
		# -- remove white space
		$tab =~ s/^\s+//;
		$tab =~ s/\s+$//;
		if (!$USEFUL_FNS->tableExists($database, $tab)) {
			log_msg($log, "ERROR: UpdateEndstampColumn() - table does not exist : [$tab]\n");
			printf("ERROR: Table does not exist : %s\n", $tab);
		} elsif ( $nc_access_keys{$tab} ) {
			$update_script = $update_file_prefix . $tab . ".sql";
			$reparented_file = $reparent_file_prefix . $tab . ".dat";
			CreateEndstampUpdateScript($update_script, $reparented_file, $tab, $nc_access_keys{$tab}, $ENDSTAMP_COL, $TIMESTAMP_COL);
			# -- run the update script if exists and non-empty
			if ( ! -e $update_script) {
				log_msg($log, "WARNING:  file [$reparented_file] does not exist.\n");
			} elsif ( -z $update_script) {
				log_msg($log, "No reparented rows. No need to update table [$tab].\n");
			} else {
				# -- file exists and is non-empty
				log_msg($log, "Updating $ENDSTAMP_COL on table [$tab] using update script [$update_script]\n");
				print "AUDIT: Updating $ENDSTAMP_COL on table [$tab] using $nc_access_keys{$tab}\n";
				$retVal = RunScript($database, $update_script);
				if ($retVal) {
					log_msg($log, "ERROR updating table [$tab]. RunScript() returned [$retVal]\n");
					print "ERROR updating table [$tab]. RunScript() returned [$retVal]\n";
				} else {
					log_msg($log, "Updated $ENDSTAMP_COL on table [$tab]. Returned [$retVal]\n");
				}
			}	# -- end else non-empty file
		}	# -- end else if table exists
	}	  # -- end foreach

}


#***************************************************************************

=head2  main

 DESCRIPTION:
   Main routine for the application.

 PARAMETERS:
   $self - The class object (implicit parameter).
   $ref  - A Reference to a hash of parameters

 RETURNS:
   $OK

=cut

sub main
{
  #	my $self = shift;
  my ($database, $logFile, $table_list_file, $admin_db);

  my $args = GetOptions(
             "database=s"  => \$database,
             "log_file=s"  => \$logFile,
             "table=s"     => \$table_list_file,
             "admin_database=s"  => \$admin_db);


	# -- a hash of access_key's keyed on NC Tables.
	# -- note there may be more NC table references in lc_relations
	# -- than there are in systables.
	# -- However, this is not a problem as we will use a tablename
	# -- from systables as a key to this hash. 
#	my %nc_access_keys;
#	my $nc_access_keys_REF = \%nc_access_keys; 
	my $nc_access_keys_REF;
   
	my $table_list_fh = new FileHandle($table_list_file, "r") 
                                    or die "Cannot open logfile: $table_list_file";

	# -- a list of NC tables from systables
	my @nc_tables_list = $table_list_fh->getlines();
	chomp(@nc_tables_list);

	$table_list_fh->close;

	# -- Open the log file
	$log = new FileHandle($logFile, "w") 
				or die "Cannot open logfile: $logFile";
	log_msg($log, "Starting Thread:\n");                                         
    
   	if (@nc_tables_list) {
			# -- first get a rough time estimate for the entire procedure
			my $nc_tables_REF = \@nc_tables_list;
			$nc_access_keys_REF = GetAccessKeys($admin_db,$database);

			my $reparented_file;
			foreach my $table ( @nc_tables_list ) { 
				# -- remove white space
				$table =~ s/^\s+//;
				$table =~ s/\s+$//;
				if (!$USEFUL_FNS->tableExists($database, $table)) {
					log_msg($log, "ERROR: UpdateEndstampColumn() - table does not exist : %s\n", $table);
					printf("ERROR: Table does not exist : %s\n", $table);
				} else {
					# -- table exists - check table has access key
					# -- if no access key found in lc_relations,
					# -- then try and get it from index on table
					# -- that contains timestamp column.
					if ( !$nc_access_keys_REF->{$table} ) {
					
						log_msg($log,"WARNING: Could not find the access key for $table in lc_relations, searching for an index with a timestamp\n");


						$nc_access_keys_REF->{$table} = GetAccessKeyFromSysCatalog($database, $table, "timestamp");
						if ( !$nc_access_keys_REF->{$table} ) {
							# -- This code should never be executed as the 
							# -- metrica_add_endstamp.pm script has already done this
							# -- check and produced a table list that does not include it
							#
                                                        my $message= "ERROR:   Cannot find the information needed to update endstamp column on table [$table].  See the Technology Pack Installation Notes for details of how to resolve this issue.\n";
					                print $message;
						        log_msg($message);

	              	                                my $sql = "update $table set $ENDSTAMP_COL=$ENDSTAMP_CURRENT";
		         															                                                                                                        log_msg("Updating table with the following SQL: $sql \n");
			     	            													                                                                                                        $USEFUL_FNS->execute($database,$sql);
																	    
							# -- get next table 
					        	next;

						} else {
							log_msg($log, "Found access key [$nc_access_keys_REF->{$table}] for table [$table] in sys catalog\n");	
						}
					}

					$reparented_file = $REPARENTED_PREFIX . $table . ".dat";
					if ( $nc_access_keys_REF->{$table} ) {
							UnloadReparentedObject($reparented_file, $database, $table, 
								$nc_access_keys_REF->{$table}, $TIMESTAMP_COL);
					}
				}
			}
			
			UpdateEndstampColumn($database, $REPARENTED_PREFIX, $UPDATE_FILE_PREFIX, 
						$nc_tables_REF, $nc_access_keys_REF); 
		} else {
			log_msg($log, "There are no NC tables to be updated\n");
			print "There are no NC tables to be updated";
		}
	
	# -- Cleanup
	log_msg($log, "Finished Thread.");	
	$log->close;

	return $OK;
}
