#!/bin/ksh
# @(#) SysManagement/delivery/unix/bin/metrica_informixadmin.ksh SysManagement_r4.79.0.9:cset.004229:2:2 04/21/04 @(#)
#
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                ADC Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Software Systems UK, 2002.
#
# Title:        metrica_informixadmin.ksh
# Author:       Shane O'Rourke
# Description:  Start and stop Informix Dynamic Server component
#
###############################################################################



# Exit codes
#
exit_rc_success=0
exit_rc_nopermission=1
exit_rc_usage=1
exit_rc_failure=1
exit_rc_var_incorrect=2
exit_rc_dir_incorrect=3
exit_rc_file_incorrect=3

# Status codes
#
RUNNING=0
STARTING=1
NONFUNCTIONAL=3
STOPPED=4
INFORMIX_STATUS=$STOPPED
INFORMIX_STATUS_STRING='informix Nonfunctional'

# INFORMIX commands
#
preparestop_command='onmode -sy'
stop_command='onmode -ky'
status_command='onstat -'
version_command='onstat -'
start_command='oninit'
bring_online_from_quiescent_command='onmode -m'

# Comment Fuzzy or Full for checkpoint command mode
#
# FUZZY checkpoint  =  'onmode -c fuzzy'
# FULL  checkpoint  =  'onmode -c'
checkpoint_command='onmode -c fuzzy'



###############################################################################
#
# Function    : usage
#
# Description : Displays program syntax
#
# Parameters  : none
#
###############################################################################

usage() {
  echo
  echo "Usage: `basename $0` {"
  echo "            -help | --help | -version | --version |"
  echo "            -status | -start | -stop | -restart | -preparetostop"
  echo "        }"
  echo
}



###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variables, file and directory variables are
#               set incorrectly, and quits accordingly if not.
#
# Parameters  : Name of environment variable
#
###############################################################################

check_env()
{
 # INFORMIX DIRECTORY ENVIRONMENT VARIABLE
 #
 if [ ! -d "$INFORMIXDIR" ] ; then
   dir_incorrect "INFORMIXDIR"
 fi

 # INFORMIX BIN DIRECTORY ENVIRONMENT VARIABLE
 #
 if [ ! -d "$INFORMIXDIR/bin" ] ; then
    dir_incorrect "INFORMIXDIR/bin"
 fi

 # ONCONFIG FILE ENVIRONMENT VARIABLE
 #
 if [ ! -f "$INFORMIXDIR/etc/$ONCONFIG" ] ; then
    file_incorrect "ONCONFIG"
 fi

 # INFORMIX SERVER ENVIRONMENT VARIABLE
 #
 if [ ! "$INFORMIXSERVER" ] ; then
    var_incorrect "INFORMIXSERVER"
 fi
}



###############################################################################
#
# Function    : var_incorrect
#
# Description : Environment variable is set incorrectly
#
# Parameters  : Name of environment variable
#
###############################################################################

var_incorrect()
{
  echo "Environment variable \$"$1" is not set correctly"
  exit $exit_rc_var_incorrect
}



###############################################################################
#
# Function    : dir_incorrect
#
# Description : Directory environment variable is set incorrectly
#
# Parameters  : Name of directory environment variable
#
###############################################################################

dir_incorrect()
{
  echo "Directory environment variable \$"$1" does not exist"
  exit $exit_rc_dir_incorrect
}


###############################################################################
#
# Function    : file_incorrect
#
# Description : File environment variable is set incorrectly
#
# Parameters  : Name of file environment variable
#
###############################################################################

file_incorrect()
{
  echo "File environment variable \$"$1" is not set correctly"
  exit $exit_rc_file_incorrect
}



###############################################################################
#
# Function    : status
#
# Description : Displays Informix Server status
#
# Parameters  : None
#
###############################################################################

status()
{
  status_Informix
  echo $INFORMIX_STATUS_STRING
  exit $exit_rc_success
}



###############################################################################
#
# Function    : status_Informix
#
# Description : Parses Informix Server status
#
# Parameters  : None
#
###############################################################################

status_Informix()
{
  check_env
  STATUS_STRING=`$status_command`
  if [[ -n `echo $STATUS_STRING | fgrep -e 'permission denied.'` ]] ; then 
    advise 'check the status of'
  fi
  if [[ -n `echo $STATUS_STRING | fgrep -e '-- On-Line --'` ]] ; then
    INFORMIX_STATUS_STRING="informix Running"
    INFORMIX_STATUS=$RUNNING
  elif [[ -n `echo $STATUS_STRING | fgrep -e '-- Quiescent --'` ]] ; then
    INFORMIX_STATUS_STRING="informix Nonfunctional"
    INFORMIX_STATUS=$NONFUNCTIONAL
  elif [[ -n `echo $STATUS_STRING | fgrep -e '-- Fast Recovery --'` ]] ; then
    INFORMIX_STATUS_STRING="informix Starting"
    INFORMIX_STATUS=$STARTING
  else
    INFORMIX_STATUS_STRING="informix Stopped"
    INFORMIX_STATUS=$STOPPED
  fi
}



###############################################################################
#
# Function    : help_Informix
#
# Description : Displays the Informix start stop administration usage
#
# Parameters  : None
#
###############################################################################

help_Informix()
{
  usage
  exit $exit_rc_success
}



###############################################################################
#
# Function    : version_Informix
#
# Description : Displays the Informix verion information
#
# Parameters  : None
#
###############################################################################

version_Informix()
{
  check_env
  $version_command
  exit $exit_rc_success
}



###############################################################################
#
# Function    : preparetostop_Informix
#
# Description : Prepares to stop the Informix Dynamic Server by
#               * Forces checkpoint (Either Fuzzy or Full)
#               * Gracefully move from On-Line to Quiescent onmode -sy
#
# Parameters  : None
#
###############################################################################

preparetostop_Informix()
{
  check_env
  $checkpoint_command
  informix_return_string=`$preparestop_command`
  if (( $? != 0 )) ; then 
    advise 'preparetostop'
  fi
  exit $exit_rc_success
}



###############################################################################
#
# Function    : start_Informix
#
# Description : Prepares to start the Informix Dynamic Server
#
# Parameters  : None
#
###############################################################################

start_Informix()
{
  check_env
  status_Informix
  if [[ "$INFORMIX_STATUS" = "$STOPPED" ]] ; then
    $start_command > temp_informix_return_string_file 2>&1
  elif [[ "$INFORMIX_STATUS" = "$NONFUNCTIONAL" ]] ; then
    $bring_online_from_quiescent_command >temp_informix_return_string_file 2>&1
  fi
  let RET_VAL=$?
  rm temp_informix_return_string_file 2>/dev/null
  if (( $RET_VAL != 0 ))
  then 
    read informix_return_string < temp_informix_return_string_file
    advise 'start' "$informix_return_string"
  fi
  exit $exit_rc_success
}



###############################################################################
#
# Function    : advise
#
# Description : outputs advice for starting/stopping informix
#               if a command has failed
#
# Parameters  : the action that has failed
#		the return string from the command that failed
#
###############################################################################

advise()
{
  if [[ "$2" = "Must be a DBSA to run this program" ]] ; then
    echo "Did not $1 Informix. Users must be DBSA to perform this action."
    echo "The current account does not have appropriate permissions."
    exit $exit_rc_nopermission
  else
    echo "Failed to $1 Informix; $1 Informix manually as user informix."
    echo "The current account does not have appropriate permissions."
    echo ""
    echo "Take the following steps:"
    echo "	1 become user informix"
    echo "	2 $1 the database server using 'metrica_informixadmin.ksh -$1'"
    echo ""
    echo "Alternatively add the current account to the informix process group"
    echo "(See groups manpage)"
    echo ""
    exit $exit_rc_nopermission
  fi
}



###############################################################################
#
# Function    : stop_Informix
#
# Description : Stops the Informix Dynamic Server
#
# Parameters  : None
#
###############################################################################

stop_Informix()
{
  check_env
  status_Informix
  if [[ $INFORMIX_STATUS != "$STOPPED" ]] ; then
    informix_return_string=`$stop_command`
  fi
  if (( $? != 0 ))
  then 
    advise 'stop'
  fi
  exit $exit_rc_success
}



###############################################################################
#
# Function    : restart_Informix
#
# Description : Restarts Informix Dynamic Server by
#               * Forces full checkpoint
#               * Stops IDS immediately
#               * Starts the Informix Dynamic Server
#
# Parameters  : None
#
###############################################################################

restart_Informix()
{
  check_env
  status_Informix
  if [[ $INFORMIX_STATUS = "$RUNNING" ]] ; then
    informix_return_string=`$stop_command`
    if (( $? != 0 ))
    then 
      advise 'restart'
    fi
    $start_command
  fi
  exit $exit_rc_success
}



###############################################################################
#
# Function    : main
#
# Description : Process command line args
#
# Parameters  : Command line arguments
#
###############################################################################

# Validate command line parameters
#
if [ $# -ne 1 ] ; then
  usage
  exit $exit_rc_usage
fi

# Process command line parameters
case $1 in
  --version) version_Informix ;;
  -version) version_Informix ;;
  -help) help_Informix ;;
  --help) help_Informix ;;
  -start) start_Informix ;;
  -stop) stop_Informix ;;
  -preparetostop) preparetostop_Informix ;;
  -restart) restart_Informix ;;
  -status) status ;;
  *) usage
esac
shift

exit $exit_rc_usage
