###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/DBColType/lib/Application.pm SysManagement_r4.79.0.9:cset.007747:4:4 08/24/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#		 ADC Software Systems UK
#			Spencer House
#			23 Sheen Road
#		  Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Software Systems UK, 2000-2003.
#
# Title:	    Application.pm      
# Author:	    Mark O'Connor    
#
# Description:      Alter table column types          
#
##############################################################################
=head1 Application Class

 DESCRIPTION:
   The purpose of the class is to contain the main application logic.

=cut

#---------------------------------------------------------------------------
#       PACKAGE NAME
#---------------------------------------------------------------------------
package Application;

#---------------------------------------------------------------------------
#   Core Perl Libraries
#---------------------------------------------------------------------------
use strict;
use FileHandle;
use Getopt::Long;

# Force output after every line (keeps stderr and stdout in sync)
autoflush STDOUT 1;
autoflush STDERR 1;

#---------------------------------------------------------------------------
#   Program Libraries
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PRIVATE GLOBAL CONSTANTS
#---------------------------------------------------------------------------
my $OK       = "OK";

#---------------------------------------------------------------------------
#       PUBLIC GLOBAL CONSTANTS
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PRIVATE GLOBAL VARIABLES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PUBLIC GLOBAL VARIABLES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PUBLIC SUBROUTINES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       CLASS METHOD SUBROUTINES
#---------------------------------------------------------------------------

#***************************************************************************

=head2 new 

 DESCRIPTION:
   This constructor parses the configuration file and initialises the 
   object data structures

 PARAMETERS:
   $class    - The name of the class (implicit parameter).

 RETURNS:
   The class object.

=cut

sub new
{
    my $class = shift;
    my $self = {};
    bless $self, $class;

    return $self;
}

#***************************************************************************

=head2 chomp_values 

 DESCRIPTION:
   Remove empty array entries

 PARAMETERS:
   $self - The class object (implicit parameter).
   @data - Data array

 RETURNS:
   Array with no empty data members

=cut

sub chomp_values
{
    my $self = shift;
    my (@data) = @_;

    return grep(!/^\s*$/, @data);
}

#***************************************************************************

=head2  execute

 DESCRIPTION:
   Use dbaccess to execute an SQL statement

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - Database name
   $sql      - SQL statement

 RETURNS:
   Lines returned as a list

=cut

sub execute
{
   my $self = shift;
   my ($database, $sql) = @_;
   my $command;

   $command = sprintf "/usr/bin/echo \"%s\" | \$INFORMIXDIR/bin/dbaccess %s 2>&1", $sql, $database;
   my @lines = `$command`;

   return chomp_values(@lines);
}

#***************************************************************************

=head2  getIndexColumnDetails

 DESCRIPTION:
   This method is used to translate the column id parameter that is stored
   in the index system catalog

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - Database name
   $table    - Table the column belongs to
   $colno    - Column number to translate

 RETURNS:
   Column name

=cut

sub getIndexColumnDetails
{
   my $self = shift;
   my ($database, $table, $colno) = @_;
   my $retValue = "";
  
   #
   # Find the column name based on the colid
   #
   my @lines = $self->execute($database, "SET LOCK MODE TO WAIT;SELECT col.colname FROM syscolumns col,systables tab WHERE col.tabid = tab.tabid AND tab.tabname = '$table' AND col.colno = abs($colno);");

   foreach my $line ( @lines ) {
      if ($line =~ /^colname\s+(.*)$/) {
         my $colname = $1;

         if ($colno > 0) {
            $retValue = $colname;
         }
         else {
            $retValue = "$colname DESC";
         }
      }
   }

   if ($retValue eq "") {
      die "Could not translate column number: $table, $colno";
   }

   return $retValue;
}

#***************************************************************************

=head2  getIndexDetails

 DESCRIPTION:
   This method is used to retrieve the index details for a given table

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - Database name
   $table     - SQL statement

 RETURNS:
  Hash references keyed by indexname
  1) Type of index
  2) Index columns
  3) Index dbspace

=cut

sub getIndexDetails
{
   my $self = shift;
   my ($database, $table) = @_;
   my $idxname;
   my @idxname;
   my %idxcolumns;
   my %idxdbspace;
   my %idxtype;

   #
   # Get the indexes associated with this table
   #
   my @lines = $self->execute($database, "SET LOCK MODE TO WAIT;SELECT idx.idxname, 'END', idx.indexkeys, idx.idxtype, frag.dbspace FROM sysindices idx, systables tab, OUTER sysfragments frag WHERE idx.tabid = tab.tabid AND idx.tabid = frag.tabid AND idx.idxname = frag.indexname AND frag.fragtype = 'I' AND tab.tabname = '$table';");

   #printf "DEBUG:\n%s", join("",@lines);

   foreach my $line ( @lines ) {
      # Necessary for ".." operator
      $_ = $line;

      if ($line =~ /^idxname\s+/ .. /^\(constant\)\s+/) {
         if ($line =~ /^idxname\s+(.*)$/) {
            @idxname = ($1);
         }
         elsif ($line =~ /^\s+(.*)$/) {
            push @idxname, $1;
         }
         elsif ($line =~ /^\(constant\)\s+/) {
            $idxname = join("",@idxname);
         }
      }
      elsif ($line =~ /^indexkeys\s+(.*)$/) {
	 #
	 # Index keys have the following format:
	 #
	 # 1 [1], -2 [1]
	 # ^-------^-------- These are the column numbers
	 # Note: A negative value indicates that this is a descending index
	 #
         my $indexKeys = $1;
	 my @columns = ();

	 foreach my $key ( split(",", $indexKeys) ) {
	    if ($key =~ /^\s*(\S+)\s+\[/) {
               my $colno = $1;
	       push @columns, $self->getIndexColumnDetails($database, $table, $colno); 
	    }
	    else {
               die "Index key has unexpected format: $indexKeys";
	    }
	 }

         $idxcolumns{$idxname} = \@columns;
      } 
      elsif ($line =~ /^idxtype\s+(.*)$/) {
         if ($1 eq "U") {
            $idxtype{$idxname} = "UNIQUE";
         }
         else {
            $idxtype{$idxname} = "";
         }
      }
      elsif ($line =~ /^dbspace\s+(.*)$/) {
         $idxdbspace{$idxname} = $1;
      }
   }

   return (\%idxtype, \%idxcolumns, \%idxdbspace);
}

#***************************************************************************

=head2  getIndexSQL

 DESCRIPTION:
   This method is used to recover the information regarding all table
   indexes involving this column.
   2 list references are returned. One list contains SQL statements to
   drop the indexes, the second contains SQL statements to create the
   indexes.

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - database name
   $table    - Table name
   $column   - column name

 RETURNS:
   1) Array reference to "DROP INDEX" SQL statements
   2) Array reference to "CREATE INDEX" SQL statements

=cut

sub getIndexSQL
{
   my $self = shift;
   my ($database, $table, $column) = @_;
   my %indexData;
   my @drop = ();
   my @create = ();

   #
   # Get the indexes associated with this table
   #
   my ($idxtype, $idxcolumns, $idxdbspace) = $self->getIndexDetails($database, $table);

   foreach my $index ( keys %{$idxcolumns} ) {
      if (grep(/^$column\s*/, @{$idxcolumns->{$index}}) > 0) {
         push @drop,   "SET LOCK MODE TO WAIT;";
         push @drop,   sprintf "DROP INDEX %s;", $index;

         push @create, "SET PDQPRIORITY 80;";
         push @create, sprintf "CREATE %s INDEX %s ON %s (%s) IN %s;", $idxtype->{$index}, $index, $table, join(",",@{$idxcolumns->{$index}}), $idxdbspace->{$index};
      }
   }

   return (\@drop, \@create);
}

#***************************************************************************

=head2  tableExists

 DESCRIPTION:
  Ensure that the table exists

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - database name
   $table    - Table name

 RETURNS:
   1 or 0

=cut

sub tableExists
{
   my $self = shift;
   my ($database, $table) = @_;
   my $returnVal = 0;

   my @lines = $self->execute($database, "SET LOCK MODE TO WAIT;SELECT tab.tabname FROM systables tab WHERE tab.tabtype = 'T' AND tab.tabname = '$table'");

   foreach my $line ( @lines ) {
      if ($line =~ /^tabname\s+/) {
         $returnVal = 1;
      }
   }

   return $returnVal;
}
#***************************************************************************

=head2  columnExists

 DESCRIPTION:
   Ensure that a column exists 

    PARAMETERS:
       $database - database name
       $table    - Table name
       $column   - Column name to check


    RETURNS:
       1 or 0

=cut

sub columnExists
{
 my $self = shift;
 my ($database, $table, $column) = @_;
 my $returnVal = 0;

 my @lines =  $self->execute($database, "SET LOCK MODE TO WAIT; select colname from syscolumns where tabid in (select tabid from systables where tabname='$table') and colname = '$column'");

 foreach my $line ( @lines ) {
  if ($line =~ /^colname\s+/) {
     $returnVal = 1;
     last;
   }
 }

 return $returnVal;
}


#***************************************************************************
=head2  tableColumnType

 DESCRIPTION:
   Returns the type of a specified table column
   Note there is only logic to spot "integer" and opaque type names 
   like "utime".

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - database name
   $table    - Table name
   $column   - column name

 RETURNS:
   Column type

=cut

sub tableColumnType
{
   my $self = shift;
   my ($database, $table, $column) = @_;

   my @lines = $self->execute($database, "SET LOCK MODE TO WAIT;SELECT col.coltype, xtd.name FROM syscolumns col,systables tab,OUTER sysxtdtypes xtd WHERE col.tabid = tab.tabid AND col.extended_id = xtd.extended_id AND tab.tabtype = 'T' AND tab.tabname = '$table' AND col.colname = '$column'");
   my $typeid = "notExist";
   my $xtdType;

   foreach my $line ( @lines ) {
      if ($line =~ /^coltype\s+(.*)$/) {
         $typeid = $1;
      }
      elsif ($line =~ /^name\s+(.*)$/) {
         $xtdType = $1;
      }
   }

   #
   # If the typeid is >= 256 then "not null" condition is set
   #
   if ($typeid >= 256) {
      $typeid -= 256;
   }

   #
   # Translate the typeid
   #
   if ($xtdType eq "utime") {
      return "utime";
   }
   elsif ($typeid eq 2) {
      return "integer";
   }
   else {
      return "typeid = $typeid";
   }
}

#***************************************************************************

=head2  main

 DESCRIPTION:
   Main routine for the application.

 PARAMETERS:
   $self - The class object (implicit parameter).
   $ref  - A Reference to a hash of parameters

 RETURNS:
   $OK

=cut

sub main
{
   my $self = shift;
   my ($ref) = @_;
   my $database = $ref->{database};
   my $logFile  = $ref->{log_file};
   my @columns  = @{$ref->{column}};
   my $fromType = $ref->{from_type};
   my $toType   = $ref->{to_type};
   my @tables   = @{$ref->{table}};

   #
   # Open the log file
   #
   my $log = new FileHandle($logFile, "w") 
	   or die "Cannot open logfile: $logFile";
   $log->print("Starting Thread:\n");                                         
   printf("Starting Thread $logFile:\n");                                         

   #
   # Process each table
   #
   foreach my $table ( @tables ) {
     
      if (!$self->tableExists($database, $table)) {
         $log->printf("ERROR: Table does not exist : %s\n", $table);
         printf("ERROR: Table does not exist : %s\n", $table);
      }
      else {
         foreach my $column ( @columns ) {

            my $type = $self->tableColumnType($database, $table, $column);

            if ($type eq $fromType) {

               $log->print("---------------------------------------\n");
               $log->printf("Altering Table: %s\nColumn: %s\n", $table, $column);
               $log->print("---------------------------------------\n");
               print("---------------------------------------\n");
               printf("Altering Table: %s\nColumn: %s\n", $table, $column);
               print("---------------------------------------\n");

	       #
	       # Fetch the Index details
	       #
               my ($dropRef, $createRef) = $self->getIndexSQL($database, $table, $column);

	       #
	       # Construct a list of upgrade SQL
	       #
	       my @sql;
	       push @sql, @{$dropRef};
	       push @sql, "SET LOCK MODE TO WAIT;";
	       push @sql, sprintf "ALTER TABLE %s MODIFY %s %s not null;", $table, $column, $toType;
	       push @sql, @{$createRef};

	       my $sqlStr = join("\n",@sql);

	       #
	       # Log message
	       #
	       $log->print("Upgrade SQL:\n");
	       $log->printf("%s\n", $sqlStr);
	       #print("Upgrade SQL:\n");
	       #printf("%s\n", $sqlStr);

	       #
	       # Execute a log the upgrade SQL
	       #
	       $log->print("\nLog:\n");
	       #print("\nLog:\n");

	       foreach my $line ( $self->execute($database, $sqlStr) ) {
                  $log->printf("%s\n", $line);
                  #printf("%s\n", $line);
	       }
            }    
            elsif ($type eq "typeid = notExist") {
            }
            else {
               $log->printf("ERROR: Table column not of expected type: table(%s) column(%s) type(%s)\n", $table, $column, $type);
               #printf("ERROR: Table column not of expected type: table(%s) column(%s) type(%s)\n", $table, $column, $type);
            }
         }
      }
   }

   #
   # Cleanup
   #
   $log->close;
   printf("Finished Thread - Check $logFile for Errors or Warnings :\n");                                         
   printf("Please Wait for Final Completion Message ....... :\n");                                         

   return $OK;
}

#---------------------------------------------------------------------------
#       PRIVATE SUBROUTINES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PACKAGE INITIALISATION
#---------------------------------------------------------------------------
1;


#
#       End (of template.pm)
#
