#!/usr/bin/perl
############################################################
#
# File: gw_healthcheck.pl
#
# Description: Monitors the file statistics produced by a
# producized gateway. Logs informational messages and
# exceptions based on thresholds in the log file passed on
# the command line. This file can then be used to monitor
# the status of the Gateway.
#
# Author: Ollie Fagan
#
# Version: @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_gateways.pl SysManagement_r4.79.0.9:cset.005253:3:3 01/18/05 @(#)
#
############################################################
use Time::localtime;
use Getopt::Long;

# version info from TC
my $version = "@(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_gateways.pl SysManagement_r4.79.0.9:cset.005253:3:3 01/18/05 @(#)";

# operations map from those in config file to the real
# operation
my %OP_MAP =
    (
     'GT' => sub {return $_[0] > $_[1]},
     'LT' => sub {return $_[0] < $_[1]},
     'EQ' => sub {return $_[0] == $_[1]},
     'NE' => sub {return $_[0] != $_[1]},
     );

# text strings to appear in log file for report
my %OP_TXT_MAP =
    (
     'GT' => "greater than",
     'LT' => "less than",
     'EQ' => "equals",
     'NE' => "not equal to",
     );

# counters to be outputted as informational messages
# for monitoring the status of a normal running 
# Gateway. 
my %INFO_COUNTERS = 
    (
     numberLIFFiles => 1,
     numberFilesParsedOK => 1,
     rawFileSizeTotal => 1,
     lifFileSizeTotal => 1);

my %gateways = ();

$Getopt::Long::ignorecase=1;
$Getopt::Long::autoabbrev=1;

my ($opt_l, $opt_c, $opt_v, $opt_h, $opt_d);

# handle both long and short options from the command line
my $result = GetOptions
    ("l=s" => \$opt_l,
     "c=s" => \$opt_c,
     "v!" => \$opt_v,
     "version!" => \$opt_v,
     "h!" => \$opt_h,
     "help!" => \$opt_h,
     "debug!" => \$opt_d);

unless (($opt_l && $opt_c) || $opt_v || $opt_h ){
    die usage();
}

if ($opt_h){
    print_help();
    exit;
}

if ($opt_v){
    print_version();
    exit;
}

# The main body of work, check the status of each of the
# Gateways.
check_status();


############################################################
#
# sub: check_status
#
# desc: The main functionality of the script, this function
# loads the configuration and then performs the status checks
# on each of the Gateways in the configuration file.
#
############################################################
sub check_status{

    # load the configuration
    process_config();

    # open the log file
    unless (open(LOG_FILE,"> $opt_l")){
        die "Could not open log file $opt_l for writing\n";
    }
    
    # process each of the gateways
    foreach (keys %gateways){
        debug("Processing gateway $_");
        process_gateway($_);
    }

    close(LOG_FILE);
}


############################################################
#
# sub: process_gateway
#
# desc: Checks the threshold and status of an individual
# Gateway. This looks at the required thresholds defined in
# the configuration and raises exceptions if thresholds have 
# been passed.
#
############################################################
sub process_gateway{
    my ($gateway) = @_;
    my $sdir = $gateways{$gateway}->{FILE_STATS_DIR};
    unless (opendir(DIR,$sdir )){
        die "Gateway $gateway - cannot open directory $sdir\n";
    }
    my @files = grep /.lif$/,  readdir(DIR);
    close DIR;
    unless (@files){
        log_message("INFORMATION","Gateway: $gateway, no file statistics found");
        # no need to keep going
        return;
    }

    # sort the files to get the newest
    my @sfiles = sort {-M $sdir . "/" . $a <=> -M $sdir . "/" . $b} @files;
    
    # got the youngest file get the stats
    debug("Got LIF file $sfiles[0], checking thresholds");
    unless (open(SF,"< $sdir/$sfiles[0]")){
        die "Gateway $gateway - Could not open stats file $sfiles[0]";
    }

    while (<SF>){
        next if ($_ =~ /^\#/);
        # trim leading spaces
        $_ =~ s/^(\s+)//g;
        my @els = split(/\s+/,$_);
        debug("values " . join(",", @els));

        # check list of informational counters
        if (exists($INFO_COUNTERS{$els[0]})){
            log_message("INFORMATION","Gateway: $gateway statistic $els[0] " .
                        "value $els[1]");
        }

        # check if it's one of the monitored entries
        next unless(exists($gateways{$gateway}->{THRESHOLD}->{$els[0]}));

        my $he = $gateways{$gateway}->{THRESHOLD}->{$els[0]};
        # now check the threshold
        if ($OP_MAP{$he->{OP}}($els[1],$he->{VALUE})){
            log_message("EXCEPTION","Gateway: $gateway statistic $els[0] " . 
                        "value $els[1] $OP_TXT_MAP{$he->{OP}} checked value " .
                        "$he->{VALUE}");
            debug("Gateway $gateway - passed the threshold for $els[0]");
        }
        else {
            debug("Threshold okay for $el[0] -> $el[1] $he->{VALUE}");
        }
        
    }
}

############################################################
#
# sub: process_config
#
# desc: Loads and validates the configuration file
#
############################################################
sub process_config{
# open the configuration file to see what thresholds we're monitoring
    open(CF,"< $opt_c") || die "Could not open configuration file $opt_c\n";
    
    my $line = 0;
    my @lines = <CF>;

# process the configuration lines in the file
  GATEWAY:
    while ($line < @lines){
        $_ = $lines[$line];
        $line++;
        next if ($_ =~ /^\#/ || $_ =~ /^$/);

        unless ($_ =~ /^GATEWAY/){
            die "Each section in configuration file must be associated with a GATEWAY\n";
        }

        # get the current gateway
        my $gateway = (split(",",$_))[1];
        chomp($gateway);
        debug("Current gateway is $gateway");
        
        $gateways{$gateway} = ();
        do {
            $line++;
            $_ = $lines[$line];
            debug("Processing line $_");
            
            next if ($_ =~ /^\#/ || $_ =~ /^$/);
            if ($_ =~ /^THRESHOLD/){ # process a requested threshold check
                my @els = split(",",$_);
                debug("Adding threshold $els[1]");
                
                unless (exists($OP_MAP{$els[2]})){
                    die "Configuration line $line Operation $els[2] not supported\n";
                }
                unless ($els[3] =~ /^\d+/){
                    die "Configuration line $line Value checked for " . 
                        "threshold must be all digits\n";
                }
                my %te = ();
                $te{COUNTER} = $els[1];
                $te{OP} = $els[2];
                $te{VALUE} = $els[3];

                # save the threshold
                $gateways{$gateway}->{THRESHOLD}->{$els[1]} = \%te; 
                debug("counter $els[1] op $els[2] value $els[3] ");
            }
            elsif ($_ =~ /FILE_STATS_DIR/) { # the file statistics directory
                my $dir = (split(",",$_))[1];
                chomp($dir);
                debug("Directory to check file stats from is $dir");
                unless (-e $dir && -d $dir){
                    die "Configuration line $line directory $dir does not exist " . 
                        "or is not a directory\n";
                }
                $gateways{$gateway}->{FILE_STATS_DIR} = $dir;
            }
            elsif ($_ =~ /^GATEWAY/){
                debug("Going to the next Gateway");
                next GATEWAY;
            }
            else {
                die "Line $line Unrecognised entry $_ in configuration\n";
            }
        } while ($line < @lines);
        
    }
    
}

############################################################
#
# sub: log_message
#
# desc: Logs a message to the script monitoring log file.
# Two types of messages are possible:
#    - Informational which give regular updates on the status
#    - Exception which indicate threshold violations according
#       to the configuration
#
############################################################
sub log_message{
    my ($type,$message) = @_;
    chomp($message);
    print LOG_FILE "$type, $message\n";
}

############################################################
#
# sub: debug
#
# desc: Prints optional debugging information if the -d option
# is specified
#
############################################################
sub debug{
    if ($opt_d){
        print $_[0] . "\n";
    }
}

############################################################
#
# sub: usage
#
# desc: Prints the usage information 
#
############################################################
sub usage{
    return "usage is metrica_check_gateways.pl " . 
        "(-l <log file> -c <config file>|-h|-v) -d\n";
}

############################################################
#
# sub; print_help
#
# desc: Prints out the help information
#
############################################################
sub print_help {
    my @HELP_MESSAGE = 
        (
         "metrica_check_gateways.pl is a healthcheck program for the checking of the ",
         "status of multiple Gateways. It parses the latest file statistics ", 
         "for each Gateway and reports threshold violations and informational ", 
         "messages in the log file specified with the -l option. A configuration ",
         "file must be specified which defines each Gateway to be monitored ",
         "and the thresholds associated with them. This script can be used in ",
         "conjunction with general Performance Manager monitoring.",
         "See associated README.metrica_check_gateways for more information. " 
         );
    
    print "\n" . join ("\n",@HELP_MESSAGE,"","") ;
    print usage();
}

############################################################
#
# sub: print_version
#
# desc: Prints out the version information, which must be
# setup by setting "keyword expansion on" in Truechange.
############################################################
sub print_version {
    print "metrica_check_gateways.pl version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_gateways.pl SysManagement_r4.79.0.9:cset.005253:3:3 01/18/05 @(#)\n";
}
