#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_table_report.ksh SysManagement_r4.79.0.9:cset.008519:5:5 11/09/07 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005-2007.
#
# Title:        metrica_table_report.ksh
# Authors:      Barry Deignan, Stuart Brooks  
# Description:  Script to:                 
#                 Determine Table Sizes, Number of extents, Number of Rows,
#                 Number of days, MBs per Day                       
#
###############################################################################

PROGNAME=`basename $0`
DATEXT=`date +%d-%b-%Y:%H:%M'`

# Set up common variables
#
. metrica_install_common.ksh

[[ -z "$INFORMIXDIR" ]] && $ECHO "INFORMIXDIR not defined" && exit 1
[[ -z "$INFORMIXSERVER" ]] && $ECHO "INFORMIXSERVER not defined" && exit 1
[[ -z "$METRICA_DIR" ]] && $ECHO "METRICA_DIR not defined" && exit 1
[[ -z "$METRICA_PERFORMANCE_DB" ]] && $ECHO "METRICA_PERFORMANCE_DB not defined" && exit 1


WORKOUTFILE=/tmp/sql$$.sql
WORKTABLEFILE=/tmp/tabs$$.sql
WORKFILE=/tmp/work$$.txt
WORKFILE1=/tmp/work1$$.txt
ONCHECKFILE=/tmp/oncheck$$.txt
REPORTFILE=$METRICA_DIR/logs/table_report.txt
OUTFILE=$METRICA_DIR/logs/table_report.csv

rm -f $WORKOUTFILE $WORKTABLEFILE $WORKFILE $WORKFILE1 $ONCHECKFILE 1>/dev/null 2>&1
touch $WORKOUTFILE $WORKTABLEFILE $WORKFILE 1>/dev/null 2>&1

#
# Print file description and copyright information
#
show_info()
{
    cat <<END

Metrica Performance Manager Table Report.
Copyright  2005-2007 Vallent Software Systems

END
}


#
# Show correct command-line usage
#
usage()
{
show_info
	cat <<END

  Usage: $PROGNAME

        options:    [ -h | -help | --help ]
                  | [ -pagecheck ]

  This script produces a report for a list of performance db tables.
  Report contains:
  - Size (KBs) allocated to table by Informix
  - Size (KBs) used for table data        
  - Number of database extents allocated to table by Informix
  - Number of rows in table
  - Number of days data in table
  - KBs loaded per day for a raw table     

  OPTIONS:
  -pagecheck Performs a more comprehensive check using the informix oncheck utility, which
             checks the number and types of table pages in use.

             This option gives a more accurate figure for the size used for table data but
             takes longer to run. Therefore, this option should not be used when running the
             table report for a large number of tables (for example, all raw tables) because
             it could take many hours to finish. Instead, it should be used to check the size
             of one table or a few tables which may potentially need resizing.

END
}

#
# Ask for Input
#

GetResponse()
{
	DefaultResponse=$2
	$ECHO -n "$1 [$DefaultResponse]: "
	read Response
	[[ -z "$Response" ]] && Response=$DefaultResponse
}



#
# Ask for SQL matching pattern for list of tables. % is wildcard. _ is 
# single character wildcard.
get_inputs()
{
	cat <<END
Please enter an SQL expression matching the list of tables you would
like to report on. The wildcard characters are "_" to match one character and
"%" to match one or more characters.

Example expressions: %raw%, nc%, ericsson%.

END
	GetResponse "Please enter SQL matching pattern" "%raw"
	SQLPattern=$Response
	DATECol="day"

}


#
# Capture Table names from input pattern
#
gen_table_size_list()
{

if [ "$PAGECHECK" -eq "0" ]; then

dbaccess $METRICA_PERFORMANCE_DB <<EOF >> /dev/null 2>&1
unload to "$WORKTABLEFILE" delimiter "|"	
select  a.tabname,
trunc(b.nptotal*2, 2),
trunc(b.npdata*2, 2),
b.nextns
from systables a, sysmaster:sysptnhdr b
where a.partnum = b.partnum
and a.tabname like "$SQLPattern"
order by a.tabname;
EOF

else

dbaccess $METRICA_PERFORMANCE_DB <<EOF >> /dev/null 2>&1
unload to "$WORKTABLEFILE" delimiter "|"	
select tabname
from systables
where tabname like "$SQLPattern"
and tabtype='T'
and tabid > 99
order by tabname; 
EOF

fi

TAB_CNT=`wc $WORKTABLEFILE | awk '{print $1}'`

}


#
# Produce Report
# 
run_report()
{
	COUNTER=0
	for i in `cat $WORKTABLEFILE`
	do
		TABLE=`echo $i | awk '{FS="|";print $1}'`
		COUNTER=`expr $COUNTER + 1 `
		clear
		echo ""
		echo "Metrica Table Report"                                 
		echo "--------------------"                                 
		echo ""
		echo $TAB_CNT" Table(s) matching the pattern "$SQLPattern" selected for report"
		echo ""
		echo "Processing Table "$COUNTER" - "$TABLE                                                  

		if [ "$PAGECHECK" -eq "1" ]; then
			oncheck -pT $METRICA_PERFORMANCE_DB:$TABLE > $ONCHECKFILE
			ALLOC_PAGES=`grep allocated $ONCHECKFILE | head -1 | awk '{print $5}'`
			FREE_PAGES=`grep Free $ONCHECKFILE | head -1 | awk '{print $2}'`
			NEXTENTS=`grep extents $ONCHECKFILE | head -1 | awk '{print $4}'`
			NROWS=`grep rows $ONCHECKFILE | head -1 | awk '{print $4}'`
			NALLOC=`expr $ALLOC_PAGES \* 2`
			NUSED=`expr $NALLOC - $FREE_PAGES \* 2`
		fi

		DAY_TYPE=`echo $TABLE | grep -c _day`
		if [ "$DAY_TYPE" -ge 1 ]; then
			DATECol="day"
		fi

		RAW_TYPE=`echo $TABLE | grep -c _raw`
		if [ "$RAW_TYPE" -ge 1 ]; then
			DATECol="day"
		fi

		WEEK_TYPE=`echo $TABLE | grep -c _week`
		if [ "$WEEK_TYPE" -ge 1 ]; then
			DATECol="week_of"
		fi

		MONTH_TYPE=`echo $TABLE | grep -c _month`
		if [ "$MONTH_TYPE" -ge 1 ]; then
			DATECol="month_of"
		fi

		NC_TYPE=`echo $TABLE | grep -c nc_`
		if [ "$NC_TYPE" -ge 1 ]; then
			DATECol="timestamp"
		fi

		if [ "$PAGECHECK" -eq "1" ]; then
			dbaccess $METRICA_PERFORMANCE_DB <<EOF >> /dev/null 2>&1 
unload to "$WORKFILE1" delimiter " "
select count(distinct $DATECol) from $TABLE;
EOF

                	if [ -f $WORKFILE1 ]; then
	                        NDAYS=`cat $WORKFILE1  | sed s/" "//g`
	                else
	                        NDAYS=0
	                fi
			rm $WORKFILE $WORKFILE1 > /dev/null 2>&1

			echo ${i}$NALLOC"|"$NUSED"|"$NEXTENTS"|"$NROWS"|"$NDAYS >> $WORKOUTFILE

		else
                	dbaccess $METRICA_PERFORMANCE_DB <<EOF >> /dev/null 2>&1
unload to "$WORKFILE" delimiter " "
select count(*) from $TABLE;
unload to "$WORKFILE1" delimiter " "
select count(distinct $DATECol) from $TABLE;
EOF
			# If table count is otherwise than 0, add count to output
			COUNT=`cat $WORKFILE | sed s/" "//g`

                        if [ -f $WORKFILE1 ]; then
                                NDAYS=`cat $WORKFILE1  | sed s/" "//g`
                        else
                                NDAYS=0
                        fi
                        rm $WORKFILE $WORKFILE1 > /dev/null 2>&1

                        echo $i$COUNT"|"$NDAYS >> $WORKOUTFILE
		fi
	done
}

#
# Print Report
# 
print_report()
{
	echo ""
	echo "Processing Finished - Formatting Report ......"       
    cat <<END > $REPORTFILE

Metrica Table Report for Table Names like $SQLPattern
---------------------------------------------------------------------------------------------------

END
	echo "Table Name,Allocated Size (KBs),Used Size (KBs),KBs Per Day,Extents,Rows,Days" > $OUTFILE   
	report_header
	totmbday=0
	totalloc=0
	totused=0
	tabcount=0
	tabpage=0
	pagecount=0
	for i in `cat $WORKOUTFILE`
	do

		tabnm=`echo $i | awk '{FS="|";print $1}'`
		if [ "$PAGECHECK" -eq "1" ]; then
			allocated=`echo $i | awk '{FS="|";print $2}'`
			used=`echo $i | awk '{FS="|";print $3}'`
			exts=`echo $i | awk '{FS="|";print $4}'`
			rows=`echo $i | awk '{FS="|";print $5}'`
		else
			allocated=`echo $i | awk '{FS="|";print $2}' | awk '{FS=".";print $1}'`
			used=`echo $i | awk '{FS="|";print $3}' | awk '{FS=".";print $1}'`
			exts=`echo $i | awk '{FS="|";print $4}'`
			rows=`echo $i | awk '{FS="|";print $5}' | awk '{FS=".";print $1}'`
		fi
		days=`echo $i | awk '{FS="|";print $6}' | awk '{FS=".";print $1}'`
		if [ "$days" = "" -o "$days" -eq "0" ]; then
			days=0
		fi

		totalloc=`expr $totalloc + $allocated `
		totused=`expr $totused + $used `

		RAW_TYPE=`echo $tabnm | grep -c raw`
		if [ "$RAW_TYPE" -gt 0 -a "$days" -gt 0 -a "$used" -gt 0 ]; then
			mbday=`expr $used / $days`
		else
			mbday=0
		fi
			totmbday=`expr $totmbday + $mbday `
		echo $tabnm","$allocated","$used","$mbday","$exts","$rows","$days >> $OUTFILE


		printf "%-55s\t%9s\t%9s\t%9s\t%7s\t%7s\t%7s\n" $tabnm $allocated $used $mbday $exts $rows $days >> $REPORTFILE

		tabcount=`expr $tabcount + 1 `
		tabpage=`expr $tabpage + 1 `

		if [ "$tabpage" -eq 50 -a "$tabcount" -lt "$TAB_CNT" ]; then
			pagecount=`expr $pagecount + 1 `
			echo " " >> $REPORTFILE
			printf "%40s\t%-4s\n" "Page" $pagecount >> $REPORTFILE 
			echo " " >> $REPORTFILE
			report_header
			tabpage=0
		fi


	done 

	totmbday=`expr $totmbday / 1000 `
	totalloc=`expr $totalloc / 1000 `
	totused=`expr $totused / 1000 `

	report_total

}



report_header()
{
	
	printf "%-55s\t%-9s\t%-9s\t%-9s\t%-7s\t%-7s\t%-7s\n" " " "Allocated" "Used" "KBs" "Number" "Number" "Number" >> $REPORTFILE
	printf "%-55s\t%-9s\t%-9s\t%-9s\t%-7s\t%-7s\t%-7s\n" " " "Space" "Space" "per" "of" "of"  "of" >> $REPORTFILE
	printf "%-55s\t%-9s\t%-9s\t%-9s\t%-7s\t%-7s\t%-7s\n" "Table Name " "KBs " "KBs " "Day " "Exts" "Rows" "Days" >> $REPORTFILE
	printf "%-55s\t%-9s\t%-9s\t%-9s\t%-7s\t%-7s\t%-7s\n" "------------" "---------" "---------" "---------" "-------" "-------" "-------" >> $REPORTFILE

}



report_total()
{
	

	echo ""    >> $REPORTFILE                              
	echo ""    >> $REPORTFILE                              
	printf "%55s\t%-9s\t%-9s\t%-9s\n" " " "Allocated" "Used" "MBs" >> $REPORTFILE
	printf "%55s\t%-9s\t%-9s\t%-9s\n" " " "Space" "Space" "per" >> $REPORTFILE
	printf "%-55s\t%-9s\t%-9s\t%-9s\n" "Totals: " "MBs " "MBs " "Day " >> $REPORTFILE
	printf "%-55s\t%-9s\t%-9s\t%-9s\n" "-------" "---------" "---------" "---------" >> $REPORTFILE
	printf "%55s\t%9s\t%9s\t%9s\n" " " $totalloc $totused $totmbday >> $REPORTFILE
	printf "%55s\t%-9s\t%-9s\t%-9s\n" " " "=========" "=========" "=========" >> $REPORTFILE
	echo ""    >> $REPORTFILE                              
	echo ""    >> $REPORTFILE                              
	echo "Note : MBs Per Day is only calculated for raw data tables"    >> $REPORTFILE                              



	echo "\nEnd of Report.\n" >> $REPORTFILE

	cat $REPORTFILE
}



#
#	Remove temp files, report file name and exit
#
clean_up()
{
	rm -f $WORKOUTFILE $WORKTABLEFILE $WORKFILE $WORKFILE1 $ONCHECKFILE 1>/dev/null 2>&1
	echo "Above report now saved in $REPORTFILE"
	echo "Comma Separated file saved in $OUTFILE"
}


main()
{
	get_inputs;
	gen_table_size_list;
	run_report;
	print_report;
	clean_up;
}

#Preserve Input Parameters
I1=$1
I2=$2
I3=$3
I4=$4
I5=$5
I6=$6
case $1 in
# No parameters specified
	"")
		PAGECHECK=0
		main
		exit 0
	;;
# Comprehensive check
	"-pagecheck")
		PAGECHECK=1
		main
		exit 0
	;;
# Help options
	"-h" | "-help" | "--help")
		usage
		exit 0
	;;

# Any other option
	*)
		usage
		exit 1
	;;
esac
